 /* +---------------------------------------------------------------- */
 /*                                                                   */
 /*  Microsoft Windows                                                */
 /*  Copyright (C) Microsoft Corporation, 1992-1999.                  */
 /*                                                                   */
 /*  File:       sspi.cpy                                             */
 /*                                                                   */
 /*  Contents:   Security Support Provider Interface                  */
 /*              Prototypes and structure definitions                 */
 /*                                                                   */
 /*  Functions:  Security Support Provider API                        */
 /*                                                                   */
 /*  History:    11-24-93   RichardW   Created                        */
 /*                                                                   */
 /* ----------------------------------------------------------------- */

 %dcl #_SSPI_H__ char ext;
 %if #_SSPI_H__ = '' %then %do;
 %#_SSPI_H__ = 'Y';

 /* Determine environment: */

 %dcl UNICODE char ext;

 %dcl #NTSUBAUTH_ char ext;
 %if #NTSUBAUTH_ = '' %then %do;
 %include subauth;
 %end;

 %dcl WIN32_CHICAGO char ext;
 %dcl #NTDEF_       char ext;
 %dcl #WINNT_       char ext;


 %dcl ISSP_LEVEL fixed;
 %dcl ISSP_MODE  fixed;

 %dcl SECURITY_WIN32 char ext;
 %if SECURITY_WIN32 ^= '' %then  %do;
 %ISSP_LEVEL = 32;
 %ISSP_MODE = 0;
 %end; /* SECURITY_WIN32 */

 %dcl SECURITY_KERNEL char ext;
 %if SECURITY_KERNEL ^= '' %then %do;
 %ISSP_LEVEL = 32;

 /* SECURITY_KERNEL trumps SECURITY_WIN32. */
 %ISSP_MODE = 0;
 %end; /* SECURITY_KERNEL */


 %if ISSP_LEVEL = 0 %then %do;
   %note ('You must define one of SECURITY_WIN32 or SECURITY_KERNEL',8);
 %end; /* !ISSP_LEVEL */


 /* Now, define platform specific mappings: */

 /* For NT-2 and up, wtypes will define HRESULT to be long. */

 %dcl SEC_WCHAR char;  %SEC_WCHAR = 'WIDECHAR';
 %dcl SEC_CHAR  char;  %SEC_CHAR  = 'char';
 %dcl SEC_TCHAR char;
 %if UNICODE ^= '' %then %do;
 %SEC_TCHAR = SEC_WCHAR;
 %end; %else %do;
 %SEC_TCHAR = SEC_CHAR;
 %end; /* UNICODE */

 define alias SECURITY_STATUS type LONG;

 /* SEC_TEXT non needed in PL/I */
 %dcl SEC_ENTRY char ext;
 %SEC_ENTRY='options (byvalue linkage(stdcall) nodescriptor)';


 %dcl SECURITY_PSTR  char;
 %dcl SECURITY_PCSTR char;
 %if UNICODE ^= '' %then %do;
 %SECURITY_PSTR  = SEC_WCHAR !! '(*) varz byaddr';
 %SECURITY_PCSTR = SEC_WCHAR !! '(*) varz byaddr nonasgn';
 %end; %else %do;
 %SECURITY_PSTR  = SEC_CHAR  !! '(*) varz byaddr';
 %SECURITY_PCSTR = SEC_CHAR  !! '(*) varz byaddr nonasgn';
 %end; /* UNICODE */


 /* Equivalent string for rpcrt: */


 /* Okay, security specific types: */


 %dcl #_SECHANDLE_DEFINED__ char ext;
 %if #_SECHANDLE_DEFINED__ = '' %then %do;
 define structure
   1 SecHandle,
     2 dwLower type ULONG_PTR,
     2 dwUpper type ULONG_PTR;
 define alias PSecHandle handle SecHandle;

 %#_SECHANDLE_DEFINED__ = 'Y';
 %end; /* __SECHANDLE_DEFINED__ */

 SecInvalidateHandle: proc (x)
                      options (inline reorder) internal;
   dcl x type PSecHandle;
   dcl (bin, iunsigned) builtin;
   x => dwLower = iunsigned(bin(-1,31));
   x => dwUpper = iunsigned(bin(-1,31));
 end SecInvalidateHandle;

 SecIsValidHandle: proc (x) returns (bit (1) aligned)
                   options (inline reorder) internal;
   dcl x type PSecHandle;
   dcl (bin, iunsigned) builtin;
   return (x => dwLower ^= iunsigned(bin(-1,31)) &
           x => dwUpper ^= iunsigned(bin(-1,31)));
 end SecIsValidHandle;

 define alias  CredHandle type  SecHandle;
 define alias PCredHandle type PSecHandle;

 define alias  CtxtHandle type  SecHandle;
 define alias PCtxtHandle type PSecHandle;


 %if WIN32_CHICAGO ^= '' %then %do;

 define alias QWORD fixed bin (64) unsigned;
 define alias  SECURITY_INTEGER type QWORD;
 define alias PSECURITY_INTEGER pointer;

 SEC_SUCCESS: proc (Status) returns (bit (1) aligned)
              options (inline reorder) internal;
   dcl Status type SECURITY_INTEGER;
   return (Status >= 0);
 end SEC_SUCCESS;

 %end; %else %if #NTDEF_ ^= '' | #WINNT_ ^= '' %then %do;

 define alias  SECURITY_INTEGER type LARGE_INTEGER;
 define alias PSECURITY_INTEGER pointer;

 %end; %else %do;

 /* no need to use "define structure 1 SECURITY_INTEGER ..." in PL/I */

 define alias  SECURITY_INTEGER fixed bin (64) unsigned;
 define alias PSECURITY_INTEGER pointer;

 %end; /* _NTDEF_ || _WINNT_ */


 define alias  TimeStamp type SECURITY_INTEGER;
 define alias PTimeStamp pointer;


 /* If we are in 32 bit mode, define the SECURITY_STRING structure, */
 /* as a clone of the base UNICODE_STRING structure.  This is used  */
 /* internally in security components, an as the string interface   */
 /* for kernel components (e.g. FSPs)                               */

 %if #NTDEF_ = '' %then %do;
 define structure
   1 SECURITY_STRING,
     2 Length        fixed bin(16) unsigned,
     2 MaximumLength fixed bin(16) unsigned,
     2 Buffer        pointer;
 define alias PSECURITY_STRING handle SECURITY_STRING;
 %end; %else %do;
 define alias  SECURITY_STRING   type UNICODE_STRING;
 define alias PSECURITY_STRING handle UNICODE_STRING;
 %end; /* _NTDEF_ */


 /* SecPkgInfo structure */
 /*  Provides general information about a security provider */

 define structure
   1 SecPkgInfoW,
     2 fCapabilities fixed bin(32) unsigned,/* Capability bitmask */
     2 wVersion      fixed bin(16) unsigned,/* Version of driver */
     2 wRPCID        fixed bin(16) unsigned,/* ID for RPC Runtime */
     2 cbMaxToken    fixed bin(32) unsigned,
                     /* Size of authentication token (max) */
     2 Name          pointer,/* Text name */
     2 Comment       pointer;/* Comment */
 define alias PSecPkgInfoW handle SecPkgInfoW;

 define structure
   1 SecPkgInfoA,
     2 fCapabilities fixed bin(32) unsigned,/* Capability bitmask */
     2 wVersion      fixed bin(16) unsigned,/* Version of driver */
     2 wRPCID        fixed bin(16) unsigned,/* ID for RPC Runtime */
     2 cbMaxToken    fixed bin(32) unsigned,
                     /* Size of authentication token (max) */
     2 Name          pointer,/* Text name */
     2 Comment       pointer;/* Comment */
 define alias PSecPkgInfoA handle SecPkgInfoA;


 %if UNICODE ^= '' %then %do;
 define alias  SecPkgInfo type  SecPkgInfoW;
 define alias PSecPkgInfo type PSecPkgInfoW;
 %end; %else %do;
 define alias  SecPkgInfo type  SecPkgInfoA;
 define alias PSecPkgInfo type PSecPkgInfoA;
 %end; /* !UNICODE */


 /*  Security Package Capabilities */
 dcl ( SECPKG_FLAG_INTEGRITY         value('00000001'xu)
       /* Supports integrity on messages */
     , SECPKG_FLAG_PRIVACY           value('00000002'xu)
       /* Supports privacy (confidentiality) */
     , SECPKG_FLAG_TOKEN_ONLY        value('00000004'xu)
       /* Only security token needed */
     , SECPKG_FLAG_DATAGRAM          value('00000008'xu)
       /* Datagram RPC support */
     , SECPKG_FLAG_CONNECTION        value('00000010'xu)
       /* Connection oriented RPC support */
     , SECPKG_FLAG_MULTI_REQUIRED    value('00000020'xu)
       /* Full 3-leg required for re-auth. */
     , SECPKG_FLAG_CLIENT_ONLY       value('00000040'xu)
       /* Server side functionality not available */
     , SECPKG_FLAG_EXTENDED_ERROR    value('00000080'xu)
       /* Supports extended error msgs */
     , SECPKG_FLAG_IMPERSONATION     value('00000100'xu)
       /* Supports impersonation */
     , SECPKG_FLAG_ACCEPT_WIN32_NAME value('00000200'xu)
       /* Accepts Win32 names */
     , SECPKG_FLAG_STREAM            value('00000400'xu)
       /* Supports stream semantics */
     , SECPKG_FLAG_NEGOTIABLE        value('00000800'xu)
       /* Can be used by the negotiate package */
     , SECPKG_FLAG_GSS_COMPATIBLE    value('00001000'xu)
       /* GSS Compatibility Available */
     , SECPKG_FLAG_LOGON             value('00002000'xu)
       /* Supports common LsaLogonUser */
     , SECPKG_FLAG_ASCII_BUFFERS     value('00004000'xu)
       /* Token Buffers are in ASCII */
     , SECPKG_FLAG_FRAGMENT          value('00008000'xu)
       /* Package can fragment to fit */
     , SECPKG_FLAG_MUTUAL_AUTH       value('00010000'xu)
       /* Package can perform mutual authentication */
     , SECPKG_FLAG_DELEGATION        value('00020000'xu)
       /* Package can delegate */
 ) fixed bin(32) unsigned;


 dcl SECPKG_ID_NONE fixed bin(16) value('FFFF'xu);


 /* SecBuffer */

 /*  Generic memory descriptors for buffers passed in to the security */
 /*  API */

 define structure
   1 SecBuffer,   /* Package can delegate */
     2 cbBuffer   fixed bin(32) unsigned,
                  /* Size of the buffer, in bytes */
     2 BufferType fixed bin(32) unsigned,/* Type of the buffer (below)*/
     2 pvBuffer   pointer;               /* Pointer to the buffer */
 define alias PSecBuffer handle SecBuffer;

 define structure
   1 SecBufferDesc,
     2 ulVersion fixed bin(32) unsigned,/* Version number */
     2 cBuffers  fixed bin(32) unsigned,/* Number of buffers */
     2 pBuffers  type PSecBuffer;       /* Pointer to array of buffers*/
 define alias PSecBufferDesc handle SecBufferDesc;

 dcl SECBUFFER_VERSION            fixed bin(32) unsigned value(0);

 dcl SECBUFFER_EMPTY              fixed bin(32) unsigned value(0);
     /* Undefined, replaced by provider */
 dcl SECBUFFER_DATA               fixed bin(32) unsigned value(1);
     /* Packet data */
 dcl SECBUFFER_TOKEN              fixed bin(32) unsigned value(2);
     /* Security token */
 dcl SECBUFFER_PKG_PARAMS         fixed bin(32) unsigned value(3);
     /* Package specific parameters */
 dcl SECBUFFER_MISSING            fixed bin(32) unsigned value(4);
     /* Missing Data indicator */
 dcl SECBUFFER_EXTRA              fixed bin(32) unsigned value(5);
     /* Extra data */
 dcl SECBUFFER_STREAM_TRAILER     fixed bin(32) unsigned value(6);
     /* Security Trailer */
 dcl SECBUFFER_STREAM_HEADER      fixed bin(32) unsigned value(7);
     /* Security Header */
 dcl SECBUFFER_NEGOTIATION_INFO   fixed bin(32) unsigned value(8);
     /* Hints from the negotiation pkg */
 dcl SECBUFFER_PADDING            fixed bin(32) unsigned value(9);
     /* non-data padding */
 dcl SECBUFFER_STREAM             fixed bin(32) unsigned value(10);
     /* whole encrypted message */
 dcl SECBUFFER_MECHLIST           fixed bin(32) unsigned value(11);
 dcl SECBUFFER_MECHLIST_SIGNATURE fixed bin(32) unsigned value(12);
 dcl SECBUFFER_TARGET             fixed bin(32) unsigned value(13);
 dcl SECBUFFER_CHANNEL_BINDINGS   fixed bin(32) unsigned value(14);

 dcl SECBUFFER_ATTRMASK      fixed bin(32) unsigned value('F0000000'xu);
 dcl SECBUFFER_READONLY      fixed bin(32) unsigned value('80000000'xu);
     /* Buffer is read-only */
 dcl SECBUFFER_READONLY_WITH_CHECKSUM
                             fixed bin(32) unsigned value('10000000'xu);
     /* Buffer is read-only, and checksummed */
 dcl SECBUFFER_RESERVED      fixed bin(32) unsigned value('60000000'xu);
     /* Flags reserved to security system */

 define structure
   1 SEC_NEGOTIATION_INFO,
     2 Size       fixed bin(32) unsigned,/* Size of this structure */
     2 NameLength fixed bin(32) unsigned,/* Length of name hint */
     2 Name       pointer,               /* Name hint */
     2 Reserved   pointer;               /* Reserved */
 define alias PSEC_NEGOTIATION_INFO handle SEC_NEGOTIATION_INFO;

 /*  Data Representation Constant: */
 dcl SECURITY_NATIVE_DREP  fixed bin(32) unsigned value('00000010'xu);
 dcl SECURITY_NETWORK_DREP fixed bin(32) unsigned value('00000000'xu);

 /*  Credential Use Flags */
 dcl SECPKG_CRED_INBOUND   fixed bin(32) unsigned value('00000001'xu);
 dcl SECPKG_CRED_OUTBOUND  fixed bin(32) unsigned value('00000002'xu);
 dcl SECPKG_CRED_BOTH      fixed bin(32) unsigned value('00000003'xu);
 dcl SECPKG_CRED_DEFAULT   fixed bin(32) unsigned value('00000004'xu);
 dcl SECPKG_CRED_RESERVED  fixed bin(32) unsigned value('F0000000'xu);

 /*  InitializeSecurityContext Requirement and return flags: */

 dcl ( ISC_REQ_DELEGATE               value('00000001'xu)
     , ISC_REQ_MUTUAL_AUTH            value('00000002'xu)
     , ISC_REQ_REPLAY_DETECT          value('00000004'xu)
     , ISC_REQ_SEQUENCE_DETECT        value('00000008'xu)
     , ISC_REQ_CONFIDENTIALITY        value('00000010'xu)
     , ISC_REQ_USE_SESSION_KEY        value('00000020'xu)
     , ISC_REQ_PROMPT_FOR_CREDS       value('00000040'xu)
     , ISC_REQ_USE_SUPPLIED_CREDS     value('00000080'xu)
     , ISC_REQ_ALLOCATE_MEMORY        value('00000100'xu)
     , ISC_REQ_USE_DCE_STYLE          value('00000200'xu)
     , ISC_REQ_DATAGRAM               value('00000400'xu)
     , ISC_REQ_CONNECTION             value('00000800'xu)
     , ISC_REQ_CALL_LEVEL             value('00001000'xu)
     , ISC_REQ_FRAGMENT_SUPPLIED      value('00002000'xu)
     , ISC_REQ_EXTENDED_ERROR         value('00004000'xu)
     , ISC_REQ_STREAM                 value('00008000'xu)
     , ISC_REQ_INTEGRITY              value('00010000'xu)
     , ISC_REQ_IDENTIFY               value('00020000'xu)
     , ISC_REQ_NULL_SESSION           value('00040000'xu)
     , ISC_REQ_MANUAL_CRED_VALIDATION value('00080000'xu)
     , ISC_REQ_RESERVED1              value('00100000'xu)
     , ISC_REQ_FRAGMENT_TO_FIT        value('00200000'xu)
     ) fixed bin(32) unsigned;

 dcl ( ISC_RET_DELEGATE               value('00000001'xu)
     , ISC_RET_MUTUAL_AUTH            value('00000002'xu)
     , ISC_RET_REPLAY_DETECT          value('00000004'xu)
     , ISC_RET_SEQUENCE_DETECT        value('00000008'xu)
     , ISC_RET_CONFIDENTIALITY        value('00000010'xu)
     , ISC_RET_USE_SESSION_KEY        value('00000020'xu)
     , ISC_RET_USED_COLLECTED_CREDS   value('00000040'xu)
     , ISC_RET_USED_SUPPLIED_CREDS    value('00000080'xu)
     , ISC_RET_ALLOCATED_MEMORY       value('00000100'xu)
     , ISC_RET_USED_DCE_STYLE         value('00000200'xu)
     , ISC_RET_DATAGRAM               value('00000400'xu)
     , ISC_RET_CONNECTION             value('00000800'xu)
     , ISC_RET_INTERMEDIATE_RETURN    value('00001000'xu)
     , ISC_RET_CALL_LEVEL             value('00002000'xu)
     , ISC_RET_EXTENDED_ERROR         value('00004000'xu)
     , ISC_RET_STREAM                 value('00008000'xu)
     , ISC_RET_INTEGRITY              value('00010000'xu)
     , ISC_RET_IDENTIFY               value('00020000'xu)
     , ISC_RET_NULL_SESSION           value('00040000'xu)
     , ISC_RET_MANUAL_CRED_VALIDATION value('00080000'xu)
     , ISC_RET_RESERVED1              value('00100000'xu)
     , ISC_RET_FRAGMENT_ONLY          value('00200000'xu)
     ) fixed bin(32) unsigned;


 dcl ( ASC_REQ_DELEGATE               value('00000001'xu)
     , ASC_REQ_MUTUAL_AUTH            value('00000002'xu)
     , ASC_REQ_REPLAY_DETECT          value('00000004'xu)
     , ASC_REQ_SEQUENCE_DETECT        value('00000008'xu)
     , ASC_REQ_CONFIDENTIALITY        value('00000010'xu)
     , ASC_REQ_USE_SESSION_KEY        value('00000020'xu)
     , ASC_REQ_ALLOCATE_MEMORY        value('00000100'xu)
     , ASC_REQ_USE_DCE_STYLE          value('00000200'xu)
     , ASC_REQ_DATAGRAM               value('00000400'xu)
     , ASC_REQ_CONNECTION             value('00000800'xu)
     , ASC_REQ_CALL_LEVEL             value('00001000'xu)
     , ASC_REQ_EXTENDED_ERROR         value('00008000'xu)
     , ASC_REQ_STREAM                 value('00010000'xu)
     , ASC_REQ_INTEGRITY              value('00020000'xu)
     , ASC_REQ_LICENSING              value('00040000'xu)
     , ASC_REQ_IDENTIFY               value('00080000'xu)
     , ASC_REQ_ALLOW_NULL_SESSION     value('00100000'xu)
     , ASC_REQ_ALLOW_NON_USER_LOGONS  value('00200000'xu)
     , ASC_REQ_ALLOW_CONTEXT_REPLAY   value('00400000'xu)
     , ASC_REQ_FRAGMENT_TO_FIT        value('00800000'xu)
     , ASC_REQ_FRAGMENT_SUPPLIED      value('00002000'xu)
     , ASC_REQ_NO_TOKEN               value('01000000'xu)
     ) fixed bin(32) unsigned;


 dcl ( ASC_RET_DELEGATE               value('00000001'xu)
     , ASC_RET_MUTUAL_AUTH            value('00000002'xu)
     , ASC_RET_REPLAY_DETECT          value('00000004'xu)
     , ASC_RET_SEQUENCE_DETECT        value('00000008'xu)
     , ASC_RET_CONFIDENTIALITY        value('00000010'xu)
     , ASC_RET_USE_SESSION_KEY        value('00000020'xu)
     , ASC_RET_ALLOCATED_MEMORY       value('00000100'xu)
     , ASC_RET_USED_DCE_STYLE         value('00000200'xu)
     , ASC_RET_DATAGRAM               value('00000400'xu)
     , ASC_RET_CONNECTION             value('00000800'xu)
     , ASC_RET_CALL_LEVEL             value('00002000'xu)
     /* skipped 1000 to be like ISC_ */
     , ASC_RET_THIRD_LEG_FAILED       value('00004000'xu)
     , ASC_RET_EXTENDED_ERROR         value('00008000'xu)
     , ASC_RET_STREAM                 value('00010000'xu)
     , ASC_RET_INTEGRITY              value('00020000'xu)
     , ASC_RET_LICENSING              value('00040000'xu)
     , ASC_RET_IDENTIFY               value('00080000'xu)
     , ASC_RET_NULL_SESSION           value('00100000'xu)
     , ASC_RET_ALLOW_NON_USER_LOGONS  value('00200000'xu)
     , ASC_RET_ALLOW_CONTEXT_REPLAY   value('00400000'xu)
     , ASC_RET_FRAGMENT_ONLY          value('00800000'xu)
     , ASC_RET_NO_TOKEN               value('01000000'xu)
     ) fixed bin(32) unsigned;

 /*  Security Credentials Attributes: */

 dcl SECPKG_CRED_ATTR_NAMES      fixed bin(32) unsigned value(1);

 define structure
   1 SecPkgCredentials_NamesW,
     2 sUserName pointer;
 define alias PSecPkgCredentials_NamesW handle SecPkgCredentials_NamesW;

 define structure
   1 SecPkgCredentials_NamesA,
     2 sUserName pointer;
 define alias PSecPkgCredentials_NamesA handle SecPkgCredentials_NamesA;

 %if UNICODE ^= '' %then %do;
 define alias  SecPkgCredentials_Names type  SecPkgCredentials_NamesW;
 define alias PSecPkgCredentials_Names type PSecPkgCredentials_NamesW;
 %end; %else %do;
 define alias  SecPkgCredentials_Names type  SecPkgCredentials_NamesA;
 define alias PSecPkgCredentials_Names type PSecPkgCredentials_NamesA;
 %end; /* !UNICODE */


 /*  Security Context Attributes: */

 dcl SECPKG_ATTR_SIZES              fixed bin(32) unsigned value(0);
 dcl SECPKG_ATTR_NAMES              fixed bin(32) unsigned value(1);
 dcl SECPKG_ATTR_LIFESPAN           fixed bin(32) unsigned value(2);
 dcl SECPKG_ATTR_DCE_INFO           fixed bin(32) unsigned value(3);
 dcl SECPKG_ATTR_STREAM_SIZES       fixed bin(32) unsigned value(4);
 dcl SECPKG_ATTR_KEY_INFO           fixed bin(32) unsigned value(5);
 dcl SECPKG_ATTR_AUTHORITY          fixed bin(32) unsigned value(6);
 dcl SECPKG_ATTR_PROTO_INFO         fixed bin(32) unsigned value(7);
 dcl SECPKG_ATTR_PASSWORD_EXPIRY    fixed bin(32) unsigned value(8);
 dcl SECPKG_ATTR_SESSION_KEY        fixed bin(32) unsigned value(9);
 dcl SECPKG_ATTR_PACKAGE_INFO       fixed bin(32) unsigned value(10);
 dcl SECPKG_ATTR_USER_FLAGS         fixed bin(32) unsigned value(11);
 dcl SECPKG_ATTR_NEGOTIATION_INFO   fixed bin(32) unsigned value(12);
 dcl SECPKG_ATTR_NATIVE_NAMES       fixed bin(32) unsigned value(13);
 dcl SECPKG_ATTR_FLAGS              fixed bin(32) unsigned value(14);
 dcl SECPKG_ATTR_USE_VALIDATED      fixed bin(32) unsigned value(15);
 dcl SECPKG_ATTR_CREDENTIAL_NAME    fixed bin(32) unsigned value(16);
 dcl SECPKG_ATTR_TARGET_INFORMATION fixed bin(32) unsigned value(17);
 dcl SECPKG_ATTR_ACCESS_TOKEN       fixed bin(32) unsigned value(18);
 dcl SECPKG_ATTR_TARGET             fixed bin(32) unsigned value(19);
 dcl SECPKG_ATTR_AUTHENTICATION_ID  fixed bin(32) unsigned value(20);

 define structure
   1 SecPkgContext_Sizes,
     2 cbMaxToken        fixed bin(32) unsigned,
     2 cbMaxSignature    fixed bin(32) unsigned,
     2 cbBlockSize       fixed bin(32) unsigned,
     2 cbSecurityTrailer fixed bin(32) unsigned;
 define alias PSecPkgContext_Sizes handle SecPkgContext_Sizes;

 define structure
   1 SecPkgContext_StreamSizes,
     2 cbHeader          fixed bin(32) unsigned,
     2 cbTrailer         fixed bin(32) unsigned,
     2 cbMaximumMessage  fixed bin(32) unsigned,
     2 cBuffers          fixed bin(32) unsigned,
     2 cbBlockSize       fixed bin(32) unsigned;
 define alias PSecPkgContext_StreamSizes
        handle SecPkgContext_StreamSizes;

 define structure
   1 SecPkgContext_NamesW,
     2 sUserName pointer;
 define alias PSecPkgContext_NamesW handle SecPkgContext_NamesW;

 define structure
   1 SecPkgContext_NamesA,
     2 sUserName pointer;
 define alias PSecPkgContext_NamesA handle SecPkgContext_NamesA;

 %if UNICODE ^= '' %then %do;
 define alias  SecPkgContext_Names type  SecPkgContext_NamesW;
 define alias PSecPkgContext_Names type PSecPkgContext_NamesW;
 %end; %else %do;
 define alias  SecPkgContext_Names type  SecPkgContext_NamesA;
 define alias PSecPkgContext_Names type PSecPkgContext_NamesA;
 %end; /* !UNICODE */


 define structure
   1 SecPkgContext_Lifespan,
     2 tsStart  type TimeStamp,
     2 tsExpiry type TimeStamp;
 define alias PSecPkgContext_Lifespan handle SecPkgContext_Lifespan;

 define structure
   1 SecPkgContext_DceInfo,
     2 AuthzSvc fixed bin(32) unsigned,
     2 pPac     pointer;
 define alias PSecPkgContext_DceInfo handle SecPkgContext_DceInfo;

 define structure
   1 SecPkgContext_KeyInfoA,
     2 sSignatureAlgorithmName pointer,
     2 sEncryptAlgorithmName   pointer,
     2 KeySize                 fixed bin(32) unsigned,
     2 SignatureAlgorithm      fixed bin(32) unsigned,
     2 EncryptAlgorithm        fixed bin(32) unsigned;
 define alias PSecPkgContext_KeyInfoA handle SecPkgContext_KeyInfoA;

 define structure
   1 SecPkgContext_KeyInfoW,
     2 sSignatureAlgorithmName pointer,
     2 sEncryptAlgorithmName   pointer,
     2 KeySize                 fixed bin(32) unsigned,
     2 SignatureAlgorithm      fixed bin(32) unsigned,
     2 EncryptAlgorithm        fixed bin(32) unsigned;
 define alias PSecPkgContext_KeyInfoW handle SecPkgContext_KeyInfoW;

 %if UNICODE ^= '' %then %do;
 define alias  SecPkgContext_KeyInfo type  SecPkgContext_KeyInfoW;
 define alias PSecPkgContext_KeyInfo type PSecPkgContext_KeyInfoW;
 %end; %else %do;
 define alias  SecPkgContext_KeyInfo type  SecPkgContext_KeyInfoA;
 define alias PSecPkgContext_KeyInfo type PSecPkgContext_KeyInfoA;
 %end;

 define structure
   1 SecPkgContext_AuthorityA,
     2 sAuthorityName pointer;
 define alias PSecPkgContext_AuthorityA handle SecPkgContext_AuthorityA;

 define structure
   1 SecPkgContext_AuthorityW,
     2 sAuthorityName pointer;
 define alias PSecPkgContext_AuthorityW handle SecPkgContext_AuthorityW;

 %if UNICODE ^= '' %then %do;
 define alias  SecPkgContext_Authority type  SecPkgContext_AuthorityW;
 define alias PSecPkgContext_Authority type PSecPkgContext_AuthorityW;
 %end; %else %do;
 define alias  SecPkgContext_Authority type  SecPkgContext_AuthorityA;
 define alias PSecPkgContext_Authority type PSecPkgContext_AuthorityA;
 %end;

 define structure
   1 SecPkgContext_ProtoInfoA,
     2 sProtocolName pointer,
     2 majorVersion  fixed bin(32) unsigned,
     2 minorVersion  fixed bin(32) unsigned;
 define alias PSecPkgContext_ProtoInfoA handle SecPkgContext_ProtoInfoA;

 define structure
   1 SecPkgContext_ProtoInfoW,
     2 sProtocolName pointer,
     2 majorVersion  fixed bin(32) unsigned,
     2 minorVersion  fixed bin(32) unsigned;
 define alias PSecPkgContext_ProtoInfoW handle SecPkgContext_ProtoInfoW;

 %if UNICODE ^= '' %then %do;
 define alias  SecPkgContext_ProtoInfo type  SecPkgContext_ProtoInfoW;
 define alias PSecPkgContext_ProtoInfo type PSecPkgContext_ProtoInfoW;
 %end; %else %do;
 define alias  SecPkgContext_ProtoInfo type  SecPkgContext_ProtoInfoA;
 define alias PSecPkgContext_ProtoInfo type PSecPkgContext_ProtoInfoA;
 %end;

 define structure
   1 SecPkgContext_PasswordExpiry,
     2 tsPasswordExpires type TimeStamp;
 define alias PSecPkgContext_PasswordExpiry
        handle SecPkgContext_PasswordExpiry;

 define structure
   1 SecPkgContext_SessionKey,
     2 SessionKeyLength fixed bin(32) unsigned,
     2 SessionKey       pointer;

 define alias PSecPkgContext_SessionKey handle SecPkgContext_SessionKey;


 define structure
   1 SecPkgContext_PackageInfoW,
     2 PackageInfo type PSecPkgInfoW;
 define alias PSecPkgContext_PackageInfoW
        handle SecPkgContext_PackageInfoW;

 define structure
   1 SecPkgContext_PackageInfoA,
     2 PackageInfo type PSecPkgInfoA;
 define alias PSecPkgContext_PackageInfoA
        handle SecPkgContext_PackageInfoA;

 define structure
   1 SecPkgContext_UserFlags,
     2 UserFlags fixed bin(32) unsigned;
 define alias PSecPkgContext_UserFlags handle SecPkgContext_UserFlags;

 define structure
   1 SecPkgContext_Flags,
     2 Flags fixed bin(32) unsigned;
 define alias PSecPkgContext_Flags handle SecPkgContext_Flags;


 %if UNICODE ^= '' %then %do;
 define alias  SecPkgContext_PackageInfo
         type  SecPkgContext_PackageInfoW;
 define alias PSecPkgContext_PackageInfo
         type PSecPkgContext_PackageInfoW;
 %end; %else %do;
 define alias  SecPkgContext_PackageInfo
         type  SecPkgContext_PackageInfoA;
 define alias PSecPkgContext_PackageInfo
         type PSecPkgContext_PackageInfoA;
 %end;

 define structure
   1 SecPkgContext_NegotiationInfoA,
     2 PackageInfo      type PSecPkgInfoA,
     2 NegotiationState fixed bin(32) unsigned;
 define alias PSecPkgContext_NegotiationInfoA
        handle SecPkgContext_NegotiationInfoA;

 define structure
   1 SecPkgContext_NegotiationInfoW,
     2 PackageInfo      type PSecPkgInfoW,
     2 NegotiationState fixed bin(32) unsigned;
 define alias PSecPkgContext_NegotiationInfoW
        handle SecPkgContext_NegotiationInfoW;

 %if UNICODE ^= '' %then %do;
 define alias  SecPkgContext_NegotiationInfo
         type  SecPkgContext_NegotiationInfoW;
 define alias PSecPkgContext_NegotiationInfo
         type PSecPkgContext_NegotiationInfoW;
 %end; %else %do;
 define alias  SecPkgContext_NegotiationInfo
         type SecPkgContext_NegotiationInfoA;
 define alias PSecPkgContext_NegotiationInfo
         type PSecPkgContext_NegotiationInfoA;
 %end;

 dcl SECPKG_NEGOTIATION_COMPLETE      fixed bin(32) unsigned value(0);
 dcl SECPKG_NEGOTIATION_OPTIMISTIC    fixed bin(32) unsigned value(1);
 dcl SECPKG_NEGOTIATION_IN_PROGRESS   fixed bin(32) unsigned value(2);
 dcl SECPKG_NEGOTIATION_DIRECT        fixed bin(32) unsigned value(3);
 dcl SECPKG_NEGOTIATION_TRY_MULTICRED fixed bin(32) unsigned value(4);

 define structure
   1 SecPkgContext_NativeNamesW,
     2 sClientName pointer,
     2 sServerName pointer;
 define alias PSecPkgContext_NativeNamesW
        handle SecPkgContext_NativeNamesW;

 define structure
   1 SecPkgContext_NativeNamesA,
     2 sClientName pointer,
     2 sServerName pointer;
 define alias PSecPkgContext_NativeNamesA
        handle SecPkgContext_NativeNamesA;

 %if UNICODE ^= '' %then %do;
 define alias  SecPkgContext_NativeNames
         type  SecPkgContext_NativeNamesW;
 define alias PSecPkgContext_NativeNames
         type PSecPkgContext_NativeNamesW;
 %end; %else %do;
 define alias  SecPkgContext_NativeNames
         type  SecPkgContext_NativeNamesA;
 define alias PSecPkgContext_NativeNames
         type PSecPkgContext_NativeNamesA;
 %end; /* !UNICODE */


 define structure
   1 SecPkgContext_CredentialNameW,
     3 CredentialType  fixed bin (32) unsigned,
     3 sCredentialName pointer;
 define alias PSecPkgContext_CredentialNameW
        handle SecPkgContext_CredentialNameW;

 define structure
   1 SecPkgContext_CredentialNameA,
     3 CredentialType  fixed bin (32) unsigned,
     3 sCredentialName pointer;
 define alias PSecPkgContext_CredentialNameA
        handle SecPkgContext_CredentialNameA;

 %if UNICODE ^= '' %then %do;
 define alias  SecPkgContext_CredentialName
         type  SecPkgContext_CredentialNameW;
 define alias PSecPkgContext_CredentialName
         type PSecPkgContext_CredentialNameW;
 %end; %else %do;
 define alias  SecPkgContext_CredentialName
         type  SecPkgContext_CredentialNameA;
 define alias PSecPkgContext_CredentialName
         type PSecPkgContext_CredentialNameA;
 %end;


 define structure
   1 SecPkgContext_AccessToken,
     3 AccessToken pointer;
 define alias PSecPkgContext_AccessToken
        handle SecPkgContext_AccessToken;

 define structure
   1 SecPkgContext_TargetInformation,
     3 MarshalledTargetInfoLength fixed bin (32) unsigned,
     3 MarshalledTargetInfo       pointer;
 define alias PSecPkgContext_TargetInformation
        handle SecPkgContext_TargetInformation;

 define structure
   1 SecPkgContext_AuthzID,
     3 AuthzIDLength fixed bin (32) unsigned,
     3 AuthzID       pointer;
 define alias PSecPkgContext_AuthzID
        handle SecPkgContext_AuthzID;

 define structure
   1 SecPkgContext_Target,
     3 TargetLength fixed bin (32) unsigned,
     3 Target       pointer;
 define alias PSecPkgContext_Target
        handle SecPkgContext_Target;

 define alias SEC_GET_KEY_FN
        entry (pointer,                     /* Argument passed in */
               pointer,                     /* Principal ID */
               fixed bin(32) unsigned,      /* Key Version */
               pointer byaddr,              /* Returned ptr to key */
               type SECURITY_STATUS byaddr) /* returned status */
        limited SEC_ENTRY;

 /* Flags for ExportSecurityContext */

 dcl ( SECPKG_CONTEXT_EXPORT_RESET_NEW   value('00000001'xu)
       /* New context is reset to initial state */
     , SECPKG_CONTEXT_EXPORT_DELETE_OLD  value('00000002'xu)
       /* Old context is deleted during export */
     ) fixed bin(32) unsigned;


 dcl AcquireCredentialsHandleW
     entry (
 %if ISSP_MODE = 0 %then %do;             /* For Kernel mode */
            type SECURITY_STRING byaddr optional,
            type SECURITY_STRING byaddr,
 %end; %else %do;
            SEC_WCHAR(*) varz byaddr optional, /* Name of principal */
            SEC_WCHAR(*) varz byaddr,    /* Name of package */
 %end;
            fixed bin(32) unsigned,      /* Flags indicating use */
            pointer,                     /* Pointer to logon ID */
            pointer,                     /* Package specific data */
            type SEC_GET_KEY_FN optional,/* Pointer to GetKey() func */
            pointer,                     /* Value to pass to GetKey() */
            type CredHandle byaddr,      /* (out) Cred Handle */
            type TimeStamp byaddr optional)
                                         /* (out) Lifetime (optional) */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('AcquireCredentialsHandleW');

 define alias ACQUIRE_CREDENTIALS_HANDLE_FN_W
        entry (
 %if ISSP_MODE = 0 %then %do;
               type SECURITY_STRING byaddr optional,
               type SECURITY_STRING byaddr,
 %end; %else %do;
               SEC_WCHAR(*) varz byaddr optional,
               SEC_WCHAR(*) varz byaddr,
 %end;
               fixed bin(32) unsigned,
               pointer,
               pointer,
               type SEC_GET_KEY_FN optional,
               pointer,
               type CredHandle byaddr,
               type TimeStamp byaddr optional)
       returns (type SECURITY_STATUS byvalue optional)
       limited SEC_ENTRY;

 dcl AcquireCredentialsHandleA
     entry (SEC_CHAR(*) varz byaddr optional, /* Name of principal */
            SEC_CHAR(*) varz byaddr,     /* Name of package */
            fixed bin(32) unsigned,      /* Flags indicating use */
            pointer,                     /* Pointer to logon ID */
            pointer,                     /* Package specific data */
            type SEC_GET_KEY_FN optional,/* Pointer to GetKey() func */
            pointer,                     /* Value to pass to GetKey() */
            type CredHandle byaddr,      /* (out) Cred Handle */
            type TimeStamp byaddr optional)
                                         /* (out) Lifetime (optional) */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('AcquireCredentialsHandleA');

 define alias ACQUIRE_CREDENTIALS_HANDLE_FN_A
        entry (SEC_CHAR(*) varz byaddr optional,
               SEC_CHAR(*) varz byaddr,
               fixed bin(32) unsigned,
               pointer,
               pointer,
               type SEC_GET_KEY_FN optional,
               pointer,
               type CredHandle byaddr,
               type TimeStamp byaddr optional)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 %dcl AcquireCredentialsHandle char ext;
 %if UNICODE ^= '' %then %do;
 %AcquireCredentialsHandle='AcquireCredentialsHandleW';
 define alias ACQUIRE_CREDENTIALS_HANDLE_FN
         type ACQUIRE_CREDENTIALS_HANDLE_FN_W;
 %end; %else %do;
 %AcquireCredentialsHandle='AcquireCredentialsHandleA';
 define alias ACQUIRE_CREDENTIALS_HANDLE_FN
         type ACQUIRE_CREDENTIALS_HANDLE_FN_A;
 %end; /* !UNICODE */

 dcl FreeCredentialsHandle
     entry (type CredHandle byaddr) /* Handle to free */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('FreeCredentialsHandle');

 define alias FREE_CREDENTIALS_HANDLE_FN
        entry (type CredHandle byaddr )
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 dcl AddCredentialsW
     entry (type CredHandle byaddr,
 %if ISSP_MODE = 0 %then %do;         /* For Kernel mode */
            type SECURITY_STRING byaddr optional,
            type SECURITY_STRING byaddr,
 %end; %else %do;
            SEC_WCHAR(*) varz byaddr optional, /* Name of principal */
            SEC_WCHAR(*) varz byaddr,    /* Name of package */
 %end;
            fixed bin(32) unsigned,      /* Flags indicating use */
            pointer,                     /* Package specific data */
            type SEC_GET_KEY_FN optional,/* Pointer to GetKey() func */
            pointer,                     /* Value to pass to GetKey() */
            type TimeStamp byaddr)       /* (out) Lifetime (optional) */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('AddCredentialsW');

 define alias ADD_CREDENTIALS_FN_W
        entry (type CredHandle byaddr,
 %if ISSP_MODE = 0 %then %do;
               type SECURITY_STRING byaddr optional,
               type SECURITY_STRING byaddr,
 %end; %else %do;
               SEC_WCHAR(*) varz byaddr optional,
               SEC_WCHAR(*) varz byaddr,
 %end;
               fixed bin(32) unsigned,
               pointer,
               type SEC_GET_KEY_FN optional,
               pointer,
               type TimeStamp byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 dcl AddCredentialsA
     entry (type CredHandle byaddr,
            SEC_CHAR(*) varz byaddr optional, /* Name of principal */
            SEC_CHAR(*) varz byaddr,     /* Name of package */
            fixed bin(32) unsigned,      /* Flags indicating use */
            pointer,                     /* Package specific data */
            type SEC_GET_KEY_FN optional,/* Pointer to GetKey() func */
            pointer,                     /* Value to pass to GetKey() */
            type TimeStamp byaddr)       /* (out) Lifetime (optional) */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('AddCredentialsA');

 define alias ADD_CREDENTIALS_FN_A
        entry (type CredHandle byaddr,
               SEC_CHAR(*) varz byaddr optional,
               SEC_CHAR(*) varz byaddr,
               fixed bin(32) unsigned,
               pointer,
               type SEC_GET_KEY_FN optional,
               pointer,
               type TimeStamp byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 %dcl AddCredentials char ext;
 %if UNICODE ^= '' %then %do;
 %AddCredentials='AddCredentialsW';
 define alias ADD_CREDENTIALS_FN type  ADD_CREDENTIALS_FN_W;
 %end; %else %do;
 %AddCredentials='AddCredentialsA';
 define alias ADD_CREDENTIALS_FN type ADD_CREDENTIALS_FN_A;
 %end;

 %if WIN32_CHICAGO ^= '' %then %do;
 dcl SspiLogonUserW
     entry (SEC_WCHAR(*) varz byaddr, /* Name of package */
            SEC_WCHAR(*) varz byaddr, /* Name of user    */
            SEC_WCHAR(*) varz byaddr, /* Domain Name     */
            SEC_WCHAR(*) varz byaddr) /* Password        */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SspiLogonUserW');

 define alias SSPI_LOGON_USER_FN_W
        entry (SEC_CHAR(*) varz byaddr,
               SEC_CHAR(*) varz byaddr,
               SEC_CHAR(*) varz byaddr,
               SEC_CHAR(*) varz byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 dcl SspiLogonUserA
     entry (SEC_CHAR(*) varz byaddr, /* Name of package */
            SEC_CHAR(*) varz byaddr, /* Name of user    */
            SEC_CHAR(*) varz byaddr, /* Domain Name     */
            SEC_CHAR(*) varz byaddr) /* Password        */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SspiLogonUserA');

 define alias SSPI_LOGON_USER_FN_A
        entry (SEC_CHAR(*) varz byaddr,
               SEC_CHAR(*) varz byaddr,
               SEC_CHAR(*) varz byaddr,
               SEC_CHAR(*) varz byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 %dcl SspiLogonUser char ext;
 %if UNICODE ^= '' %then %do;
 %SspiLogonUser='SspiLogonUserW';
 define alias SSPI_LOGON_USER_FN type SSPI_LOGON_USER_FN_W;
 %end; %else %do;
 %SspiLogonUser='SspiLogonUserA';
 define alias SSPI_LOGON_USER_FN type SSPI_LOGON_USER_FN_A;
 %end; /* !UNICODE */
 %end; /* WIN32_CHICAGO */



 /***********************************/
 /*                                 */
 /* Context Management Functions    */
 /*                                 */
 /***********************************/

 dcl InitializeSecurityContextW
     entry (type CredHandle byaddr, /* Cred to base context */
            type CtxtHandle byaddr optional,/* Existing context (OPT) */
 %if ISSP_MODE = 0 %then %do;
            type SECURITY_STRING byaddr optional,
 %end; %else %do;
            SEC_WCHAR(*) varz byaddr optional,  /* Name of target */
 %end;
            fixed bin(32) unsigned,        /* Context Requirements */
            fixed bin(32) unsigned,        /* Reserved, MBZ */
            fixed bin(32) unsigned,        /* Data rep of target */
            type PSecBufferDesc byaddr optional,/* Input Buffers */
            fixed bin(32) unsigned,        /* Reserved, MBZ */
            type CtxtHandle byaddr,        /* (out) New Context handle*/
            type PSecBufferDesc byaddr,    /* (inout) Output Buffers */
            fixed bin(32) unsigned byaddr, /* (out) Context attrs */
            type TimeStamp byaddr optional)/* (out) Life span (OPT) */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('InitializeSecurityContextW');

 define alias INITIALIZE_SECURITY_CONTEXT_FN_W
        entry (type CredHandle byaddr,
               type CtxtHandle byaddr optional,
 %if ISSP_MODE = 0 %then %do;
               type SECURITY_STRING byaddr optional,
 %end; %else %do;
               SEC_WCHAR(*) varz byaddr optional,
 %end;
               fixed bin(32) unsigned,
               fixed bin(32) unsigned,
               fixed bin(32) unsigned,
               type PSecBufferDesc byaddr optional,
               fixed bin(32) unsigned,
               type CtxtHandle byaddr,
               type PSecBufferDesc byaddr,
               fixed bin(32) unsigned byaddr,
               type TimeStamp byaddr optional)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 dcl InitializeSecurityContextA
     entry (type CredHandle byaddr,        /* Cred to base context */
            type CtxtHandle byaddr optional,/* Existing context (OPT) */
            SEC_CHAR(*) varz byaddr optional,    /* Name of target */
            fixed bin(32) unsigned,        /* Context Requirements */
            fixed bin(32) unsigned,        /* Reserved, MBZ */
            fixed bin(32) unsigned,        /* Data rep of target */
            type PSecBufferDesc byaddr optional,/* Input Buffers */
            fixed bin(32) unsigned,        /* Reserved, MBZ */
            type CtxtHandle byaddr,        /* (out) New Context handle*/
            type PSecBufferDesc byaddr,    /* (inout) Output Buffers */
            fixed bin(32) unsigned byaddr, /* (out) Context attrs */
            type TimeStamp byaddr optional)/* (out) Life span (OPT) */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('InitializeSecurityContextA');

 define alias  INITIALIZE_SECURITY_CONTEXT_FN_A
        entry (type CredHandle byaddr,
               type CtxtHandle byaddr optional,
               SEC_CHAR(*) varz byaddr optional,
               fixed bin(32) unsigned,
               fixed bin(32) unsigned,
               fixed bin(32) unsigned,
               type PSecBufferDesc byaddr optional,
               fixed bin(32) unsigned,
               type CtxtHandle byaddr,
               type PSecBufferDesc byaddr,
               fixed bin(32) unsigned byaddr,
               type TimeStamp byaddr optional)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 %dcl InitializeSecurityContext char ext;
 %if UNICODE ^= '' %then %do;
 %InitializeSecurityContext='InitializeSecurityContextW';
 define alias INITIALIZE_SECURITY_CONTEXT_FN
         type INITIALIZE_SECURITY_CONTEXT_FN_W;
 %end; %else %do;
 %InitializeSecurityContext='InitializeSecurityContextA';
 define alias INITIALIZE_SECURITY_CONTEXT_FN
         type INITIALIZE_SECURITY_CONTEXT_FN_A;
 %end; /* !UNICODE */

 dcl AcceptSecurityContext
     entry (type CredHandle byaddr,        /* Cred to base context */
            type CtxtHandle byaddr optional,/* Existing context (OPT) */
            type PSecBufferDesc byaddr,    /* Input buffer */
            fixed bin(32) unsigned,        /* Context Requirements */
            fixed bin(32) unsigned,        /* Target Data Rep */
            type CtxtHandle byaddr,        /* (out) New context handle*/
            type PSecBufferDesc byaddr,    /* (inout) Output buffers */
            fixed bin(32) unsigned byaddr, /* (out) Context attributes*/
            type TimeStamp byaddr optional)/* (out) Life span (OPT) */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('AcceptSecurityContext');

 define alias ACCEPT_SECURITY_CONTEXT_FN
        entry (type CredHandle byaddr,
               type CtxtHandle byaddr optional,
               type PSecBufferDesc byaddr,
               fixed bin(32) unsigned,
               fixed bin(32) unsigned,
               type CtxtHandle byaddr,
               type PSecBufferDesc byaddr,
               fixed bin(32) unsigned byaddr,
               type TimeStamp byaddr optional)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 dcl CompleteAuthToken
     entry (type CtxtHandle byaddr,     /* Context to complete */
            type PSecBufferDesc byaddr) /* Token to complete */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('CompleteAuthToken');

 define alias COMPLETE_AUTH_TOKEN_FN
        entry (type CtxtHandle byaddr,
               type PSecBufferDesc byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 dcl ImpersonateSecurityContext
     entry (type CtxtHandle byaddr) /* Context to impersonate */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('ImpersonateSecurityContext');

 define alias IMPERSONATE_SECURITY_CONTEXT_FN
        entry (type CtxtHandle byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 dcl RevertSecurityContext
     entry (type CtxtHandle byaddr) /* Context from which to re */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('RevertSecurityContext');

 define alias REVERT_SECURITY_CONTEXT_FN
        entry (type CtxtHandle byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 dcl QuerySecurityContextToken
     entry (type CtxtHandle byaddr,
            pointer byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('QuerySecurityContextToken');

 define alias QUERY_SECURITY_CONTEXT_TOKEN_FN
        entry (type CtxtHandle byaddr,
               pointer byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 dcl DeleteSecurityContext
     entry (type CtxtHandle byaddr) /* Context to delete */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('DeleteSecurityContext');

 define alias DELETE_SECURITY_CONTEXT_FN
        entry (type CtxtHandle byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 dcl ApplyControlToken
     entry (type CtxtHandle byaddr,     /* Context to modify */
            type PSecBufferDesc byaddr) /* Input token to apply */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('ApplyControlToken');

 define alias APPLY_CONTROL_TOKEN_FN
        entry (type CtxtHandle byaddr,
               type PSecBufferDesc byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 dcl QueryContextAttributesW
     entry (type CtxtHandle byaddr, /* Context to query */
            fixed bin(32) unsigned, /* Attribute to query */
            pointer)                /* Buffer for attributes */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('QueryContextAttributesW');

 define alias QUERY_CONTEXT_ATTRIBUTES_FN_W
        entry (type CtxtHandle byaddr,
               fixed bin(32) unsigned,
               pointer)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 dcl QueryContextAttributesA
     entry (type CtxtHandle byaddr, /* Context to query */
            fixed bin(32) unsigned, /* Attribute to query */
            pointer)                /* Buffer for attributes */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('QueryContextAttributesA');

 define alias QUERY_CONTEXT_ATTRIBUTES_FN_A
        entry (type CtxtHandle byaddr,
               fixed bin(32) unsigned,
               pointer)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 %dcl QueryContextAttributes char ext;
 %if UNICODE ^= '' %then %do;
 %QueryContextAttributes='QueryContextAttributesW';
 define alias QUERY_CONTEXT_ATTRIBUTES_FN
         type QUERY_CONTEXT_ATTRIBUTES_FN_W;
 %end; %else %do;
 %QueryContextAttributes='QueryContextAttributesA';
 define alias QUERY_CONTEXT_ATTRIBUTES_FN
         type QUERY_CONTEXT_ATTRIBUTES_FN_A;
 %end; /* !UNICODE */


 dcl SetContextAttributesW
     entry (type CtxtHandle byaddr, /* Context to query */
            fixed bin(32) unsigned, /* Attribute to set */
            pointer,                /* Buffer for attributes */
            fixed bin(32) unsigned) /* Size (in bytes) of Buffer */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SetContextAttributesW');

 define alias SET_CONTEXT_ATTRIBUTES_FN_W
        entry (type CtxtHandle byaddr, /* Context to query */
               fixed bin(32) unsigned, /* Attribute to set */
               pointer,                /* Buffer for attributes */
               fixed bin(32) unsigned) /* Size (in bytes) of Buffer */
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 dcl SetContextAttributesA
     entry (type CtxtHandle byaddr, /* Context to query */
            fixed bin(32) unsigned, /* Attribute to set */
            pointer,                /* Buffer for attributes */
            fixed bin(32) unsigned) /* Size (in bytes) of Buffer */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SetContextAttributesA');

 define alias SET_CONTEXT_ATTRIBUTES_FN_A
        entry (type CtxtHandle byaddr, /* Context to query */
               fixed bin(32) unsigned, /* Attribute to set */
               pointer,                /* Buffer for attributes */
               fixed bin(32) unsigned) /* Size (in bytes) of Buffer */
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 %dcl SetContextAttributes char ext;
 %if UNICODE ^= '' %then %do;
 %SetContextAttributes='SetContextAttributesW';
 define alias SET_CONTEXT_ATTRIBUTES_FN
         type SET_CONTEXT_ATTRIBUTES_FN_W;
 %end; %else %do;
 %SetContextAttributes='SetContextAttributesA';
 define alias SET_CONTEXT_ATTRIBUTES_FN
         type SET_CONTEXT_ATTRIBUTES_FN_A;
 %end; /* !UNICODE */


 dcl QueryCredentialsAttributesW
     entry (type CredHandle byaddr, /* Credential to query */
            fixed bin(32) unsigned, /* Attribute to query */
            pointer)                /* Buffer for attributes */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('QueryCredentialsAttributesW');

 define alias QUERY_CREDENTIALS_ATTRIBUTES_FN_W
        entry (type CredHandle byaddr,
               fixed bin(32) unsigned,
               pointer)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 dcl QueryCredentialsAttributesA
     entry (type CredHandle byaddr, /* Credential to query */
            fixed bin(32) unsigned, /* Attribute to query */
            pointer)                /* Buffer for attributes */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('QueryCredentialsAttributesA');

 define alias QUERY_CREDENTIALS_ATTRIBUTES_FN_A
        entry (type CredHandle byaddr,
               fixed bin(32) unsigned,
               pointer)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 %dcl QueryCredentialsAttributes char ext;
 %if UNICODE ^= '' %then %do;
 %QueryCredentialsAttributes='QueryCredentialsAttributesW';
 define alias QUERY_CREDENTIALS_ATTRIBUTES_FN
         type QUERY_CREDENTIALS_ATTRIBUTES_FN_W;
 %end; %else %do;
 %QueryCredentialsAttributes='QueryCredentialsAttributesA';
 define alias QUERY_CREDENTIALS_ATTRIBUTES_FN
         type QUERY_CREDENTIALS_ATTRIBUTES_FN_A;
 %end; /* !UNICODE */


 dcl FreeContextBuffer
     entry (pointer) /* buffer to free */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('FreeContextBuffer');

 define alias FREE_CONTEXT_BUFFER_FN
        entry (pointer)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 /******************************/
 /*                            */
 /*    Message Support API     */
 /*                            */
 /******************************/

 dcl MakeSignature
     entry (type CtxtHandle byaddr,     /* Context to use */
            fixed bin(32) unsigned,     /* Quality of Protection */
            type PSecBufferDesc byaddr, /* Message to sign */
            fixed bin(32) unsigned)     /* Message Sequence Num. */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('MakeSignature');

 define alias MAKE_SIGNATURE_FN
        entry (type CtxtHandle byaddr,
               fixed bin(32) unsigned,
               type PSecBufferDesc byaddr,
               fixed bin(32) unsigned)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 dcl VerifySignature
     entry (type CtxtHandle byaddr,        /* Context to use */
            type PSecBufferDesc byaddr,    /* Message to verify */
            fixed bin(32) unsigned,        /* Sequence Num. */
            fixed bin(32) unsigned byaddr) /* QOP used */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('VerifySignature');

 define alias VERIFY_SIGNATURE_FN
        entry (type CtxtHandle byaddr,
               type PSecBufferDesc byaddr,
               fixed bin(32) unsigned,
               fixed bin(32) unsigned byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 dcl EncryptMessage
     entry (type CtxtHandle byaddr,
            fixed bin(32) unsigned,
            type PSecBufferDesc byaddr,
            fixed bin(32) unsigned)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('EncryptMessage');

 define alias ENCRYPT_MESSAGE_FN
        entry (type CtxtHandle byaddr,
               fixed bin(32) unsigned,
               type PSecBufferDesc byaddr,
               fixed bin(32) unsigned)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 dcl DecryptMessage
     entry (type CtxtHandle byaddr,
            type PSecBufferDesc byaddr,
            fixed bin(32) unsigned,
            fixed bin(32) unsigned byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('DecryptMessage');

 define alias DECRYPT_MESSAGE_FN
        entry (type CtxtHandle byaddr,
               type PSecBufferDesc byaddr,
               fixed bin(32) unsigned,
               fixed bin(32) unsigned byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;


 /********************/
 /*                  */
 /*    Misc.         */
 /*                  */
 /********************/

 dcl EnumerateSecurityPackagesW
     entry (fixed bin(32) unsigned byaddr, /* Receives num. packages */
            type PSecPkgInfoW byaddr)      /* Receives array of info */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('EnumerateSecurityPackagesW');

 define alias ENUMERATE_SECURITY_PACKAGES_FN_W
        entry (fixed bin(32) unsigned byaddr,
               type PSecPkgInfoW byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 dcl EnumerateSecurityPackagesA
     entry (fixed bin(32) unsigned byaddr, /* Receives num. packages */
            type PSecPkgInfoA byaddr)      /* Receives array of info */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('EnumerateSecurityPackagesA');

 define alias ENUMERATE_SECURITY_PACKAGES_FN_A
        entry (fixed bin(32) unsigned byaddr,
               type PSecPkgInfoA byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 %dcl EnumerateSecurityPackages char ext;
 %if UNICODE ^= '' %then %do;
 %EnumerateSecurityPackages='EnumerateSecurityPackagesW';
 define alias ENUMERATE_SECURITY_PACKAGES_FN
         type ENUMERATE_SECURITY_PACKAGES_FN_W;
 %end; %else %do;
 %EnumerateSecurityPackages='EnumerateSecurityPackagesA';
 define alias ENUMERATE_SECURITY_PACKAGES_FN
         type ENUMERATE_SECURITY_PACKAGES_FN_A;
 %end; /* !UNICODE */

 dcl QuerySecurityPackageInfoW
     entry (
 %if ISSP_MODE = 0 %then %do;
            type SECURITY_STRING byaddr,
 %end; %else %do;
            SEC_WCHAR(*) varz byaddr,    /* Name of package */
 %end;
            type PSecPkgInfoW byaddr)    /* Receives package info */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('QuerySecurityPackageInfoW');

 define alias QUERY_SECURITY_PACKAGE_INFO_FN_W
        entry (
 %if ISSP_MODE = 0 %then %do;
               type SECURITY_STRING byaddr,
 %end; %else %do;
               SEC_WCHAR(*) varz byaddr,
 %end;
               type PSecPkgInfoW byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 dcl QuerySecurityPackageInfoA
     entry (SEC_CHAR(*) varz byaddr,     /* Name of package */
            type PSecPkgInfoA byaddr)    /* Receives package info */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('QuerySecurityPackageInfoA');

 define alias  QUERY_SECURITY_PACKAGE_INFO_FN_A
        entry (SEC_CHAR(*) varz byaddr,
               type PSecPkgInfoA byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 %dcl QuerySecurityPackageInfo char ext;
 %if UNICODE ^= '' %then %do;
 %QuerySecurityPackageInfo='QuerySecurityPackageInfoW';
 define alias QUERY_SECURITY_PACKAGE_INFO_FN
         type QUERY_SECURITY_PACKAGE_INFO_FN_W;
 %end; %else %do;
 %QuerySecurityPackageInfo='QuerySecurityPackageInfoA';
 define alias QUERY_SECURITY_PACKAGE_INFO_FN
         type QUERY_SECURITY_PACKAGE_INFO_FN_A;
 %end; /* !UNICODE */


 %if ISSP_MODE = 0 %then %do;

 %dcl DeleteSecurityContextDefer char ext;
 %DeleteSecurityContextDefer='DeleteSecurityContext';
 %dcl FreeCredentialsHandleDefer char ext;
 %FreeCredentialsHandleDefer='FreeCredentialsHandle';

 %if 0 ^= 0 %then %do;

 /* Deferred mode calls for rdr */

 dcl DeleteSecurityContextDefer
     entry (type CtxtHandle byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('DeleteSecurityContextDefer');

 dcl FreeCredentialsHandleDefer
     entry (type CredHandle byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('FreeCredentialsHandleDefer');

 %end;

 %end;

 define ordinal
   SecDelegationType
     ( SecFull,
       SecService,
       SecTree,
       SecDirectory,
       SecObject);
 dcl PSecDelegationType pointer;

 dcl DelegateSecurityContext
     entry (type CtxtHandle byaddr,  /* IN Active context to delegate */
 %if ISSP_MODE = 0 %then %do;
            type SECURITY_STRING byaddr,    /* IN Target path */
 %end; %else %do;
            SEC_CHAR(*) varz byaddr,
 %end;
            type SecDelegationType,         /* IN Type of delegation */
            type TimeStamp byaddr optional, /* IN OPTIONAL time limit */
            type SecBuffer byaddr optional,
                                      /* IN OPTIONAL package specific */
            type SecBufferDesc byaddr)
                                  /* OUT Token for applycontroltoken. */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('DelegateSecurityContext');


 /*******************************/
 /*                             */
 /*    Proxies                  */
 /*                             */
 /*******************************/


 /* Proxies are only available on NT platforms */


 /*******************************/
 /*                             */
 /*    Context export/import    */
 /*                             */
 /*******************************/



 dcl ExportSecurityContext
     entry (type CtxtHandle byaddr, /* (in) context to export */
            type ULONG,             /* (in) option flags */
            type SecBuffer byaddr,  /* (out) marshalled context */
            pointer byaddr)
                    /* (out, optional) token handle for impersonation */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('ExportSecurityContext');

 define alias EXPORT_SECURITY_CONTEXT_FN
        entry (type CtxtHandle byaddr,
               type ULONG,
               type SecBuffer byaddr,
               pointer byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 dcl ImportSecurityContextW
     entry (
 %if ISSP_MODE = 0 %then %do;
            type SECURITY_STRING byaddr,
 %end; %else %do;
            SEC_WCHAR(*) varz byaddr,
 %end;
            type SecBuffer byaddr,  /* (in) marshalled context */
            pointer,    /* (in, optional) handle to token for context */
            type CtxtHandle byaddr) /* (out) new context handle */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('ImportSecurityContextW');

 define alias IMPORT_SECURITY_CONTEXT_FN_W
        entry (
 %if ISSP_MODE = 0 %then %do;
               type SECURITY_STRING byaddr,
 %end; %else %do;
               SEC_WCHAR(*) varz byaddr,
 %end;
               type SecBuffer byaddr,
               pointer,
               type CtxtHandle byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 dcl ImportSecurityContextA
      entry (SEC_CHAR(*) varz byaddr,
             type SecBuffer byaddr, /* (in) marshalled context */
             pointer,   /* (in, optional) handle to token for context */
            type CtxtHandle byaddr) /* (out) new context handle */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('ImportSecurityContextA');

 define alias IMPORT_SECURITY_CONTEXT_FN_A
        entry (SEC_CHAR(*) varz byaddr,
               type SecBuffer byaddr,
               pointer,
               type CtxtHandle byaddr)
        returns (type SECURITY_STATUS byvalue optional)
        limited SEC_ENTRY;

 %dcl ImportSecurityContext char ext;
 %if UNICODE ^= '' %then %do;
 %ImportSecurityContext='ImportSecurityContextW';
 define alias IMPORT_SECURITY_CONTEXT_FN
         type IMPORT_SECURITY_CONTEXT_FN_W;
 %end; %else %do;
 %ImportSecurityContext='ImportSecurityContextA';
 define alias IMPORT_SECURITY_CONTEXT_FN
         type IMPORT_SECURITY_CONTEXT_FN_A;
 %end; /* !UNICODE */


 %if ISSP_MODE = 0 %then %do;
 dcl SecMakeSPN
     entry (type UNICODE_STRING byaddr,
            type UNICODE_STRING byaddr,
            type UNICODE_STRING byaddr optional,
            type USHORT,
            type UNICODE_STRING byaddr optional,
            type UNICODE_STRING byaddr,
            type ULONG byaddr optional,
            type BOOLEAN)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('SecMakeSPN');

 dcl SecMakeSPNEx
     entry (type UNICODE_STRING byaddr,
            type UNICODE_STRING byaddr,
            type UNICODE_STRING byaddr optional,
            type USHORT,
            type UNICODE_STRING byaddr optional,
            type UNICODE_STRING byaddr optional,
            type UNICODE_STRING byaddr,
            type ULONG byaddr optional,
            type BOOLEAN)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('SecMakeSPNEx');

 dcl SecLookupAccountSid
     entry (type PSID,
            type ULONG byaddr,
            type UNICODE_STRING byaddr,
            type ULONG byaddr optional,
            type UNICODE_STRING byaddr optional,
            type SID_NAME_USE byaddr)
     returns (type NTSTATUS byvalue optional)
     SEC_ENTRY external('SecLookupAccountSid');

 dcl SecLookupAccountName
     entry (type UNICODE_STRING byaddr,
            type ULONG byaddr,
            type PSID,
            type SID_NAME_USE byaddr,
            type ULONG byaddr optional,
            type UNICODE_STRING byaddr optional)
     returns (type NTSTATUS byvalue optional)
     SEC_ENTRY external('SecLookupAccountName');

 dcl SecLookupWellKnownSid
     entry (type WELL_KNOWN_SID_TYPE,
            type PSID,
            type ULONG,
            type ULONG byaddr optional)
     returns (type NTSTATUS byvalue optional)
     SEC_ENTRY external('SecLookupWellKnownSid');
 %end;


 /********************************/
 /*                              */
 /*  Fast access for RPC:        */
 /*                              */
 /********************************/

 dcl SECURITY_ENTRYPOINT_ANSIW char value('InitSecurityInterfaceW');
 dcl SECURITY_ENTRYPOINT_ANSIA char value('InitSecurityInterfaceA');
 dcl SECURITY_ENTRYPOINTW SEC_TCHAR value('InitSecurityInterfaceW');
 dcl SECURITY_ENTRYPOINTA SEC_TCHAR value('InitSecurityInterfaceA');


 %if UNICODE ^= '' %then %do;
 dcl SECURITY_ENTRYPOINT SEC_TCHAR value(SECURITY_ENTRYPOINTW);
 dcl SECURITY_ENTRYPOINT_ANSI char value(SECURITY_ENTRYPOINT_ANSIW);
 %end; %else %do;
 dcl SECURITY_ENTRYPOINT SEC_TCHAR value(SECURITY_ENTRYPOINTA);
 dcl SECURITY_ENTRYPOINT_ANSI char value(SECURITY_ENTRYPOINT_ANSIA);
 %end; /* UNICODE */


 %dcl FreeCredentialHandle char ext;
 %FreeCredentialHandle='FreeCredentialsHandle';

 define structure
   1 SECURITY_FUNCTION_TABLE_W,
     2 dwVersion                 fixed bin(32) unsigned,
     2 EnumerateSecurityPackagesW type ENUMERATE_SECURITY_PACKAGES_FN_W,
     2 QueryCredentialsAttributesW
                                 type QUERY_CREDENTIALS_ATTRIBUTES_FN_W,
     2 AcquireCredentialsHandleW  type ACQUIRE_CREDENTIALS_HANDLE_FN_W,
     2 FreeCredentialsHandle      type FREE_CREDENTIALS_HANDLE_FN,
 %if WIN32_CHICAGO = '' %then %do;
     2 Reserved2                  pointer,
 %end; %else %do;
     2 SspiLogonUserW             type SSPI_LOGON_USER_FN,
 %end;
     2 InitializeSecurityContextW type INITIALIZE_SECURITY_CONTEXT_FN_W,
     2 AcceptSecurityContext      type ACCEPT_SECURITY_CONTEXT_FN,
     2 CompleteAuthToken          type COMPLETE_AUTH_TOKEN_FN,
     2 DeleteSecurityContext      type DELETE_SECURITY_CONTEXT_FN,
     2 ApplyControlToken          type APPLY_CONTROL_TOKEN_FN,
     2 QueryContextAttributesW    type QUERY_CONTEXT_ATTRIBUTES_FN_W,
     2 ImpersonateSecurityContext type IMPERSONATE_SECURITY_CONTEXT_FN,
     2 RevertSecurityContext      type REVERT_SECURITY_CONTEXT_FN,
     2 MakeSignature              type MAKE_SIGNATURE_FN,
     2 VerifySignature            type VERIFY_SIGNATURE_FN,
     2 FreeContextBuffer          type FREE_CONTEXT_BUFFER_FN,
     2 QuerySecurityPackageInfoW  type QUERY_SECURITY_PACKAGE_INFO_FN_W,
     2 Reserved3                  pointer,
     2 Reserved4                  pointer,
     2 ExportSecurityContext      type EXPORT_SECURITY_CONTEXT_FN,
     2 ImportSecurityContextW     type IMPORT_SECURITY_CONTEXT_FN_W,
     2 AddCredentialsW            type ADD_CREDENTIALS_FN_W,
     2 Reserved8                  pointer,
     2 QuerySecurityContextToken  type QUERY_SECURITY_CONTEXT_TOKEN_FN,
     2 EncryptMessage             type ENCRYPT_MESSAGE_FN,
     2 DecryptMessage             type DECRYPT_MESSAGE_FN,
     2 SetContextAttributesW      type SET_CONTEXT_ATTRIBUTES_FN_W;
 define alias  SecurityFunctionTableW type   SECURITY_FUNCTION_TABLE_W;
 define alias PSecurityFunctionTableW handle SECURITY_FUNCTION_TABLE_W;

 define structure
   1 SECURITY_FUNCTION_TABLE_A,
     2 dwVersion                  fixed bin(32) unsigned,
     2 EnumerateSecurityPackagesA type ENUMERATE_SECURITY_PACKAGES_FN_A,
     2 QueryCredentialsAttributesA
                                 type QUERY_CREDENTIALS_ATTRIBUTES_FN_A,
     2 AcquireCredentialsHandleA  type ACQUIRE_CREDENTIALS_HANDLE_FN_A,
     2 FreeCredentialHandle       type FREE_CREDENTIALS_HANDLE_FN,
 %if WIN32_CHICAGO = '' %then %do;
     2 Reserved2                  pointer,
 %end; %else %do;
     2 SspiLogonUserA             type SSPI_LOGON_USER_FN,
 %end;
     2 InitializeSecurityContextA type INITIALIZE_SECURITY_CONTEXT_FN_A,
     2 AcceptSecurityContext      type ACCEPT_SECURITY_CONTEXT_FN,
     2 CompleteAuthToken          type COMPLETE_AUTH_TOKEN_FN,
     2 DeleteSecurityContext      type DELETE_SECURITY_CONTEXT_FN,
     2 ApplyControlToken          type APPLY_CONTROL_TOKEN_FN,
     2 QueryContextAttributesA    type QUERY_CONTEXT_ATTRIBUTES_FN_A,
     2 ImpersonateSecurityContext type IMPERSONATE_SECURITY_CONTEXT_FN,
     2 RevertSecurityContext      type REVERT_SECURITY_CONTEXT_FN,
     2 MakeSignature              type MAKE_SIGNATURE_FN,
     2 VerifySignature            type VERIFY_SIGNATURE_FN,
     2 FreeContextBuffer          type FREE_CONTEXT_BUFFER_FN,
     2 QuerySecurityPackageInfoA  type QUERY_SECURITY_PACKAGE_INFO_FN_A,
     2 Reserved3                  pointer,
     2 Reserved4                  pointer,
     2 ExportSecurityContext      type EXPORT_SECURITY_CONTEXT_FN,
     2 ImportSecurityContextA     type IMPORT_SECURITY_CONTEXT_FN_A,
     2 AddCredentialsA            type ADD_CREDENTIALS_FN_A,
     2 Reserved8                  pointer,
     2 QuerySecurityContextToken  type QUERY_SECURITY_CONTEXT_TOKEN_FN,
     2 EncryptMessage             type ENCRYPT_MESSAGE_FN,
     2 DecryptMessage             type DECRYPT_MESSAGE_FN,
     2 SetContextAttributesW      type SET_CONTEXT_ATTRIBUTES_FN_A;
 define alias  SecurityFunctionTableA type   SECURITY_FUNCTION_TABLE_A;
 define alias PSecurityFunctionTableA handle SECURITY_FUNCTION_TABLE_A;

 %if UNICODE ^= '' %then %do;
 define alias  SecurityFunctionTable type  SecurityFunctionTableW;
 define alias PSecurityFunctionTable type PSecurityFunctionTableW;
 %end; %else %do;
 define alias  SecurityFunctionTable type  SecurityFunctionTableA;
 define alias PSecurityFunctionTable type PSecurityFunctionTableA;
 %end; /* !UNICODE */

 /* Function table has all routines through DecryptMessage */
 dcl SECURITY_SUPPORT_PROVIDER_INTERFACE_VERSION
     fixed bin(32) unsigned value (1);
 /* Function table has all routines through SetContextAttributes */
 dcl SECURITY_SUPPORT_PROVIDER_INTERFACE_VERSION_2
     fixed bin(32) unsigned value(2);


 dcl InitSecurityInterfaceA
     entry ()
     returns (type PSecurityFunctionTableA byvalue optional)
     SEC_ENTRY external('InitSecurityInterfaceA');

 define alias INIT_SECURITY_INTERFACE_A
        entry ()
        returns (type PSecurityFunctionTableA byvalue optional)
        limited SEC_ENTRY;

 dcl InitSecurityInterfaceW
     entry ()
     returns (type PSecurityFunctionTableW byvalue optional)
     SEC_ENTRY external('InitSecurityInterfaceW');

 define alias INIT_SECURITY_INTERFACE_W
        entry ()
        returns (type PSecurityFunctionTableW byvalue optional)
        limited SEC_ENTRY;

 %dcl InitSecurityInterface char ext;
 %if UNICODE ^= '' %then %do;
 %InitSecurityInterface='InitSecurityInterfaceW';
 define alias INIT_SECURITY_INTERFACE type INIT_SECURITY_INTERFACE_W;
 %end; %else %do;
 %InitSecurityInterface='InitSecurityInterfaceA';
 define alias INIT_SECURITY_INTERFACE type INIT_SECURITY_INTERFACE_A;
 %end; /* !UNICODE */


 /* SASL Profile Support */


 dcl SaslEnumerateProfilesA
     entry (type LPSTR byaddr,
            type ULONG byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SaslEnumerateProfilesA');

 dcl SaslEnumerateProfilesW
     entry (type LPWSTR byaddr,
            type ULONG byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SaslEnumerateProfilesW');

 %dcl SaslEnumerateProfiles char ext;
 %if UNICODE ^= '' %then %do;
   %SaslEnumerateProfiles='SaslEnumerateProfilesW';
 %end; %else %do;
   %SaslEnumerateProfiles='SaslEnumerateProfilesA';
 %end;


 dcl SaslGetProfilePackageA
     entry (char (*) varz byaddr,
            type PSecPkgInfoA byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SaslGetProfilePackageA');

 dcl SaslGetProfilePackageW
     entry (Wchar (*) varz byaddr,
            type PSecPkgInfoW byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SaslGetProfilePackageW');

 %dcl SaslGetProfilePackage char ext;
 %if UNICODE ^= '' %then %do;
   %SaslGetProfilePackage='SaslGetProfilePackageW';
 %end; %else %do;
   %SaslGetProfilePackage='SaslGetProfilePackageA';
 %end;

 dcl SaslIdentifyPackageA
     entry (type PSecBufferDesc byaddr,
            type PSecPkgInfoA byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SaslIdentifyPackageA');

 dcl SaslIdentifyPackageW
     entry (type PSecBufferDesc byaddr,
            type PSecPkgInfoW byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SaslIdentifyPackageW');

 %dcl SaslIdentifyPackage char ext;
 %if UNICODE ^= '' %then %do;
   %SaslIdentifyPackage='SaslIdentifyPackageW';
 %end; %else %do;
   %SaslIdentifyPackage='SaslIdentifyPackageA';
 %end;

 dcl SaslInitializeSecurityContextW
     entry (type CredHandle byaddr,       /* Cred to base context */
            type CtxtHandle byaddr,       /* Existing context (OPT) */
            WCHAR (*) varz byaddr,        /* Name of target */
            fixed bin(32) unsigned,       /* Context Requirements */
            fixed bin(32) unsigned,       /* Reserved, MBZ */
            fixed bin(32) unsigned,       /* Data rep of target */
            type PSecBufferDesc byaddr,   /* Input Buffers */
            fixed bin(32) unsigned,       /* Reserved, MBZ */
            type CtxtHandle byaddr,       /* (out) New Context handle */
            type PSecBufferDesc byaddr,   /* (inout) Output Buffers */
            fixed bin(32) unsigned byaddr,/* (out) Context attrs */
            type TimeStamp byaddr)        /* (out) Life span (OPT) */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SaslInitializeSecurityContextW');

 dcl SaslInitializeSecurityContextA
     entry (type CredHandle byaddr,       /* Cred to base context */
            type CtxtHandle byaddr,       /* Existing context (OPT) */
            char (*) varz byaddr,         /* Name of target */
            fixed bin(32) unsigned,       /* Context Requirements */
            fixed bin(32) unsigned,       /* Reserved, MBZ */
            fixed bin(32) unsigned,       /* Data rep of target */
            type PSecBufferDesc byaddr,   /* Input Buffers */
            fixed bin(32) unsigned,       /* Reserved, MBZ */
            type CtxtHandle byaddr,       /* (out) New Context handle */
            type PSecBufferDesc byaddr,   /* (inout) Output Buffers */
            fixed bin(32) unsigned byaddr,/* (out) Context attrs */
            type TimeStamp byaddr)        /* (out) Life span (OPT) */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SaslInitializeSecurityContextA');


 %dcl SaslInitializeSecurityContext char ext;
 %if UNICODE ^= '' %then %do;
   %SaslInitializeSecurityContext='SaslInitializeSecurityContextW';
 %end; %else %do;
   %SaslInitializeSecurityContext='SaslInitializeSecurityContextA';
 %end;


 dcl SaslAcceptSecurityContext
     entry (type CredHandle byaddr,       /* Cred to base context */
            type CtxtHandle byaddr,       /* Existing context (OPT) */
            type PSecBufferDesc byaddr,   /* Input buffer */
            fixed bin(32) unsigned,       /* Context Requirements */
            fixed bin(32) unsigned,       /* Target Data Rep */
            type CtxtHandle byaddr,       /* (out) New context handle */
            type PSecBufferDesc byaddr,   /* (inout) Output buffers */
            pointer,                      /* (out) Context attributes */
            type TimeStamp byaddr)        /* (out) Life span (OPT) */
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SaslAcceptSecurityContext');

 dcl SASL_OPTION_SEND_SIZE        type ULONG value (1);
     /* Maximum size to send to peer */
 dcl SASL_OPTION_RECV_SIZE        type ULONG value (2);
     /* Maximum size willing to receive */
 dcl SASL_OPTION_AUTHZ_STRING     type ULONG value (3);
     /* Authorization string */
 dcl SASL_OPTION_AUTHZ_PROCESSING type ULONG value (4);
     /* Authorization string processing */

 define ordinal
   SASL_AUTHZID_STATE
     ( Sasl_AuthZIDForbidden,  /* allow no AuthZID strings to be  */
                               /* specified - error out (default) */
       Sasl_AuthZIDProcessed   /* AuthZID Strings processed by    */
                               /* Application or SSP              */
     ) prec (31);

 dcl SaslSetContextOption
    entry (type CtxtHandle byaddr,
           type ULONG,
           pointer,
           type ULONG)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SaslSetContextOption');


 dcl SaslGetContextOption
     entry (type CtxtHandle byaddr,
           type ULONG,
           pointer,
           type ULONG,
           type ULONG byaddr optional)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('SaslGetContextOption');

 /* This is the combined authentication identity structure that may   */
 /* be used with the negotiate package, NTLM, Kerberos, or SCHANNEL   */

 %dcl #SEC_WINNT_AUTH_IDENTITY_VERSION char ext;
 %if #SEC_WINNT_AUTH_IDENTITY_VERSION = '' %then %do;
 %#SEC_WINNT_AUTH_IDENTITY_VERSION = 'Y';
 dcl SEC_WINNT_AUTH_IDENTITY_VERSION fixed bin(31) value('200'xn);

 define structure
   1 SEC_WINNT_AUTH_IDENTITY_EXW,
     2 Version           fixed bin(32) unsigned,
     2 Length            fixed bin(32) unsigned,
     2 User              pointer,
     2 UserLength        fixed bin(32) unsigned,
     2 Domain            pointer,
     2 DomainLength      fixed bin(32) unsigned,
     2 Password          pointer,
     2 PasswordLength    fixed bin(32) unsigned,
     2 Flags             fixed bin(32) unsigned,
     2 PackageList       pointer,
     2 PackageListLength fixed bin(32) unsigned;
 define alias PSEC_WINNT_AUTH_IDENTITY_EXW
        handle SEC_WINNT_AUTH_IDENTITY_EXW;


 define structure
   1 SEC_WINNT_AUTH_IDENTITY_EXA,
     2 Version           fixed bin(32) unsigned,
     2 Length            fixed bin(32) unsigned,
     2 User              pointer,
     2 UserLength        fixed bin(32) unsigned,
     2 Domain            pointer,
     2 DomainLength      fixed bin(32) unsigned,
     2 Password          pointer,
     2 PasswordLength    fixed bin(32) unsigned,
     2 Flags             fixed bin(32) unsigned,
     2 PackageList       pointer,
     2 PackageListLength fixed bin(32) unsigned;
 define alias PSEC_WINNT_AUTH_IDENTITY_EXA
        handle SEC_WINNT_AUTH_IDENTITY_EXA;

 %if UNICODE ^= '' %then %do;
 define alias  SEC_WINNT_AUTH_IDENTITY_EX
         type  SEC_WINNT_AUTH_IDENTITY_EXW;
 define alias PSEC_WINNT_AUTH_IDENTITY_EX
         type PSEC_WINNT_AUTH_IDENTITY_EXW;
 %end; %else %do;
 define alias  SEC_WINNT_AUTH_IDENTITY_EX
         type  SEC_WINNT_AUTH_IDENTITY_EXA;
 define alias PSEC_WINNT_AUTH_IDENTITY_EX
         type PSEC_WINNT_AUTH_IDENTITY_EXA;
 %end;

 %end; /* #SEC_WINNT_AUTH_IDENTITY_VERSION */


 /* Common types used by negotiable security packages */

 dcl SEC_WINNT_AUTH_IDENTITY_MARSHALLED fixed bin(32) unsigned
                                        value('4'xu);
 /* all data is in one buffer */
 dcl SEC_WINNT_AUTH_IDENTITY_ONLY       fixed bin(32) unsigned
                                        value('8'xu);
 /* these credentials are for identity only - no PAC needed */


 /* Routines for manipulating packages */

 define structure
   1 SECURITY_PACKAGE_OPTIONS,
    2 Size          fixed bin(32) unsigned,
    2 Type          fixed bin(32) unsigned,
    2 Flags         fixed bin(32) unsigned,
    2 SignatureSize fixed bin(32) unsigned,
    2 Signature     pointer;
 define alias PSECURITY_PACKAGE_OPTIONS handle SECURITY_PACKAGE_OPTIONS;

 dcl SECPKG_OPTIONS_TYPE_UNKNOWN fixed bin(32) unsigned value(0);
 dcl SECPKG_OPTIONS_TYPE_LSA     fixed bin(32) unsigned value(1);
 dcl SECPKG_OPTIONS_TYPE_SSPI    fixed bin(32) unsigned value(2);

 dcl SECPKG_OPTIONS_PERMANENT    fixed bin(32) unsigned
                                 value('00000001'xu);

 dcl AddSecurityPackageA
     entry (SEC_CHAR (*) varz byaddr,
            type SECURITY_PACKAGE_OPTIONS byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('AddSecurityPackageA');

 dcl AddSecurityPackageW
     entry (SEC_WCHAR (*) varz byaddr,
            type SECURITY_PACKAGE_OPTIONS byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('AddSecurityPackageW');

 %dcl AddSecurityPackage char ext;
 %if UNICODE ^= '' %then %do;
   %AddSecurityPackage='AddSecurityPackageW';
 %end; %else %do;
   %AddSecurityPackage='AddSecurityPackageA';
 %end;

 dcl DeleteSecurityPackageA
     entry (SEC_CHAR(*) varz byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('DeleteSecurityPackageA');

 dcl DeleteSecurityPackageW
     entry (SEC_WCHAR(*) varz byaddr)
     returns (type SECURITY_STATUS byvalue optional)
     SEC_ENTRY external('DeleteSecurityPackageW');

 %dcl DeleteSecurityPackage char ext;
 %if UNICODE ^= '' %then %do;
   %DeleteSecurityPackage='DeleteSecurityPackageW';
 %end; %else %do;
   %DeleteSecurityPackage='DeleteSecurityPackageA';
 %end;

 %end; /* __SSPI_H__ */
