 /*********************************************************************/
 /*                                                                   */
 /* Copyright (c) 1995-1999  Microsoft Corporation                    */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*     wininet.cpy                                                   */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*     Contains manifests, macros, types and prototypes for          */
 /*     Microsoft Windows Internet Extensions                         */
 /*                                                                   */
 /*********************************************************************/

 %dcl #WININET_ char ext;
 %if #WININET_ = '' %then %do;
 %#WININET_ = 'Y';

 %dcl UNICODE char ext;
 %dcl #WINX32_ char ext; /* set to 'Y' only if IE >= 5 guaranteed */

 %dcl #WINBASE_ char ext;
 %if #WINBASE_ = '' %then %do;
 %include winbase;
 %end;

 %dcl INTERNETAPI char ext;
 %INTERNETAPI = '';
 %dcl URLCACHEAPI char ext;
 %URLCACHEAPI = '';

 %dcl BOOLAPI char ext;
 %BOOLAPI = 'returns (byvalue optional type BOOL) INTERNETAPI WINAPI';

 /* internet types */

 define alias   HINTERNET     type LPVOID;
 define alias LPHINTERNET     pointer; /* to HINTERNET */

 define alias   INTERNET_PORT type WORD;
 define alias LPINTERNET_PORT pointer; /* to INTERNET_PORT */

 /* Internet APIs */

 /* manifests */

 dcl INTERNET_INVALID_PORT_NUMBER    type INTERNET_PORT value(0),
     /* use the protocol-specific default */
     INTERNET_DEFAULT_FTP_PORT       type INTERNET_PORT value(21),
     /* default for FTP servers */
     INTERNET_DEFAULT_GOPHER_PORT    type INTERNET_PORT value(70),
     /*    "     "  gopher " */
     INTERNET_DEFAULT_HTTP_PORT      type INTERNET_PORT value(80),
     /*    "     "  HTTP   " */
     INTERNET_DEFAULT_HTTPS_PORT     type INTERNET_PORT value(443),
     /*    "     "  HTTPS  " */
     INTERNET_DEFAULT_SOCKS_PORT     type INTERNET_PORT value(1080);

 /* maximum field lengths (arbitrary) */
 dcl INTERNET_MAX_HOST_NAME_LENGTH   fixed bin(31) value(256),
     INTERNET_MAX_USER_NAME_LENGTH   fixed bin(31) value(128),
     INTERNET_MAX_PASSWORD_LENGTH    fixed bin(31) value(128),
     INTERNET_MAX_PORT_NUMBER_LENGTH fixed bin(31) value(5),
              /* INTERNET_PORT is unsigned short */
     INTERNET_MAX_PORT_NUMBER_VALUE  fixed bin(31) value(65535),
              /* maximum unsigned short value */
     INTERNET_MAX_PATH_LENGTH        fixed bin(31) value(2048),
     INTERNET_MAX_SCHEME_LENGTH      fixed bin(31) value(32),
              /* longest protocol name */
     INTERNET_MAX_URL_LENGTH         fixed bin (31)
     value (INTERNET_MAX_SCHEME_LENGTH +
            3 /* length ('://') */ + 1 +
            INTERNET_MAX_PATH_LENGTH);

 /* values returned by InternetQueryOption() with */
 /* INTERNET_OPTION_KEEP_CONNECTION:                     */

 dcl INTERNET_KEEP_ALIVE_UNKNOWN  type DWORD
                                  value(iunsigned(bin(-1,31))),
     INTERNET_KEEP_ALIVE_ENABLED  type DWORD value(1),
     INTERNET_KEEP_ALIVE_DISABLED type DWORD value(0);

 /* flags returned by InternetQueryOption() with */
 /* INTERNET_OPTION_REQUEST_FLAGS                       */

 dcl INTERNET_REQFLAG_FROM_CACHE           type DWORD
       value('00000001'xu), /* request came from cache */
     INTERNET_REQFLAG_ASYNC                type DWORD
       value('00000002'xu), /* request was made asynchronously */
     INTERNET_REQFLAG_VIA_PROXY            type DWORD
       value('00000004'xu), /* request was made via a proxy */
     INTERNET_REQFLAG_NO_HEADERS           type DWORD
       value('00000008'xu), /* response contained no headers */
     INTERNET_REQFLAG_PASSIVE              type DWORD
       value('00000010'xu), /* FTP: passive-mode connection */
     INTERNET_REQFLAG_CACHE_WRITE_DISABLED type DWORD
       value('00000040'xu), /* HTTPS: this request not cacheable */
     INTERNET_REQFLAG_NET_TIMEOUT          type DWORD
       value('00000080'xu);  /* w/ _FROM_CACHE: net request timed out */

 /* flags common to open functions (not InternetOpen()): */

 dcl INTERNET_FLAG_RELOAD            type DWORD value('80000000'xu);
                            /* retrieve the original item */

 /* flags for InternetOpenUrl(): */

 dcl INTERNET_FLAG_RAW_DATA          type DWORD value('40000000'xu),
        /* receive the item as raw data */
     INTERNET_FLAG_EXISTING_CONNECT  type DWORD value('20000000'xu);
        /* do not create new connection object */

 /* flags for InternetOpen(): */

 dcl INTERNET_FLAG_ASYNC             type DWORD value('10000000'xu);
     /* this request is asynchronous (where supported) */

 /* protocol-specific flags: */

 dcl INTERNET_FLAG_PASSIVE           type DWORD value('08000000'xu);
     /* used for FTP connections */

 /* additional cache flags */

 dcl INTERNET_FLAG_NO_CACHE_WRITE    type DWORD value('04000000'xu),
              /* don't write this item to the cache */
     INTERNET_FLAG_DONT_CACHE        type DWORD
              value(INTERNET_FLAG_NO_CACHE_WRITE),
     INTERNET_FLAG_MAKE_PERSISTENT   type DWORD value('02000000'xn),
              /* make this item persistent in cache */
     INTERNET_FLAG_FROM_CACHE        type DWORD value('01000000'xu),
              /* use offline semantics */
     INTERNET_FLAG_OFFLINE           type DWORD
              value(INTERNET_FLAG_FROM_CACHE),

 /* additional flags */
     INTERNET_FLAG_SECURE            type DWORD value('00800000'xu),
        /* use PCT/SSL if applicable (HTTP) */
     INTERNET_FLAG_KEEP_CONNECTION   type DWORD value('00400000'xu),
        /* use keep-alive semantics */
     INTERNET_FLAG_NO_AUTO_REDIRECT  type DWORD value('00200000'xu),
        /* don't handle redirections automatically */
     INTERNET_FLAG_READ_PREFETCH     type DWORD value('00100000'xu),
        /* do background read prefetch */
     INTERNET_FLAG_NO_COOKIES        type DWORD value('00080000'xu),
        /* no automatic cookie handling */
     INTERNET_FLAG_NO_AUTH           type DWORD value('00040000'xu),
        /* no automatic authentication handling */
     INTERNET_FLAG_CACHE_IF_NET_FAIL type DWORD value('00010000'xu),
        /* return cache file if net request fails */

 /* Security Ignore Flags, Allow HttpOpenRequest to overide   */
 /* Secure Channel (SSL/PCT) failures of the following types. */
     INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP  type DWORD
          value('00008000'xu), /* ex: https:/... to http:/... */
     INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS type DWORD
          value('00004000'xu), /* ex: http:/... to https:/... */
     INTERNET_FLAG_IGNORE_CERT_DATE_INVALID type DWORD
          value('00002000'xu), /* expired X509 Cert. */
     INTERNET_FLAG_IGNORE_CERT_CN_INVALID   type DWORD
          value('00001000'xu), /* bad common name in X509 Cert. */

 /* more caching flags */
     INTERNET_FLAG_RESYNCHRONIZE   type DWORD value('00000800'xu),
        /* asking wininet to update an item if it is newer */
     INTERNET_FLAG_HYPERLINK       type DWORD value('00000400'xu),
        /* asking wininet to do */
        /* hyperlinking semantic which works right for scripts */
     INTERNET_FLAG_NO_UI           type DWORD value('00000200'xu),
     INTERNET_FLAG_PRAGMA_NOCACHE  type DWORD value('00000100'xu),
       /* asking wininet to add "pragma: no-cache" */
     INTERNET_FLAG_CACHE_ASYNC     type DWORD value('00000080'xu),
       /* ok to perform lazy cache-write */
     INTERNET_FLAG_FORMS_SUBMIT    type DWORD value('00000040'xu),
       /* this is a forms submit */
     INTERNET_FLAG_FWD_BACK        type DWORD value('00000020'xu),
       /* fwd-back button op */
     INTERNET_FLAG_NEED_FILE       type DWORD value('00000010'xu),
       /* need a file for this request */
     INTERNET_FLAG_MUST_CACHE_REQUEST type DWORD
       value (INTERNET_FLAG_NEED_FILE),

 /* PL/I: dependencies inverted (dcl before use!) */
     INTERNET_FLAG_TRANSFER_ASCII  type DWORD value('00000001'xu),
     INTERNET_FLAG_TRANSFER_BINARY type DWORD value('00000002'xu);

 /* flags field masks */

 dcl SECURITY_INTERNET_MASK type DWORD
     value (ior (INTERNET_FLAG_IGNORE_CERT_CN_INVALID,
                 INTERNET_FLAG_IGNORE_CERT_DATE_INVALID,
                 INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS,
                 INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP));


 dcl INET_FLAGS_MASK type DWORD
     value (ior (INTERNET_FLAG_RELOAD,
                 INTERNET_FLAG_RAW_DATA,
                 INTERNET_FLAG_EXISTING_CONNECT,
                 INTERNET_FLAG_ASYNC,
                 INTERNET_FLAG_PASSIVE,
                 INTERNET_FLAG_NO_CACHE_WRITE,
                 INTERNET_FLAG_MAKE_PERSISTENT,
                 INTERNET_FLAG_FROM_CACHE,
                 INTERNET_FLAG_SECURE,
                 INTERNET_FLAG_KEEP_CONNECTION,
                 INTERNET_FLAG_NO_AUTO_REDIRECT,
                 INTERNET_FLAG_READ_PREFETCH,
                 INTERNET_FLAG_NO_COOKIES,
                 INTERNET_FLAG_NO_AUTH,
                 SECURITY_INTERNET_MASK,
                 INTERNET_FLAG_RESYNCHRONIZE,
                 INTERNET_FLAG_HYPERLINK,
                 INTERNET_FLAG_NO_UI,
                 INTERNET_FLAG_PRAGMA_NOCACHE,
                 INTERNET_FLAG_CACHE_ASYNC,
                 INTERNET_FLAG_FORMS_SUBMIT,
                 INTERNET_FLAG_NEED_FILE,
                 INTERNET_FLAG_TRANSFER_ASCII,
                 INTERNET_FLAG_TRANSFER_BINARY,
                 INTERNET_FLAG_FWD_BACK
                 /*, INTERNET_FLAG_BGUPDATE */ ));

 dcl INTERNET_ERROR_MASK_INSERT_CDROM      type DWORD value('1'xu),
     INTERNET_ERROR_MASK_COMBINED_SEC_CERT type DWORD value('2'xu),
     INTERNET_ERROR_MASK_NEED_MSN_SSPI_PKG type DWORD value('4'xu),
     INTERNET_ERROR_MASK_LOGIN_FAILURE_DISPLAY_ENTITY_BODY
                                           type DWORD value('8'xu);

 dcl INTERNET_OPTIONS_MASK type DWORD value (inot(INET_FLAGS_MASK));

 /* common per-API flags (new APIs) */

 dcl WININET_API_FLAG_ASYNC       type DWORD value ('00000001'xu),
       /* force async operation */
     WININET_API_FLAG_SYNC        type DWORD value ('00000004'xu),
       /* force sync operation */
     WININET_API_FLAG_USE_CONTEXT type DWORD value ('00000008'xu);
       /* use value supplied in dwContext (even if 0) */
 /* INTERNET_NO_CALLBACK - if this value is presented as the          */
 /* dwContext parameter, then no call-backs will be made for that API */

 dcl INTERNET_NO_CALLBACK         type DWORD value(0);

 /* structures/types */

 /* INTERNET_SCHEME - enumerated URL scheme type */

 define ordinal
      INTERNET_SCHEME
       ( INTERNET_SCHEME_PARTIAL value(-2),
         INTERNET_SCHEME_UNKNOWN value(-1),
         INTERNET_SCHEME_DEFAULT value(0),
         INTERNET_SCHEME_FTP,
         INTERNET_SCHEME_GOPHER,
         INTERNET_SCHEME_HTTP,
         INTERNET_SCHEME_HTTPS,
         INTERNET_SCHEME_FILE,
         INTERNET_SCHEME_NEWS,
         INTERNET_SCHEME_MAILTO,
         INTERNET_SCHEME_SOCKS,
         INTERNET_SCHEME_JAVASCRIPT,
         INTERNET_SCHEME_VBSCRIPT
       ) prec(31);
 dcl ( INTERNET_SCHEME_FIRST value(INTERNET_SCHEME_FTP)
     , INTERNET_SCHEME_LAST  value(INTERNET_SCHEME_VBSCRIPT)
     ) type INTERNET_SCHEME;
 define alias LPINTERNET_SCHEME pointer; /* to INTERNET_SCHEME */


 /* INTERNET_ASYNC_RESULT - this structure is returned to the app-    */
 /* lication via the callback with INTERNET_STATUS_REQUEST_COMPLETE.  */
 /* It is not sufficient to just return the result of the async       */
 /* operation. If the API failed then the app cannot call             */
 /* GetLastError() because the thread context will be incorrect.      */
 /* Both the value returned by the async API and any resultant error  */
 /* code are made available.                                          */
 /* The app need not check dwError if dwResult indicates that the API */
 /* succeeded (in this case dwError will be ERROR_SUCCESS)            */

 define structure
   1 INTERNET_ASYNC_RESULT,
     2 dwResult type DWORD,
       /* the HINTERNET, DWORD or BOOL return code from an async API */
     2 dwError  type DWORD;
       /* the error code if the API failed */
 define alias LPINTERNET_ASYNC_RESULT handle INTERNET_ASYNC_RESULT;

 /* INTERNET_PROXY_INFO - structure supplied with             */
 /* INTERNET_OPTION_PROXY to get / set proxy information on a */
 /* InternetOpen() handle                                     */

 define structure
   1 INTERNET_PROXY_INFO,
     2 dwAccessType    type DWORD,
       /* INTERNET_OPEN_TYPE_DIRECT, INTERNET_OPEN_TYPE_PROXY, */
       /* or INTERNET_OPEN_TYPE_PRECONFIG (set only) */
     2 lpszProxy       type LPCTSTR,
       /* proxy server list */
     2 lpszProxyBypass type LPCTSTR;
       /* proxy bypass list */
 define alias LPINTERNET_PROXY_INFO handle INTERNET_PROXY_INFO;

 /* INTERNET_PER_CONN_OPTION_LIST - set per-connection options such   */
 /* as proxy and autoconfig info                                      */

 /* Set and queried using Internet[Set|Query]Option with              */
 /* INTERNET_OPTION_PER_CONNECTION_OPTION                             */


 define structure
   1 INTERNET_PER_CONN_OPTIONA,
     2 dwOption   type DWORD,   /* option to be queried or set */
     2 Value union,
       3 dwValue  type DWORD,    /* dword value for the option */
       3 pszValue type LPSTR,
                       /* pointer to string value for the option */
       3 ftValue  type FILETIME; /* file-time value for the option */

 define alias LPINTERNET_PER_CONN_OPTIONA
        handle  INTERNET_PER_CONN_OPTIONA;

 define structure
   1 INTERNET_PER_CONN_OPTIONW,
     2 dwOption   type DWORD,   /* option to be queried or set */
     2 Value union,
       3 dwValue  type DWORD,    /* dword value for the option */
       3 pszValue type LPWSTR,
                       /* pointer to string value for the option */
       3 ftValue  type FILETIME; /* file-time value for the option */
 define alias LPINTERNET_PER_CONN_OPTIONW
        handle  INTERNET_PER_CONN_OPTIONW;

 %if UNICODE ^= '' %then %do;
   define alias   INTERNET_PER_CONN_OPTION
           type   INTERNET_PER_CONN_OPTIONW;
   define alias LPINTERNET_PER_CONN_OPTION
           type LPINTERNET_PER_CONN_OPTIONW;
 %end; %else %do;
   define alias   INTERNET_PER_CONN_OPTION
           type   INTERNET_PER_CONN_OPTIONA;
   define alias LPINTERNET_PER_CONN_OPTION
           type LPINTERNET_PER_CONN_OPTIONA;
 %end; /* UNICODE */

 define structure
   1 INTERNET_PER_CONN_OPTION_LISTA,
     2 dwSize        type DWORD,
       /* size of the INTERNET_PER_CONN_OPTION_LIST struct */
     2 pszConnection type LPSTR,
       /* connection name to set/query options */
     2 dwOptionCount type DWORD,
       /* number of options to set/query */
     2 dwOptionError type DWORD,
       /* on error, which option failed */
     2 pOptions     type LPINTERNET_PER_CONN_OPTIONA;
       /* array of options to set/query */
 define alias LPINTERNET_PER_CONN_OPTION_LISTA
        handle  INTERNET_PER_CONN_OPTION_LISTA;

 define structure
   1 INTERNET_PER_CONN_OPTION_LISTW,
     2 dwSize        type DWORD,
       /* size of the INTERNET_PER_CONN_OPTION_LIST struct */
     2 pszConnection type LPWSTR,
       /* connection name to set/query options */
     2 dwOptionCount type DWORD,
       /* number of options to set/query */
     2 dwOptionError type DWORD,
       /* on error, which option failed */
     2 pOptions     type LPINTERNET_PER_CONN_OPTIONW;
       /* array of options to set/query */
 define alias LPINTERNET_PER_CONN_OPTION_LISTW
        handle  INTERNET_PER_CONN_OPTION_LISTW;

 %if UNICODE ^= '' %then %do;
   define alias   INTERNET_PER_CONN_OPTION_LIST
           type   INTERNET_PER_CONN_OPTION_LISTW;
   define alias LPINTERNET_PER_CONN_OPTION_LIST
           type LPINTERNET_PER_CONN_OPTION_LISTW;
 %end; %else %do;
   define alias   INTERNET_PER_CONN_OPTION_LIST
           type   INTERNET_PER_CONN_OPTION_LISTA;
   define alias LPINTERNET_PER_CONN_OPTION_LIST
           type LPINTERNET_PER_CONN_OPTION_LISTA;
 %end; /* UNICODE */


 /* Options used in INTERNET_PER_CONN_OPTON struct */

 dcl INTERNET_PER_CONN_FLAGS                type DWORD value(1);
 dcl INTERNET_PER_CONN_PROXY_SERVER         type DWORD value(2);
 dcl INTERNET_PER_CONN_PROXY_BYPASS         type DWORD value(3);
 dcl INTERNET_PER_CONN_AUTOCONFIG_URL       type DWORD value(4);
 dcl INTERNET_PER_CONN_AUTODISCOVERY_FLAGS  type DWORD value(5);


 /* PER_CONN_FLAGS */

 dcl PROXY_TYPE_DIRECT                   type DWORD value('00000001'xu);
     /* direct to net */
 dcl PROXY_TYPE_PROXY                    type DWORD value('00000002'xu);
     /* via named proxy */
 dcl PROXY_TYPE_AUTO_PROXY_URL           type DWORD value('00000004'xu);
     /* autoproxy URL */
 dcl PROXY_TYPE_AUTO_DETECT              type DWORD value('00000008'xu);
     /* use autoproxy detection */


 /* PER_CONN_AUTODISCOVERY_FLAGS */

 dcl AUTO_PROXY_FLAG_USER_SET            type DWORD value('00000001'xu);
     /* user changed this setting */
 dcl AUTO_PROXY_FLAG_ALWAYS_DETECT       type DWORD value('00000002'xu);
     /* force detection even when its not needed */
 dcl AUTO_PROXY_FLAG_DETECTION_RUN       type DWORD value('00000004'xu);
     /* detection has been run */
 dcl AUTO_PROXY_FLAG_MIGRATED            type DWORD value('00000008'xu);
     /* migration has just been done  */
 dcl AUTO_PROXY_FLAG_DONT_CACHE_PROXY_RESULT
                                         type DWORD value('00000010'xu);
     /* don't cache result of host=proxy name */
 dcl AUTO_PROXY_FLAG_CACHE_INIT_RUN      type DWORD value('00000020'xu);
     /* don't initalize and run unless URL expired */
 dcl AUTO_PROXY_FLAG_DETECTION_SUSPECT   type DWORD value('00000040'xu);
     /* if we're on a LAN & Modem, with only one IP, bad?!? */


 /* INTERNET_VERSION_INFO - version information returned via */
 /* InternetQueryOption(..., INTERNET_OPTION_VERSION, ...)          */

 define structure
   1 INTERNET_VERSION_INFO,
       2 dwMajorVersion type DWORD,
       2 dwMinorVersion type DWORD;
 define alias LPINTERNET_VERSION_INFO handle INTERNET_VERSION_INFO;

 /* HTTP_VERSION_INFO - query or set global HTTP version (1.0 or 1.1) */

 define structure
   1 HTTP_VERSION_INFO,
       2 dwMajorVersion type DWORD,
       2 dwMinorVersion type DWORD;
 define alias LPHTTP_VERSION_INFO handle HTTP_VERSION_INFO;

 /* INTERNET_CONNECTED_INFO - information used to set the global */
 /* connected state */

 define structure
   1 INTERNET_CONNECTED_INFO,
     3 dwConnectedState type DWORD,
       /* dwConnectedState - new connected/disconnected state. */
       /* See INTERNET_STATE_CONNECTED, etc.                   */
     3 dwFlags type DWORD ;
       /* dwFlags - flags controlling connected->disconnected  */
       /* (or disconnected->connected) transition. See below   */

 define alias LPINTERNET_CONNECTED_INFO handle INTERNET_CONNECTED_INFO;

 /* flags for INTERNET_CONNECTED_INFO dwFlags */

 /* ISO_FORCE_DISCONNECTED - if set when putting Wininet into   */
 /* disconnected mode, all outstanding requests will be aborted */
 /* with a cancelled error                                      */

 dcl ISO_FORCE_DISCONNECTED  fixed bin (31) value ('00000001'xn);

 /* URL_COMPONENTS - the constituent parts of an URL.                 */
 /* Used in InternetCrackUrl() and InternetCreateUrl()                */
 /*                                                                   */
 /* For InternetCrackUrl(), if a pointer field and its corresponding  */
 /* length field are both 0 then that component is not returned.      */
 /* If the pointer field is NULL but the length field is not zero,    */
 /* then both the pointer and length fields are returned.             */
 /* If both pointer and corresponding length fields are non-zero then */
 /* the pointer field points to a buffer where the component is       */
 /* copied. The component may be un-escaped, depending on dwFlags.    */
 /*                                                                   */
 /* For InternetCreateUrl(), the pointer fields should be NULL if the */
 /* component is not required. If the corresponding length field is   */
 /* zero then the pointer field is the address of a zero-terminated   */
 /* string. If the length field is not zero then it is the string     */
 /* length of the corresponding pointer field                         */

 define structure
   1 URL_COMPONENTSA,
     2 dwStructSize     type DWORD,
           /* size of this structure. Used in version check */
     2 lpszScheme       type LPSTR, /* pointer to scheme name */
     2 dwSchemeLength   type DWORD, /* length of scheme name */
     2 nScheme          type INTERNET_SCHEME,
                        /* enumerated scheme type (if known) */
     2 lpszHostName     type LPSTR, /* pointer to host name */
     2 dwHostNameLength type DWORD, /* length of host name */
     2 nPort            type INTERNET_PORT, /* converted port number */
     2 lpszUserName     type LPSTR, /* pointer to user name */
     2 dwUserNameLength type DWORD, /* length of user name */
     2 lpszPassword     type LPSTR, /* pointer to password */
     2 dwPasswordLength type DWORD, /* length of password */
     2 lpszUrlPath      type LPSTR, /* pointer to URL-path */
     2 dwUrlPathLength  type DWORD, /* length of URL-path */
     2 lpszExtraInfo    type LPSTR,
           /* pointer to extra information (e.g. ?foo or #foo) */
     2 dwExtraInfoLength type DWORD;/* length of extra information */
 define alias LPURL_COMPONENTSA handle URL_COMPONENTSA;

 define structure
   1 URL_COMPONENTSW,
     2 dwStructSize      type DWORD,
           /* size of this structure. Used in version check */
     2 lpszScheme        type LPWSTR,/* pointer to scheme name */
     2 dwSchemeLength    type DWORD, /* length of scheme name */
     2 nScheme           type INTERNET_SCHEME,
                         /* enumerated scheme type (if known) */
     2 lpszHostName      type LPWSTR,/* pointer to host name */
     2 dwHostNameLength  type DWORD, /* length of host name */
     2 nPort             type INTERNET_PORT,/* converted port number */
     2 lpszUserName      type LPWSTR,/* pointer to user name */
     2 dwUserNameLength  type DWORD, /* length of user name */
     2 lpszPassword      type LPWSTR,/* pointer to password */
     2 dwPasswordLength  type DWORD, /* length of password */
     2 lpszUrlPath       type LPWSTR,/* pointer to URL-path */
     2 dwUrlPathLength   type DWORD, /* length of URL-path */
     2 lpszExtraInfo     type LPWSTR,
           /* pointer to extra information (e.g. ?foo or #foo) */
     2 dwExtraInfoLength type DWORD; /* length of extra information */
 define alias LPURL_COMPONENTSW handle URL_COMPONENTSW;

 %if UNICODE ^= '' %then %do;
    define alias   URL_COMPONENTS type URL_COMPONENTSW;
    define alias LPURL_COMPONENTS type LPURL_COMPONENTSW;
 %end; %else %do;
    define alias   URL_COMPONENTS type URL_COMPONENTSA;
    define alias LPURL_COMPONENTS type LPURL_COMPONENTSA;
 %end; /* UNICODE */


 /* INTERNET_CERTIFICATE_INFO lpBuffer - contains the certificate */
 /* returned from the server                                      */

 define structure
   1 INTERNET_CERTIFICATE_INFO,
     2 ftExpiry              type FILETIME,
       /* date the certificate expires. */
     2 ftStart               type FILETIME,
       /* date the certificate becomes valid. */
     2 lpszSubjectInfo       type LPTSTR,
       /* the name of organization, site, and server */
       /* the cert. was issued for.                  */
     2 lpszIssuerInfo        type LPTSTR,
       /* the name of organization, site, and server */
       /* the cert. was issued by.                   */
     2 lpszProtocolName      type LPTSTR,
       /* name of the protocol used to provide the secure connection. */
     2 lpszSignatureAlgName  type LPTSTR,
       /* the name of the algorithm used for signing the certificate. */
     2 lpszEncryptionAlgName type LPTSTR,
       /* the name of the algorithm used for doing encryption */
       /* over the secure channel (SSL/PCT) connection.       */
     2 dwKeySize             type DWORD;
       /* size of the key. */

 define alias LPINTERNET_CERTIFICATE_INFO
        handle INTERNET_CERTIFICATE_INFO;

 /* INTERNET_BUFFERS - combines headers and data. May be chained for  */
 /* e.g. file upload or scatter/gather operations.                    */
 /* For chunked read/write, lpcszHeader contains the chunked-ext      */

 define structure
 1 INTERNET_BUFFERSA,
   3 dwStructSize    type DWORD,  /* Set to stg(INTERNET_BUFFERS)     */
   3 Next            handle INTERNET_BUFFERSA, /* chain of buffers    */
   3 lpcszHeader     type LPCSTR, /* pointer to headers (may be NULL) */
   3 dwHeadersLength type DWORD,  /* length of headers if not NULL    */
   3 dwHeadersTotal  type DWORD,  /* size of hds if not enough buffer */
   3 lpvBuffer       ptr,         /* to data buffer (may be NULL)     */
   3 dwBufferLength  type DWORD,  /* length of data buffer if not NULL*/
   3 dwBufferTotal   type DWORD,  /* total size of chunk, or content- */
                                  /* length if not chunked */
   3 dwOffsetLow     type DWORD,  /* used for read-ranges (only used  */
                                  /* in HttpSendRequest2) */
   3 dwOffsetHigh    type DWORD;
 define alias LPINTERNET_BUFFERSA handle INTERNET_BUFFERSA;

 define structure
 1 INTERNET_BUFFERSW,
   3 dwStructSize    type DWORD,  /* Set to stg(INTERNET_BUFFERS)     */
   3 Next            handle INTERNET_BUFFERSW, /* chain of buffers    */
   3 lpcszHeader     type LPCWSTR,/* pointer to headers (may be NULL) */
   3 dwHeadersLength type DWORD,  /* length of headers if not NULL    */
   3 dwHeadersTotal  type DWORD,  /* size of hds if not enough buffer */
   3 lpvBuffer       ptr,         /* to data buffer (may be NULL)     */
   3 dwBufferLength  type DWORD,  /* length of data buffer if not NULL*/
   3 dwBufferTotal   type DWORD,  /* total size of chunk, or content- */
                                  /* length if not chunked */
   3 dwOffsetLow     type DWORD,  /* used for read-ranges (only used  */
                                  /* in HttpSendRequest2) */
   3 dwOffsetHigh    type DWORD;
 define alias LPINTERNET_BUFFERSW handle INTERNET_BUFFERSW;
 %if UNICODE ^= '' %then %do;
    define alias   INTERNET_BUFFERS type INTERNET_BUFFERSW;
    define alias LPINTERNET_BUFFERS type LPINTERNET_BUFFERSW;
 %end; %else %do;
    define alias   INTERNET_BUFFERS type INTERNET_BUFFERSA;
    define alias LPINTERNET_BUFFERS type LPINTERNET_BUFFERSA;
 %end; /* UNICODE */

 /* prototypes */

 dcl InternetTimeFromSystemTimeA
     entry (type SYSTEMTIME byaddr nonasgn, /* input GMT time */
            type DWORD,                     /* RFC format */
            char (*) varz byaddr,           /* output string buffer */
            type DWORD)                     /* output buffer size */
     BOOLAPI external ('InternetTimeFromSystemTimeA');
 dcl InternetTimeFromSystemTimeW
     entry (type SYSTEMTIME byaddr nonasgn, /* input GMT time */
            type DWORD,                     /* RFC format */
            WCHAR (*) varz byaddr,          /* output string buffer */
            type DWORD)                     /* output buffer size */
     BOOLAPI external ('InternetTimeFromSystemTimeW');
 %dcl InternetTimeFromSystemTime char ext;
 %if UNICODE ^= '' %then %do;
   %InternetTimeFromSystemTime = 'InternetTimeFromSystemTimeW';
 %end; %else %if #WINX32_ ^= '' %then %do;
   %InternetTimeFromSystemTime = 'InternetTimeFromSystemTimeA';
 %end; %else %do;
 %deact InternetTimeFromSystemTime;
 dcl InternetTimeFromSystemTime
     entry (type SYSTEMTIME byaddr nonasgn, /* input GMT time */
            type DWORD,                     /* RFC format */
            char (*) varz byaddr,           /* output string buffer */
            type DWORD)                     /* output buffer size */
     BOOLAPI external ('InternetTimeFromSystemTime');
 %end;

 /* constants for InternetTimeFromSystemTime */

 dcl INTERNET_RFC1123_FORMAT  type DWORD value(0),
     INTERNET_RFC1123_BUFSIZE type DWORD value(30);

 dcl InternetTimeToSystemTimeA
     entry (char (*) varz nonasgn byaddr,   /* NULL terminated string */
            type SYSTEMTIME byaddr,         /* output GMT time */
            type DWORD)
     BOOLAPI external ('InternetTimeToSystemTimeA');
 dcl InternetTimeToSystemTimeW
     entry (WCHAR (*) varz nonasgn byaddr,  /* NULL terminated string */
            type SYSTEMTIME byaddr,         /* output GMT time */
            type DWORD)
     BOOLAPI external ('InternetTimeToSystemTimeW');
 %dcl InternetTimeToSystemTime char ext;
 %if UNICODE ^= '' %then %do;
   %InternetTimeToSystemTime = 'InternetTimeToSystemTime';
 %end; %else %if #WINX32_ ^= '' %then %do;
   %InternetTimeToSystemTime = 'InternetTimeToSystemTimeA';
 %end; %else %do;
 %deact InternetTimeToSystemTime;
 dcl InternetTimeToSystemTime
     entry (char (*) varz nonasgn byaddr,   /* NULL terminated string */
            type SYSTEMTIME byaddr,         /* output GMT time */
            type DWORD)
     BOOLAPI external ('InternetTimeToSystemTime');
 %end;

 dcl InternetCrackUrlA
     entry (char (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD,
            type URL_COMPONENTSA byaddr)
     BOOLAPI external ('InternetCrackUrlA');
 dcl InternetCrackUrlW
     entry (WCHAR(*) varz byaddr nonasgn,
            type DWORD,
            type DWORD,
            type URL_COMPONENTSW byaddr)
     BOOLAPI external ('InternetCrackUrlW');
 %dcl InternetCrackUrl char ext;
 %if UNICODE ^= '' %then %do;
 %InternetCrackUrl = 'InternetCrackUrlW';
 %end; %else %do;
 %InternetCrackUrl = 'InternetCrackUrlA';
 %end; /* !UNICODE */

 dcl InternetCreateUrlA
     entry (type URL_COMPONENTSA byaddr,
            type DWORD,
            char (*) varz byaddr,
            type DWORD byaddr)
     BOOLAPI external ('InternetCreateUrlA');
 dcl InternetCreateUrlW
     entry (type URL_COMPONENTSW byaddr,
            type DWORD,
            WCHAR(*) varz byaddr,
            type DWORD byaddr)
     BOOLAPI external ('InternetCreateUrlW');
 %dcl InternetCreateUrl char ext;
 %if UNICODE ^= '' %then %do;
 %InternetCreateUrl = 'InternetCreateUrlW';
 %end; %else %do;
 %InternetCreateUrl = 'InternetCreateUrlA';
 %end; /* !UNICODE */

 dcl InternetCanonicalizeUrlA
     entry (char (*) varz nonasgn byaddr,
            char (*) varz byaddr,
            type DWORD byaddr,
            type DWORD)
     BOOLAPI external ('InternetCanonicalizeUrlA');
 dcl InternetCanonicalizeUrlW
     entry (WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr,
            type DWORD byaddr,
            type DWORD)
     BOOLAPI external ('InternetCanonicalizeUrlW');
 %dcl InternetCanonicalizeUrl char ext;
 %if UNICODE ^= '' %then   %do;
 %InternetCanonicalizeUrl = 'InternetCanonicalizeUrlW';
 %end; %else %do;
 %InternetCanonicalizeUrl = 'InternetCanonicalizeUrlA';
 %end; /* !UNICODE */


 dcl InternetCombineUrlA
     entry (char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr,
            char (*) varz byaddr,
            type DWORD byaddr,
            type DWORD)
     BOOLAPI external ('InternetCombineUrlA');
 dcl InternetCombineUrlW
     entry (WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr,
            type DWORD byaddr,
            type DWORD)
     BOOLAPI external ('InternetCombineUrlW');
 %dcl InternetCombineUrl char ext;
 %if UNICODE ^= '' %then %do;
 %InternetCombineUrl = 'InternetCombineUrlW';
 %end; %else %do;
 %InternetCombineUrl = 'InternetCombineUrlA';
 %end; /* !UNICODE */

 /* flags for InternetCrackUrl() and InternetCreateUrl() */

 dcl ICU_ESCAPE             type DWORD value('80000000'xu),
                /* (un)escape URL characters */
     ICU_USERNAME           type DWORD value('40000000'xu);
                /* use internal username & password */

 /* flags for InternetCanonicalizeUrl() and InternetCombineUrl() */


 dcl ICU_NO_ENCODE          type DWORD value('20000000'xu),
                /* Don't convert unsafe characters to escape sequence */
     ICU_DECODE             type DWORD value('10000000'xu),
                /* Convert %XX escape sequences to characters */
     ICU_NO_META            type DWORD value('08000000'xu),
                /* Don't convert .. etc. meta path sequences */
     ICU_ENCODE_SPACES_ONLY type DWORD value('04000000'xu),
                /* Encode spaces only */
     ICU_BROWSER_MODE       type DWORD value('02000000'xu),
                /* Special encode/decode rules for browser */
     ICU_ENCODE_PERCENT     type DWORD value('00001000'xu);
                /* Encode any percent (ASCII25) signs encountered, */
                /* default is to not encode percent.               */


 dcl InternetOpenA
     entry (char (*) varz nonasgn byaddr,
            type DWORD,
            char (*) varz nonasgn byaddr optional,
            char (*) varz nonasgn byaddr optional,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('InternetOpenA');
 dcl InternetOpenW
     entry (WCHAR(*) varz byaddr nonasgn,
            type DWORD,
            WCHAR(*) varz byaddr nonasgn optional,
            WCHAR(*) varz byaddr nonasgn optional,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('InternetOpenW');
 %dcl InternetOpen char ext;
 %if UNICODE ^= '' %then %do;
 %InternetOpen = 'InternetOpenW';
 %end; %else %do;
 %InternetOpen = 'InternetOpenA';
 %end; /* !UNICODE */

 /* access types for InternetOpen() */

 dcl INTERNET_OPEN_TYPE_PRECONFIG type DWORD value(0),
                                  /* use registry configuration */
     INTERNET_OPEN_TYPE_DIRECT    type DWORD value(1),
                                  /* direct to net */
     INTERNET_OPEN_TYPE_PROXY     type DWORD value(3),
                                  /* via named proxy */
     INTERNET_OPEN_TYPE_PRECONFIG_WITH_NO_AUTOPROXY type DWORD
         value(4);                /* prevent using java/script/INS */

 /* old names for access types */
 dcl ( PRE_CONFIG_INTERNET_ACCESS value(INTERNET_OPEN_TYPE_PRECONFIG)
     , LOCAL_INTERNET_ACCESS      value(INTERNET_OPEN_TYPE_DIRECT)
     , CERN_PROXY_INTERNET_ACCESS value(INTERNET_OPEN_TYPE_PROXY)
     ) type DWORD;


 dcl InternetCloseHandle
     entry (type HINTERNET)
     BOOLAPI external('InternetCloseHandle');


 dcl InternetConnectA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr,
            type INTERNET_PORT,
            char (*) varz nonasgn byaddr optional,
            char (*) varz nonasgn byaddr optional,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('InternetConnectA');
 dcl InternetConnectW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn,
            type INTERNET_PORT,
            WCHAR(*) varz byaddr nonasgn optional,
            WCHAR(*) varz byaddr nonasgn optional,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('InternetConnectW');
 %dcl InternetConnect char ext;
 %if UNICODE ^= '' %then %do;
 %InternetConnect = 'InternetConnectW';
 %end; %else %do;
 %InternetConnect = 'InternetConnectA';
 %end; /* !UNICODE */

 /* service types for InternetConnect() */

 dcl INTERNET_SERVICE_FTP    type DWORD value(1),
     INTERNET_SERVICE_GOPHER type DWORD value(2),
     INTERNET_SERVICE_HTTP   type DWORD value(3);


 dcl InternetOpenUrlA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr optional,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('InternetOpenUrlA');

 dcl InternetOpenUrlW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn optional,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('InternetOpenUrlW');
 %dcl InternetOpenUrl char ext;
 %if UNICODE ^= '' %then %do;
 %InternetOpenUrl = 'InternetOpenUrlW';
 %end; %else %do;
 %InternetOpenUrl = 'InternetOpenUrlA';
 %end; /* !UNICODE */

 dcl InternetReadFile
     entry (type HINTERNET,
            type LPVOID,
            type DWORD,
            type DWORD byaddr)
     BOOLAPI external ('InternetReadFile');

 dcl InternetReadFileExA
     entry (type HINTERNET,
            type INTERNET_BUFFERSA byaddr,
            type DWORD,
            type DWORD)
     BOOLAPI external ('InternetReadFileExA');

 dcl InternetReadFileExW
     entry (type HINTERNET,
            type INTERNET_BUFFERSW byaddr,
            type DWORD,
            type DWORD)
     BOOLAPI external ('InternetReadFileExW');
 %dcl InternetReadFileEx char ext;
 %if UNICODE ^= '' %then %do;
 %InternetReadFileEx = 'InternetReadFileExW';
 %end; %else %do;
 %InternetReadFileEx = 'InternetReadFileExA';
 %end; /* !UNICODE */

 /* flags for InternetReadFileEx() */

 dcl ( IRF_ASYNC       value (WININET_API_FLAG_ASYNC)
     , IRF_SYNC        value (WININET_API_FLAG_SYNC)
     , IRF_USE_CONTEXT value (WININET_API_FLAG_USE_CONTEXT)
     , IRF_NO_WAIT     value ('00000008'xu)
     ) type DWORD;


 dcl InternetSetFilePointer
     entry (type HINTERNET,
            type LONG,
            type PVOID,
            type DWORD,
            type DWORD)
     returns (byvalue optional type DWORD)
     INTERNETAPI WINAPI external ('InternetSetFilePointer');

 dcl InternetWriteFile
     entry (type HINTERNET,
            type LPCVOID,
            type DWORD,
            type DWORD byaddr)
     BOOLAPI external ('InternetWriteFile');

 dcl InternetQueryDataAvailable
     entry (type HINTERNET,
            type DWORD byaddr optional,
            type DWORD,
            type DWORD)
     BOOLAPI external ('InternetQueryDataAvailable');

 dcl InternetFindNextFileA
     entry (type HINTERNET,
            type LPVOID)
     BOOLAPI external ('InternetFindNextFileA');
 dcl InternetFindNextFileW
     entry (type HINTERNET,
            type LPVOID)
     BOOLAPI external ('InternetFindNextFileW');
 %dcl InternetFindNextFile char ext;
 %if UNICODE ^= '' %then %do;
 %InternetFindNextFile = 'InternetFindNextFileW';
 %end; %else %do;
 %InternetFindNextFile = 'InternetFindNextFileA';
 %end; /* !UNICODE */


 dcl InternetQueryOptionA
     entry (type HINTERNET,
            type DWORD,
            type LPVOID,
            type DWORD byaddr)
     BOOLAPI external ('InternetQueryOptionA');
 dcl InternetQueryOptionW
     entry (type HINTERNET,
            type DWORD,
            type LPVOID,
            type DWORD byaddr)
     BOOLAPI external ('InternetQueryOptionW');
 %dcl InternetQueryOption char ext;
 %if UNICODE ^= '' %then %do;
 %InternetQueryOption = 'InternetQueryOptionW';
 %end; %else %do;
 %InternetQueryOption = 'InternetQueryOptionA';
 %end; /* !UNICODE */


 dcl InternetSetOptionA
     entry (type HINTERNET,
            type DWORD,
            type LPVOID,
            type DWORD)
     BOOLAPI external ('InternetSetOptionA');
 dcl InternetSetOptionW
     entry (type HINTERNET,
            type DWORD,
            type LPVOID,
            type DWORD)
     BOOLAPI external ('InternetSetOptionW');
 %dcl InternetSetOption char ext;
 %if UNICODE ^= '' %then %do;
 %InternetSetOption = 'InternetSetOptionW';
 %end; %else %do;
 %InternetSetOption = 'InternetSetOptionA';
 %end; /* !UNICODE */


 dcl InternetSetOptionExA
     entry (type HINTERNET,
            type DWORD,
            type LPVOID,
            type DWORD,
            type DWORD)
     BOOLAPI external ('InternetSetOptionExA');

 dcl InternetSetOptionExW
     entry (type HINTERNET,
            type DWORD,
            type LPVOID,
            type DWORD,
            type DWORD)
     BOOLAPI external ('InternetSetOptionExW');
 %dcl InternetSetOptionEx char ext;
 %if UNICODE ^= '' %then %do;
 %InternetSetOptionEx = 'InternetSetOptionExW';
 %end; %else %do;
 %InternetSetOptionEx = 'InternetSetOptionExA';
 %end; /* !UNICODE */

 dcl InternetLockRequestFile
     entry (type HINTERNET,
            type HANDLE byaddr)
     BOOLAPI external ('InternetLockRequestFile');


 dcl InternetUnlockRequestFile
     entry (type HANDLE)
     BOOLAPI external ('InternetUnlockRequestFile');


 /* flags for InternetSetOptionEx() */

 dcl ( ISO_GLOBAL      value('00000001'xu) /* modify option globally */
     , ISO_REGISTRY    value('00000002'xu)
                       /* write option to registry (where applicable) */
     , ISO_VALID_FLAGS value(ior(ISO_GLOBAL, ISO_REGISTRY))
     ) type DWORD;


 /* options manifests for Internet{Query|Set}Option */

 dcl INTERNET_OPTION_CALLBACK                    type DWORD value(1),
     INTERNET_OPTION_CONNECT_TIMEOUT             type DWORD value(2),
     INTERNET_OPTION_CONNECT_RETRIES             type DWORD value(3),
     INTERNET_OPTION_CONNECT_BACKOFF             type DWORD value(4),
     INTERNET_OPTION_SEND_TIMEOUT                type DWORD value(5),
     INTERNET_OPTION_CONTROL_SEND_TIMEOUT        type DWORD
       value(INTERNET_OPTION_SEND_TIMEOUT),
     INTERNET_OPTION_RECEIVE_TIMEOUT             type DWORD value(6),
     INTERNET_OPTION_CONTROL_RECEIVE_TIMEOUT     type DWORD
       value(INTERNET_OPTION_RECEIVE_TIMEOUT),
     INTERNET_OPTION_DATA_SEND_TIMEOUT           type DWORD value(7),
     INTERNET_OPTION_DATA_RECEIVE_TIMEOUT        type DWORD value(8),
     INTERNET_OPTION_HANDLE_TYPE                 type DWORD value(9),
     INTERNET_OPTION_LISTEN_TIMEOUT              type DWORD value(11),
     INTERNET_OPTION_READ_BUFFER_SIZE            type DWORD value(12),
     INTERNET_OPTION_WRITE_BUFFER_SIZE           type DWORD value(13),

     INTERNET_OPTION_ASYNC_ID                    type DWORD value(15),
     INTERNET_OPTION_ASYNC_PRIORITY              type DWORD value(16),

     INTERNET_OPTION_PARENT_HANDLE               type DWORD value(21),
     INTERNET_OPTION_KEEP_CONNECTION             type DWORD value(22),
     INTERNET_OPTION_REQUEST_FLAGS               type DWORD value(23),
     INTERNET_OPTION_EXTENDED_ERROR              type DWORD value(24),

     INTERNET_OPTION_OFFLINE_MODE                type DWORD value(26),
     INTERNET_OPTION_CACHE_STREAM_HANDLE         type DWORD value(27),
     INTERNET_OPTION_USERNAME                    type DWORD value(28),
     INTERNET_OPTION_PASSWORD                    type DWORD value(29),
     INTERNET_OPTION_ASYNC                       type DWORD value(30),
     INTERNET_OPTION_SECURITY_FLAGS              type DWORD value(31),
     INTERNET_OPTION_SECURITY_CERTIFICATE_STRUCT type DWORD value(32),
     INTERNET_OPTION_DATAFILE_NAME               type DWORD value(33),
     INTERNET_OPTION_URL                         type DWORD value(34),
     INTERNET_OPTION_SECURITY_CERTIFICATE        type DWORD value(35),
     INTERNET_OPTION_SECURITY_KEY_BITNESS        type DWORD value(36),
     INTERNET_OPTION_REFRESH                     type DWORD value(37),
     INTERNET_OPTION_PROXY                       type DWORD value(38),
     INTERNET_OPTION_SETTINGS_CHANGED            type DWORD value(39),
     INTERNET_OPTION_VERSION                     type DWORD value(40),
     INTERNET_OPTION_USER_AGENT                  type DWORD value(41),
     INTERNET_OPTION_END_BROWSER_SESSION         type DWORD value(42),
     INTERNET_OPTION_PROXY_USERNAME              type DWORD value(43),
     INTERNET_OPTION_PROXY_PASSWORD              type DWORD value(44),
     INTERNET_OPTION_CONTEXT_VALUE               type DWORD value(45),
     INTERNET_OPTION_CONNECT_LIMIT               type DWORD value(46),
     INTERNET_OPTION_SECURITY_SELECT_CLIENT_CERT type DWORD value(47),
     INTERNET_OPTION_POLICY                      type DWORD value(48),
     INTERNET_OPTION_DISCONNECTED_TIMEOUT        type DWORD value(49),
     INTERNET_OPTION_CONNECTED_STATE             type DWORD value(50),
     INTERNET_OPTION_IDLE_STATE                  type DWORD value(51),
     INTERNET_OPTION_OFFLINE_SEMANTICS           type DWORD value(52),
     INTERNET_OPTION_SECONDARY_CACHE_KEY         type DWORD value(53),
     INTERNET_OPTION_CALLBACK_FILTER             type DWORD value(54),
     INTERNET_OPTION_CONNECT_TIME                type DWORD value(55),
     INTERNET_OPTION_SEND_THROUGHPUT             type DWORD value(56),
     INTERNET_OPTION_RECEIVE_THROUGHPUT          type DWORD value(57),
     INTERNET_OPTION_REQUEST_PRIORITY            type DWORD value(58),
     INTERNET_OPTION_HTTP_VERSION                type DWORD value(59),
     INTERNET_OPTION_RESET_URLCACHE_SESSION      type DWORD value(60),
     INTERNET_OPTION_ERROR_MASK                  type DWORD value(62),
     INTERNET_OPTION_FROM_CACHE_TIMEOUT          type DWORD value(63),
     INTERNET_OPTION_BYPASS_EDITED_ENTRY         type DWORD value(64),
     INTERNET_OPTION_CODEPAGE                    type DWORD value(68),
     INTERNET_OPTION_CACHE_TIMESTAMPS            type DWORD value(69),
     INTERNET_OPTION_DISABLE_AUTODIAL            type DWORD value(70),
     INTERNET_OPTION_MAX_CONNS_PER_SERVER        type DWORD value(73),
     INTERNET_OPTION_MAX_CONNS_PER_1_0_SERVER    type DWORD value(74),
     INTERNET_OPTION_PER_CONNECTION_OPTION       type DWORD value(75),
     INTERNET_OPTION_DIGEST_AUTH_UNLOAD          type DWORD value(76),
     INTERNET_OPTION_IGNORE_OFFLINE              type DWORD value(77);

 dcl INTERNET_FIRST_OPTION type DWORD
       value(INTERNET_OPTION_CALLBACK),
     INTERNET_LAST_OPTION type DWORD
       value(INTERNET_OPTION_IGNORE_OFFLINE);

 /* values for INTERNET_OPTION_PRIORITY */

 dcl INTERNET_PRIORITY_FOREGROUND type DWORD value(1000);

 /* handle types */

 dcl INTERNET_HANDLE_TYPE_INTERNET         type DWORD value(1),
     INTERNET_HANDLE_TYPE_CONNECT_FTP      type DWORD value(2),
     INTERNET_HANDLE_TYPE_CONNECT_GOPHER   type DWORD value(3),
     INTERNET_HANDLE_TYPE_CONNECT_HTTP     type DWORD value(4),
     INTERNET_HANDLE_TYPE_FTP_FIND         type DWORD value(5),
     INTERNET_HANDLE_TYPE_FTP_FIND_HTML    type DWORD value(6),
     INTERNET_HANDLE_TYPE_FTP_FILE         type DWORD value(7),
     INTERNET_HANDLE_TYPE_FTP_FILE_HTML    type DWORD value(8),
     INTERNET_HANDLE_TYPE_GOPHER_FIND      type DWORD value(9),
     INTERNET_HANDLE_TYPE_GOPHER_FIND_HTML type DWORD value(10),
     INTERNET_HANDLE_TYPE_GOPHER_FILE      type DWORD value(11),
     INTERNET_HANDLE_TYPE_GOPHER_FILE_HTML type DWORD value(12),
     INTERNET_HANDLE_TYPE_HTTP_REQUEST     type DWORD value(13),
     INTERNET_HANDLE_TYPE_FILE_REQUEST     type DWORD value(14);

 /* values for INTERNET_OPTION_SECURITY_FLAGS */

 dcl SECURITY_FLAG_SECURE             type DWORD value('00000001'xu),
                                      /* can query only */
     SECURITY_FLAG_STRENGTH_WEAK      type DWORD value('10000000'xu),
     SECURITY_FLAG_STRENGTH_MEDIUM    type DWORD value('40000000'xu),
     SECURITY_FLAG_STRENGTH_STRONG    type DWORD value('20000000'xu),
     SECURITY_FLAG_UNKNOWNBIT         type DWORD value('80000000'xu),
     SECURITY_FLAG_FORTEZZA           type DWORD value('08000000'xu),
     SECURITY_FLAG_NORMALBITNESS      type DWORD
       value(SECURITY_FLAG_STRENGTH_WEAK),

     /* following are unused */
     SECURITY_FLAG_SSL                type DWORD value('00000002'xu),
     SECURITY_FLAG_SSL3               type DWORD value('00000004'xu),
     SECURITY_FLAG_PCT                type DWORD value('00000008'xu),
     SECURITY_FLAG_PCT4               type DWORD value('00000010'xu),
     SECURITY_FLAG_IETFSSL4           type DWORD value('00000020'xu),

     /* following are for backwards compatibility only. */
     SECURITY_FLAG_40BIT              type DWORD
       value(SECURITY_FLAG_STRENGTH_WEAK),
     SECURITY_FLAG_128BIT             type DWORD
       value(SECURITY_FLAG_STRENGTH_MEDIUM),
     SECURITY_FLAG_56BIT              type DWORD
       value(SECURITY_FLAG_STRENGTH_STRONG),

     /* setable flags */
     SECURITY_FLAG_IGNORE_REVOCATION  type DWORD value('00000080'xu),
     SECURITY_FLAG_IGNORE_UNKNOWN_CA  type DWORD value('00000100'xu),
     SECURITY_FLAG_IGNORE_WRONG_USAGE type DWORD value('00000200'xu),

     SECURITY_FLAG_IGNORE_CERT_CN_INVALID   type DWORD
       value(INTERNET_FLAG_IGNORE_CERT_CN_INVALID),
     SECURITY_FLAG_IGNORE_CERT_DATE_INVALID type DWORD
       value(INTERNET_FLAG_IGNORE_CERT_DATE_INVALID),

     SECURITY_FLAG_IGNORE_REDIRECT_TO_HTTPS type DWORD
       value(INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS),
     SECURITY_FLAG_IGNORE_REDIRECT_TO_HTTP  type DWORD
       value(INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP),

     SECURITY_SET_MASK       type DWORD
       value ( ior (SECURITY_FLAG_IGNORE_REVOCATION,
                    SECURITY_FLAG_IGNORE_UNKNOWN_CA,
                    SECURITY_FLAG_IGNORE_CERT_CN_INVALID,
                    SECURITY_FLAG_IGNORE_CERT_DATE_INVALID,
                    SECURITY_FLAG_IGNORE_WRONG_USAGE));


 dcl InternetGetLastResponseInfoA
     entry (type DWORD byaddr,
            char (*) varz byaddr optional,
            type DWORD byaddr)
     BOOLAPI external ('InternetGetLastResponseInfoA');
 dcl InternetGetLastResponseInfoW
     entry (type DWORD byaddr,
            WCHAR(*) varz byaddr optional,
            type DWORD byaddr)
     BOOLAPI external ('InternetGetLastResponseInfoW');
 %dcl InternetGetLastResponseInfo char ext;
 %if UNICODE ^= '' %then %do;
 %InternetGetLastResponseInfo = 'InternetGetLastResponseInfoW';
 %end; %else %do;
 %InternetGetLastResponseInfo = 'InternetGetLastResponseInfoA';
 %end; /* !UNICODE */

 /* callback function for InternetSetStatusCallback */

 define alias INTERNET_STATUS_CALLBACK
       entry (type HINTERNET,
              type DWORD,
              type DWORD,
              type LPVOID,
              type DWORD)
       limited CALLBACK;

 dcl InternetSetStatusCallbackA
     entry (type HINTERNET,
            type INTERNET_STATUS_CALLBACK optional byvalue)
     returns (byvalue optional type INTERNET_STATUS_CALLBACK)
     INTERNETAPI WINAPI external ('InternetSetStatusCallbackA');
 dcl InternetSetStatusCallbackW
     entry (type HINTERNET,
            type INTERNET_STATUS_CALLBACK optional byvalue)
     returns (byvalue optional type INTERNET_STATUS_CALLBACK)
     INTERNETAPI WINAPI external ('InternetSetStatusCallbackW');
 %dcl InternetSetStatusCallback char ext;
 %if UNICODE ^= '' %then %do;
   %InternetSetStatusCallback = 'InternetSetStatusCallbackW';
 %end; %else %if #WINX32_ ^= '' %then %do;
   %InternetSetStatusCallback = 'InternetSetStatusCallbackA';
 %end; %else %do;
 %deact InternetSetStatusCallback;
 dcl InternetSetStatusCallback
     entry (type HINTERNET,
            type INTERNET_STATUS_CALLBACK optional byvalue)
     returns (byvalue optional type INTERNET_STATUS_CALLBACK)
     INTERNETAPI WINAPI external ('InternetSetStatusCallback');
 %end;

 /* status manifests for Internet status callback */

 dcl INTERNET_STATUS_RESOLVING_NAME        type DWORD value(10),
     INTERNET_STATUS_NAME_RESOLVED         type DWORD value(11),
     INTERNET_STATUS_CONNECTING_TO_SERVER  type DWORD value(20),
     INTERNET_STATUS_CONNECTED_TO_SERVER   type DWORD value(21),
     INTERNET_STATUS_SENDING_REQUEST       type DWORD value(30),
     INTERNET_STATUS_REQUEST_SENT          type DWORD value(31),
     INTERNET_STATUS_RECEIVING_RESPONSE    type DWORD value(40),
     INTERNET_STATUS_RESPONSE_RECEIVED     type DWORD value(41),
     INTERNET_STATUS_CTL_RESPONSE_RECEIVED type DWORD value(42),
     INTERNET_STATUS_PREFETCH              type DWORD value(43),
     INTERNET_STATUS_CLOSING_CONNECTION    type DWORD value(50),
     INTERNET_STATUS_CONNECTION_CLOSED     type DWORD value(51),
     INTERNET_STATUS_HANDLE_CREATED        type DWORD value(60),
     INTERNET_STATUS_HANDLE_CLOSING        type DWORD value(70),
     INTERNET_STATUS_DETECTING_PROXY       type DWORD value(80),
     INTERNET_STATUS_REQUEST_COMPLETE      type DWORD value(100),
     INTERNET_STATUS_REDIRECT              type DWORD value(110),
     INTERNET_STATUS_INTERMEDIATE_RESPONSE type DWORD value(120),
     INTERNET_STATUS_USER_INPUT_REQUIRED   type DWORD value(140),
     INTERNET_STATUS_STATE_CHANGE          type DWORD value(200);

 /* the following can be indicated in a state change notification: */

 dcl INTERNET_STATE_CONNECTED            type DWORD value('00000001'xu),
       /* connected state (mutually exclusive with disconnected) */
     INTERNET_STATE_DISCONNECTED         type DWORD value('00000002'xu),
       /* disconnected from network */
     INTERNET_STATE_DISCONNECTED_BY_USER type DWORD value('00000010'xu),
       /* disconnected by user request */
     INTERNET_STATE_IDLE                 type DWORD value('00000100'xu),
       /* no network requests being made (by Wininet) */
     INTERNET_STATE_BUSY                 type DWORD value('00000200'xu);
       /* network requests being made (by Wininet) */

 /* if the following value is returned by InternetSetStatusCallback,  */
 /* then probably an invalid (non-code) address was supplied for the  */
 /* callback.                                                         */
 /* PL/I Note: comparison must be of the form:                        */
 /* "if ENTRYADDR (result) = INET_INVALID_STATUS_CALLBACK then"       */
 dcl INTERNET_INVALID_STATUS_CALLBACK ptr value (ptrvalue (-1));

 /* FTP */

 /* manifests */

 dcl FTP_TRANSFER_TYPE_UNKNOWN type DWORD value('00000000'xn),
     FTP_TRANSFER_TYPE_ASCII   type DWORD
       value(INTERNET_FLAG_TRANSFER_ASCII),
     FTP_TRANSFER_TYPE_BINARY  type DWORD
       value(INTERNET_FLAG_TRANSFER_BINARY),
     FTP_TRANSFER_TYPE_MASK    type DWORD
       value (ior (FTP_TRANSFER_TYPE_ASCII, FTP_TRANSFER_TYPE_BINARY));


 /* prototypes */

 dcl FtpFindFirstFileA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr optional,
            type WIN32_FIND_DATA byaddr,
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('FtpFindFirstFileA');
 dcl FtpFindFirstFileW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn optional,
            type WIN32_FIND_DATA byaddr,
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('FtpFindFirstFileW');
 %dcl FtpFindFirstFile char ext;
 %if UNICODE ^= '' %then %do;
 %FtpFindFirstFile = 'FtpFindFirstFileW';
 %end; %else %do;
 %FtpFindFirstFile = 'FtpFindFirstFileA';
 %end; /* !UNICODE */


 dcl FtpGetFileA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr,
            type BOOL,
            type DWORD,
            type DWORD,
            type DWORD)
     BOOLAPI external ('FtpGetFileA');
 dcl FtpGetFileW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn,
            type BOOL,
            type DWORD,
            type DWORD,
            type DWORD)
     BOOLAPI external ('FtpGetFileW');
 %dcl FtpGetFile char ext;
 %if UNICODE ^= '' %then %do;
 %FtpGetFile = 'FtpGetFileW';
 %end; %else %do;
 %FtpGetFile = 'FtpGetFileA';
 %end; /* !UNICODE */


 dcl FtpPutFileA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD)
     BOOLAPI external ('FtpPutFileA');

 dcl FtpPutFileW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn,
            type DWORD,
            type DWORD)
     BOOLAPI external ('FtpPutFileW');
 %dcl FtpPutFile char ext;
 %if UNICODE ^= '' %then %do;
 %FtpPutFile = 'FtpPutFileW';
 %end; %else %do;
 %FtpPutFile = 'FtpPutFileA';
 %end; /* !UNICODE */

 dcl FtpGetFileEx
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr optional,
            WCHAR (*) varz nonasgn byaddr optional,
            type BOOL,
            type DWORD,
            type DWORD,
            type DWORD_PTR)
     BOOLAPI external ('FtpGetFileEx');

 dcl FtpPutFileEx
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn optional,
            char(*) varz byaddr nonasgn optional,
            type DWORD,
            type DWORD,
            type DWORD_PTR)
     BOOLAPI external ('FtpPutFileEx');

 dcl FtpDeleteFileA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr)
     BOOLAPI external ('FtpDeleteFileA');

 dcl FtpDeleteFileW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn)
     BOOLAPI external ('FtpDeleteFileW');
 %dcl FtpDeleteFile char ext;
 %if UNICODE ^= '' %then %do;
 %FtpDeleteFile = 'FtpDeleteFileW';
 %end; %else %do;
 %FtpDeleteFile = 'FtpDeleteFileA';
 %end; /* !UNICODE */


 dcl FtpRenameFileA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr)
     BOOLAPI external ('FtpRenameFileA');
 dcl FtpRenameFileW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn)
     BOOLAPI external ('FtpRenameFileW');
 %dcl FtpRenameFile char ext;
 %if UNICODE ^= '' %then %do;
 %FtpRenameFile = 'FtpRenameFileW';
 %end; %else %do;
 %FtpRenameFile = 'FtpRenameFileA';
 %end; /* !UNICODE */


 dcl FtpOpenFileA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('FtpOpenFileA');

 dcl FtpOpenFileW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('FtpOpenFileW');
 %dcl FtpOpenFile char ext;
 %if UNICODE ^= '' %then %do;
 %FtpOpenFile = 'FtpOpenFileW';
 %end; %else %do;
 %FtpOpenFile = 'FtpOpenFileA';
 %end; /* !UNICODE */


 dcl FtpCreateDirectoryA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr)
     BOOLAPI external ('FtpCreateDirectoryA');

 dcl FtpCreateDirectoryW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn)
     BOOLAPI external ('FtpCreateDirectoryW');
 %dcl FtpCreateDirectory char ext;
 %if UNICODE ^= '' %then %do;
 %FtpCreateDirectory = 'FtpCreateDirectoryW';
 %end; %else %do;
 %FtpCreateDirectory = 'FtpCreateDirectoryA';
 %end; /* !UNICODE */


 dcl FtpRemoveDirectoryA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr)
     BOOLAPI external ('FtpRemoveDirectoryA');
 dcl FtpRemoveDirectoryW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn)
     BOOLAPI external ('FtpRemoveDirectoryW');
 %dcl FtpRemoveDirectory char ext;
 %if UNICODE ^= '' %then %do;
 %FtpRemoveDirectory = 'FtpRemoveDirectoryW';
 %end; %else %do;
 %FtpRemoveDirectory = 'FtpRemoveDirectoryA';
 %end; /* !UNICODE */


 dcl FtpSetCurrentDirectoryA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr)
     BOOLAPI external ('FtpSetCurrentDirectoryA');
 dcl FtpSetCurrentDirectoryW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn)
     BOOLAPI external ('FtpSetCurrentDirectoryW');
 %dcl FtpSetCurrentDirectory char ext;
 %if UNICODE ^= '' %then %do;
 %FtpSetCurrentDirectory = 'FtpSetCurrentDirectoryW';
 %end; %else %do;
 %FtpSetCurrentDirectory = 'FtpSetCurrentDirectoryA';
 %end; /* !UNICODE */


 dcl FtpGetCurrentDirectoryA
     entry (type HINTERNET,
            char (*) varz byaddr,
            type DWORD byaddr)
     BOOLAPI external ('FtpGetCurrentDirectoryA');
 dcl FtpGetCurrentDirectoryW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr,
            type DWORD byaddr)
     BOOLAPI external ('FtpGetCurrentDirectoryW');
 %dcl FtpGetCurrentDirectory char ext;
 %if UNICODE ^= '' %then %do;
 %FtpGetCurrentDirectory = 'FtpGetCurrentDirectoryW';
 %end; %else %do;
 %FtpGetCurrentDirectory = 'FtpGetCurrentDirectoryA';
 %end; /* !UNICODE */

 dcl FtpCommandA
     entry (type HINTERNET,
            type BOOL,
            type DWORD,
            char(*) varz byaddr nonasgn,
            type DWORD_PTR,
            type HINTERNET byaddr optional)
     BOOLAPI external('FtpCommandA');

 dcl FtpCommandW
     entry (type HINTERNET,
            type BOOL,
            type DWORD,
            WCHAR(*) varz byaddr nonasgn,
            type DWORD_PTR,
            type HINTERNET byaddr optional)
     BOOLAPI external('FtpCommandW');
 %dcl FtpCommand char ext;
 %if UNICODE ^= '' %then %do;
 %FtpCommand = 'FtpCommandW';
 %end; %else %do;
 %FtpCommand = 'FtpCommandA';
 %end; /* !UNICODE */


 dcl FtpGetFileSize
     entry (type HINTERNET,
            type DWORD byaddr optional)
     returns (byvalue optional type DWORD)
     INTERNETAPI WINAPI external ('FtpGetFileSize');

 /* Gopher */

 /* manifests */

 /* string field lengths (   characters, not bytes) */

 dcl MAX_GOPHER_DISPLAY_TEXT  fixed bin(31) value(128),
     MAX_GOPHER_SELECTOR_TEXT fixed bin(31) value(256),
     MAX_GOPHER_HOST_NAME     fixed bin(31)
         value(INTERNET_MAX_HOST_NAME_LENGTH),
     MAX_GOPHER_LOCATOR_LENGTH fixed bin(31)
         value (  1
                + MAX_GOPHER_DISPLAY_TEXT
                + 1
                + MAX_GOPHER_SELECTOR_TEXT
                + 1
                + MAX_GOPHER_HOST_NAME
                + 1
                + INTERNET_MAX_PORT_NUMBER_LENGTH
                + 1
                + 1
                + 2);

 /* structures/types */

 /* GOPHER_FIND_DATA - returns the results of a            */
 /* GopherFindFirstFile() / InternetFindNextFile() request */

 define structure
   1 GOPHER_FIND_DATAA,
    2 DisplayString        char (MAX_GOPHER_DISPLAY_TEXT) varz,
    2 GopherType           type DWORD,/* GOPHER_TYPE_, if known */
    2 SizeLow              type DWORD,
    2 SizeHigh             type DWORD,
    2 LastModificationTime type FILETIME,
    2 Locator              char (MAX_GOPHER_LOCATOR_LENGTH) varz;
 define alias LPGOPHER_FIND_DATAA handle GOPHER_FIND_DATAA;

 define structure
   1 GOPHER_FIND_DATAW,
    2 DisplayString        WCHAR (MAX_GOPHER_DISPLAY_TEXT) varz,
    2 GopherType           type DWORD,/* GOPHER_TYPE_, if known */
    2 SizeLow              type DWORD,
    2 SizeHigh             type DWORD,
    2 LastModificationTime type FILETIME,
    2 Locator              WCHAR (MAX_GOPHER_LOCATOR_LENGTH) varz;
 define alias LPGOPHER_FIND_DATAW handle GOPHER_FIND_DATAW;

 %if UNICODE ^= '' %then %do;
 define alias GOPHER_FIND_DATA   type GOPHER_FIND_DATAW;
 define alias LPGOPHER_FIND_DATA type LPGOPHER_FIND_DATAW;
 %end; %else %do;
 define alias GOPHER_FIND_DATA   type GOPHER_FIND_DATAA;
 define alias LPGOPHER_FIND_DATA type LPGOPHER_FIND_DATAA;
 %end;
 /* UNICODE */


 /* manifests for GopherType */

 dcl GOPHER_TYPE_TEXT_FILE      fixed bin(31) value('00000001'xn),
     GOPHER_TYPE_DIRECTORY      fixed bin(31) value('00000002'xn),
     GOPHER_TYPE_CSO            fixed bin(31) value('00000004'xn),
     GOPHER_TYPE_ERROR          fixed bin(31) value('00000008'xn),
     GOPHER_TYPE_MAC_BINHEX     fixed bin(31) value('00000010'xn),
     GOPHER_TYPE_DOS_ARCHIVE    fixed bin(31) value('00000020'xn),
     GOPHER_TYPE_UNIX_UUENCODED fixed bin(31) value('00000040'xn),
     GOPHER_TYPE_INDEX_SERVER   fixed bin(31) value('00000080'xn),
     GOPHER_TYPE_TELNET         fixed bin(31) value('00000100'xn),
     GOPHER_TYPE_BINARY         fixed bin(31) value('00000200'xn),
     GOPHER_TYPE_REDUNDANT      fixed bin(31) value('00000400'xn),
     GOPHER_TYPE_TN3270         fixed bin(31) value('00000800'xn),
     GOPHER_TYPE_GIF            fixed bin(31) value('00001000'xn),
     GOPHER_TYPE_IMAGE          fixed bin(31) value('00002000'xn),
     GOPHER_TYPE_BITMAP         fixed bin(31) value('00004000'xn),
     GOPHER_TYPE_MOVIE          fixed bin(31) value('00008000'xn),
     GOPHER_TYPE_SOUND          fixed bin(31) value('00010000'xn),
     GOPHER_TYPE_HTML           fixed bin(31) value('00020000'xn),
     GOPHER_TYPE_PDF            fixed bin(31) value('00040000'xn),
     GOPHER_TYPE_CALENDAR       fixed bin(31) value('00080000'xn),
     GOPHER_TYPE_INLINE         fixed bin(31) value('00100000'xn),
     GOPHER_TYPE_UNKNOWN        fixed bin(31) value('20000000'xn),
     GOPHER_TYPE_ASK            fixed bin(31) value('40000000'xn),
     GOPHER_TYPE_GOPHER_PLUS    fixed bin(31) value('80000000'xn);

 /* gopher type macros */

 IS_GOPHER_FILE: proc (type) returns (bit (1) byvalue)
                 options(inline reorder) internal;
   dcl type fixed bin (31);
   dcl iand builtin;
   return (iand (type, GOPHER_TYPE_FILE_MASK) ^= 0);
 end IS_GOPHER_FILE;

 IS_GOPHER_DIRECTORY: proc (type) returns (bit (1) byvalue)
                      options(inline reorder) internal;
   dcl type fixed bin (31);
   dcl iand builtin;
   return (iand (type, GOPHER_TYPE_DIRECTORY) ^= 0);
 end IS_GOPHER_DIRECTORY;

 IS_GOPHER_ERROR: proc (type) returns (bit (1) byvalue)
                  options(inline reorder) internal;
   dcl type fixed bin (31);
   dcl iand builtin;
   return (iand (type, GOPHER_TYPE_ERROR) ^= 0);
 end IS_GOPHER_ERROR;

 IS_GOPHER_INDEX_SERVER: proc (type) returns (bit (1) byvalue)
                         options(inline reorder) internal;
   dcl type fixed bin (31);
   dcl iand builtin;
   return (iand (type, GOPHER_TYPE_INDEX_SERVER) ^= 0);
 end IS_GOPHER_INDEX_SERVER;

 IS_GOPHER_TELNET_SESSION: proc (type)  returns (bit (1) byvalue)
                           options(inline reorder) internal;
   dcl type fixed bin (31);
   dcl iand builtin;
   return (iand (type, GOPHER_TYPE_TELNET) ^= 0);
 end IS_GOPHER_TELNET_SESSION;

 IS_GOPHER_BACKUP_SERVER: proc (type) returns (bit (1) byvalue)
                          options(inline reorder) internal;
   dcl type fixed bin (31);
   dcl iand builtin;
   return (iand (type, GOPHER_TYPE_REDUNDANT) ^= 0);
 end IS_GOPHER_BACKUP_SERVER;

 IS_GOPHER_TN3270_SESSION: proc (type)  returns (bit (1) byvalue)
                           options(inline reorder) internal;
   dcl type fixed bin (31);
   dcl iand builtin;
   return (iand (type, GOPHER_TYPE_TN3270) ^= 0);
 end IS_GOPHER_TN3270_SESSION;

 IS_GOPHER_ASK: proc (type) returns (bit (1) byvalue)
                options(inline reorder) internal;
   dcl type fixed bin (31);
   dcl iand builtin;
   return (iand (type, GOPHER_TYPE_ASK) ^= 0);
 end IS_GOPHER_ASK;

 IS_GOPHER_PLUS: proc (type) returns (bit (1) byvalue)
                 options(inline reorder) internal;
   dcl type fixed bin (31);
   dcl iand builtin;
   return (iand (type, GOPHER_TYPE_GOPHER_PLUS) ^= 0);
 end IS_GOPHER_PLUS;

 IS_GOPHER_TYPE_KNOWN: proc (type) returns (bit (1) byvalue)
                       options(inline reorder) internal;
   dcl type fixed bin (31);
   dcl iand builtin;
   return (iand (type, GOPHER_TYPE_UNKNOWN) ^= 0);
 end IS_GOPHER_TYPE_KNOWN;


 /* GOPHER_TYPE_FILE_MASK - use this to determine if a locator */
 /* identifies a (known) file type                             */

 dcl GOPHER_TYPE_FILE_MASK fixed bin (31)
     value (ior (GOPHER_TYPE_TEXT_FILE,
                 GOPHER_TYPE_MAC_BINHEX,
                 GOPHER_TYPE_DOS_ARCHIVE,
                 GOPHER_TYPE_UNIX_UUENCODED,
                 GOPHER_TYPE_BINARY,
                 GOPHER_TYPE_GIF,
                 GOPHER_TYPE_IMAGE,
                 GOPHER_TYPE_BITMAP,
                 GOPHER_TYPE_MOVIE,
                 GOPHER_TYPE_SOUND,
                 GOPHER_TYPE_HTML,
                 GOPHER_TYPE_PDF,
                 GOPHER_TYPE_CALENDAR,
                 GOPHER_TYPE_INLINE));


 /* structured gopher attributes (as defined in gopher+ protocol */
 /* document) */

 define structure
   1 GOPHER_ADMIN_ATTRIBUTE_TYPE,
    2 Comment      type LPCTSTR,
    2 EmailAddress type LPCTSTR;

 define alias LPGOPHER_ADMIN_ATTRIBUTE_TYPE
        handle GOPHER_ADMIN_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_MOD_DATE_ATTRIBUTE_TYPE,
    2 DateAndTime type FILETIME;

 define alias LPGOPHER_MOD_DATE_ATTRIBUTE_TYPE
        handle GOPHER_MOD_DATE_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_TTL_ATTRIBUTE_TYPE,
    2 Ttl type DWORD;

 define alias LPGOPHER_TTL_ATTRIBUTE_TYPE
        handle GOPHER_TTL_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_SCORE_ATTRIBUTE_TYPE,
    2 Score type INT;

 define alias LPGOPHER_SCORE_ATTRIBUTE_TYPE
        handle GOPHER_SCORE_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_SCORE_RANGE_ATTRIBUTE_TYPE,
    2 LowerBound type INT,
    2 UpperBound type INT;

 define alias LPGOPHER_SCORE_RANGE_ATTRIBUTE_TYPE
        handle GOPHER_SCORE_RANGE_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_SITE_ATTRIBUTE_TYPE,
    2 Site type LPCTSTR;

 define alias LPGOPHER_SITE_ATTRIBUTE_TYPE
        handle GOPHER_SITE_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_ORGANIZATION_ATTRIBUTE_TYPE,
    2 Organization type LPCTSTR;

 define alias LPGOPHER_ORGANIZATION_ATTRIBUTE_TYPE
        handle GOPHER_ORGANIZATION_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_LOCATION_ATTRIBUTE_TYPE,
    2 Location type LPCTSTR;

 define alias LPGOPHER_LOCATION_ATTRIBUTE_TYPE
        handle GOPHER_LOCATION_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_GEOGRAPHICAL_LOCATION_ATTRIBUTE_TYPE,
    2 DegreesNorth type INT,
    2 MinutesNorth type INT,
    2 SecondsNorth type INT,
    2 DegreesEast  type INT,
    2 MinutesEast  type INT,
    2 SecondsEast  type INT;

 define alias  LPGOPHER_GEOGRAPHICAL_LOCATION_ATTRIBUTE_TYPE
        handle GOPHER_GEOGRAPHICAL_LOCATION_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_TIMEZONE_ATTRIBUTE_TYPE,
    2 Zone type INT;

 define alias LPGOPHER_TIMEZONE_ATTRIBUTE_TYPE
        handle GOPHER_TIMEZONE_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_PROVIDER_ATTRIBUTE_TYPE,
    2 Provider type LPCTSTR;

 define alias LPGOPHER_PROVIDER_ATTRIBUTE_TYPE
        handle GOPHER_PROVIDER_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_VERSION_ATTRIBUTE_TYPE,
    2 Version type LPCTSTR;

 define alias LPGOPHER_VERSION_ATTRIBUTE_TYPE
        handle GOPHER_VERSION_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_ABSTRACT_ATTRIBUTE_TYPE,
    2 ShortAbstract type LPCTSTR,
    2 AbstractFile  type LPCTSTR;

 define alias LPGOPHER_ABSTRACT_ATTRIBUTE_TYPE
        handle GOPHER_ABSTRACT_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_VIEW_ATTRIBUTE_TYPE,
    2 ContentType type LPCTSTR,
    2 Language    type LPCTSTR,
    2 Size        type DWORD;

 define alias LPGOPHER_VIEW_ATTRIBUTE_TYPE
        handle GOPHER_VIEW_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_VERONICA_ATTRIBUTE_TYPE,
    2 TreeWalk type BOOL;

 define alias LPGOPHER_VERONICA_ATTRIBUTE_TYPE
        handle GOPHER_VERONICA_ATTRIBUTE_TYPE;

 define structure
   1 GOPHER_ASK_ATTRIBUTE_TYPE,
    2 QuestionType type LPCTSTR,
    2 QuestionText type LPCTSTR;

 define alias LPGOPHER_ASK_ATTRIBUTE_TYPE
        handle GOPHER_ASK_ATTRIBUTE_TYPE;

 /* GOPHER_UNKNOWN_ATTRIBUTE_TYPE - this is returned if we retrieve   */
 /* an attribute that is not specified in the current gopher/gopher+  */
 /* documentation.                                                    */
 /* It is up to the application to parse the information.             */

 define structure
   1 GOPHER_UNKNOWN_ATTRIBUTE_TYPE,
     2 Text type LPCTSTR;

 define alias LPGOPHER_UNKNOWN_ATTRIBUTE_TYPE
        handle GOPHER_UNKNOWN_ATTRIBUTE_TYPE;


 /* GOPHER_ATTRIBUTE_TYPE - returned in the user's buffer when an */
 /* enumerated GopherGetAttribute call is made               */

 define structure
   1 GOPHER_ATTRIBUTE_TYPE,
    2 CategoryId     type DWORD,/* e.g. GOPHER_CATEGORY_ID_ADMIN */
    2 AttributeId    type DWORD,/* e.g. GOPHER_ATTRIBUTE_ID_ADMIN */
    2 AttributeType  union,
      3 Admin        type GOPHER_ADMIN_ATTRIBUTE_TYPE,
      3 ModDate      type GOPHER_MOD_DATE_ATTRIBUTE_TYPE,
      3 Ttl          type GOPHER_TTL_ATTRIBUTE_TYPE,
      3 Score        type GOPHER_SCORE_ATTRIBUTE_TYPE,
      3 ScoreRange   type GOPHER_SCORE_RANGE_ATTRIBUTE_TYPE,
      3 Site         type GOPHER_SITE_ATTRIBUTE_TYPE,
      3 Organization type GOPHER_ORGANIZATION_ATTRIBUTE_TYPE,
      3 Location     type GOPHER_LOCATION_ATTRIBUTE_TYPE,
      3 GeographicalLocation
                     type GOPHER_GEOGRAPHICAL_LOCATION_ATTRIBUTE_TYPE,
      3 TimeZone     type GOPHER_TIMEZONE_ATTRIBUTE_TYPE,
      3 Provider     type GOPHER_PROVIDER_ATTRIBUTE_TYPE,
      3 Version      type GOPHER_VERSION_ATTRIBUTE_TYPE,
      3 Abstract     type GOPHER_ABSTRACT_ATTRIBUTE_TYPE,
      3 View         type GOPHER_VIEW_ATTRIBUTE_TYPE,
      3 Veronica     type GOPHER_VERONICA_ATTRIBUTE_TYPE,
      3 Ask          type GOPHER_ASK_ATTRIBUTE_TYPE,
      3 Unknown      type GOPHER_UNKNOWN_ATTRIBUTE_TYPE;

 define alias LPGOPHER_ATTRIBUTE_TYPE handle GOPHER_ATTRIBUTE_TYPE;

 dcl MAX_GOPHER_CATEGORY_NAME    fixed bin(31) value(128),
     /* arbitrary */
     MAX_GOPHER_ATTRIBUTE_NAME   fixed bin(31) value(128),
     /*     " */
     MIN_GOPHER_ATTRIBUTE_LENGTH fixed bin(31) value(256);
     /*     " */

 /* known gopher attribute categories. See below for ordinals */

 dcl GOPHER_INFO_CATEGORY      TCHAR value('+INFO'),
     GOPHER_ADMIN_CATEGORY     TCHAR value('+ADMIN'),
     GOPHER_VIEWS_CATEGORY     TCHAR value('+VIEWS'),
     GOPHER_ABSTRACT_CATEGORY  TCHAR value('+ABSTRACT'),
     GOPHER_VERONICA_CATEGORY  TCHAR value('+VERONICA');

 /* known gopher attributes. These are the attribute names as defined */
 /* in the gopher+ protocol document                                  */

 dcl GOPHER_ADMIN_ATTRIBUTE    TCHAR value('Admin'),
     GOPHER_MOD_DATE_ATTRIBUTE TCHAR value('Mod-Date'),
     GOPHER_TTL_ATTRIBUTE      TCHAR value('TTL'),
     GOPHER_SCORE_ATTRIBUTE    TCHAR value('Score'),
     GOPHER_RANGE_ATTRIBUTE    TCHAR value('Score-range'),
     GOPHER_SITE_ATTRIBUTE     TCHAR value('Site'),
     GOPHER_ORG_ATTRIBUTE      TCHAR value('Org'),
     GOPHER_LOCATION_ATTRIBUTE TCHAR value('Loc'),
     GOPHER_GEOG_ATTRIBUTE     TCHAR value('Geog'),
     GOPHER_TIMEZONE_ATTRIBUTE TCHAR value('TZ'),
     GOPHER_PROVIDER_ATTRIBUTE TCHAR value('Provider'),
     GOPHER_VERSION_ATTRIBUTE  TCHAR value('Version'),
     GOPHER_ABSTRACT_ATTRIBUTE TCHAR value('Abstract'),
     GOPHER_VIEW_ATTRIBUTE     TCHAR value('View'),
     GOPHER_TREEWALK_ATTRIBUTE TCHAR value('treewalk');

 /* identifiers for attribute strings */

 dcl ( GOPHER_ATTRIBUTE_ID_BASE     value('abcccc00'xn)
     , GOPHER_CATEGORY_ID_ALL       value(GOPHER_ATTRIBUTE_ID_BASE+ 1)
     , GOPHER_CATEGORY_ID_INFO      value(GOPHER_ATTRIBUTE_ID_BASE+ 2)
     , GOPHER_CATEGORY_ID_ADMIN     value(GOPHER_ATTRIBUTE_ID_BASE+ 3)
     , GOPHER_CATEGORY_ID_VIEWS     value(GOPHER_ATTRIBUTE_ID_BASE+ 4)
     , GOPHER_CATEGORY_ID_ABSTRACT  value(GOPHER_ATTRIBUTE_ID_BASE+ 5)
     , GOPHER_CATEGORY_ID_VERONICA  value(GOPHER_ATTRIBUTE_ID_BASE+ 6)
     , GOPHER_CATEGORY_ID_ASK       value(GOPHER_ATTRIBUTE_ID_BASE+ 7)
     , GOPHER_CATEGORY_ID_UNKNOWN   value(GOPHER_ATTRIBUTE_ID_BASE+ 8)
     , GOPHER_ATTRIBUTE_ID_ALL      value(GOPHER_ATTRIBUTE_ID_BASE+ 9)
     , GOPHER_ATTRIBUTE_ID_ADMIN    value(GOPHER_ATTRIBUTE_ID_BASE+10)
     , GOPHER_ATTRIBUTE_ID_MOD_DATE value(GOPHER_ATTRIBUTE_ID_BASE+11)
     , GOPHER_ATTRIBUTE_ID_TTL      value(GOPHER_ATTRIBUTE_ID_BASE+12)
     , GOPHER_ATTRIBUTE_ID_SCORE    value(GOPHER_ATTRIBUTE_ID_BASE+13)
     , GOPHER_ATTRIBUTE_ID_RANGE    value(GOPHER_ATTRIBUTE_ID_BASE+14)
     , GOPHER_ATTRIBUTE_ID_SITE     value(GOPHER_ATTRIBUTE_ID_BASE+15)
     , GOPHER_ATTRIBUTE_ID_ORG      value(GOPHER_ATTRIBUTE_ID_BASE+16)
     , GOPHER_ATTRIBUTE_ID_LOCATION value(GOPHER_ATTRIBUTE_ID_BASE+17)
     , GOPHER_ATTRIBUTE_ID_GEOG     value(GOPHER_ATTRIBUTE_ID_BASE+18)
     , GOPHER_ATTRIBUTE_ID_TIMEZONE value(GOPHER_ATTRIBUTE_ID_BASE+19)
     , GOPHER_ATTRIBUTE_ID_PROVIDER value(GOPHER_ATTRIBUTE_ID_BASE+20)
     , GOPHER_ATTRIBUTE_ID_VERSION  value(GOPHER_ATTRIBUTE_ID_BASE+21)
     , GOPHER_ATTRIBUTE_ID_ABSTRACT value(GOPHER_ATTRIBUTE_ID_BASE+22)
     , GOPHER_ATTRIBUTE_ID_VIEW     value(GOPHER_ATTRIBUTE_ID_BASE+23)
     , GOPHER_ATTRIBUTE_ID_TREEWALK value(GOPHER_ATTRIBUTE_ID_BASE+24)
     , GOPHER_ATTRIBUTE_ID_UNKNOWN  value(GOPHER_ATTRIBUTE_ID_BASE+25)
     ) fixed bin(31);

 /* prototypes */

 dcl GopherCreateLocatorA
     entry (char (*) varz nonasgn byaddr,
            type INTERNET_PORT,
            char (*) varz nonasgn byaddr optional,
            char (*) varz nonasgn byaddr optional,
            type DWORD,
            char (*) varz byaddr optional,
            type DWORD byaddr)
     BOOLAPI external ('GopherCreateLocatorA');

 dcl GopherCreateLocatorW
     entry (WCHAR(*) varz byaddr nonasgn,
            type INTERNET_PORT,
            WCHAR(*) varz byaddr nonasgn optional,
            WCHAR(*) varz byaddr nonasgn optional,
            type DWORD,
            WCHAR(*) varz byaddr optional,
            type DWORD byaddr)
     BOOLAPI external ('GopherCreateLocatorW');
 %dcl GopherCreateLocator char ext;
 %if UNICODE ^= '' %then %do;
 %GopherCreateLocator = 'GopherCreateLocatorW';
 %end; %else %do;
 %GopherCreateLocator = 'GopherCreateLocatorA';
 %end; /* !UNICODE */


 dcl GopherGetLocatorTypeA
     entry (char (*) varz nonasgn byaddr,
            type DWORD byaddr)
     BOOLAPI external ('GopherGetLocatorTypeA');

 dcl GopherGetLocatorTypeW
     entry (WCHAR(*) varz byaddr nonasgn,
            type DWORD byaddr)
     BOOLAPI external ('GopherGetLocatorTypeW');
 %dcl GopherGetLocatorType char ext;
 %if UNICODE ^= '' %then %do;
 %GopherGetLocatorType = 'GopherGetLocatorTypeW';
 %end; %else %do;
 %GopherGetLocatorType = 'GopherGetLocatorTypeA';
 %end; /* !UNICODE */


 dcl GopherFindFirstFileA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr optional,
            char (*) varz nonasgn byaddr optional,
            type GOPHER_FIND_DATAA byaddr optional,
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('GopherFindFirstFileA');

 dcl GopherFindFirstFileW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn optional,
            WCHAR(*) varz byaddr nonasgn optional,
            type GOPHER_FIND_DATAW byaddr optional,
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('GopherFindFirstFileW');
 %dcl GopherFindFirstFile char ext;
 %if UNICODE ^= '' %then %do;
 %GopherFindFirstFile = 'GopherFindFirstFileW';
 %end; %else %do;
 %GopherFindFirstFile = 'GopherFindFirstFileA';
 %end; /* !UNICODE */


 dcl GopherOpenFileA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr optional,
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('GopherOpenFileA');

 dcl GopherOpenFileW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn optional,
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('GopherOpenFileW');
 %dcl GopherOpenFile char ext;
 %if UNICODE ^= '' %then %do;
 %GopherOpenFile = 'GopherOpenFileW';
 %end; %else %do;
 %GopherOpenFile = 'GopherOpenFileA';
 %end;
 /* !UNICODE */

 define alias GOPHER_ATTRIBUTE_ENUMERATOR
    entry (type GOPHER_ATTRIBUTE_TYPE byaddr,
           type DWORD)
    returns (byvalue optional type BOOL)
    limited CALLBACK;


 dcl GopherGetAttributeA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr optional,
            type LPBYTE,
            type DWORD,
            type DWORD byaddr,
            type GOPHER_ATTRIBUTE_ENUMERATOR optional byvalue,
            type DWORD)
     BOOLAPI external ('GopherGetAttributeA');

 dcl GopherGetAttributeW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn optional,
            type LPBYTE,
            type DWORD,
            type DWORD byaddr,
            type GOPHER_ATTRIBUTE_ENUMERATOR optional byvalue,
            type DWORD)
     BOOLAPI external ('GopherGetAttributeW');
 %dcl GopherGetAttribute char ext;
 %if UNICODE ^= '' %then %do;
 %GopherGetAttribute = 'GopherGetAttributeW';
 %end; %else %do;
 %GopherGetAttribute = 'GopherGetAttributeA';
 %end;
 /* !UNICODE */

 /* HTTP */

 /* manifests */

 /* the default major/minor HTTP version numbers */

 dcl HTTP_MAJOR_VERSION fixed bin(31) value(1),
     HTTP_MINOR_VERSION fixed bin(31) value(0);

 dcl HTTP_VERSIONA      char  value('HTTP/1.0');
 dcl HTTP_VERSIONW      WCHAR value('HTTP/1.0');

 %dcl HTTP_VERSION char ext;
 %if UNICODE ^= '' %then %do;
 %HTTP_VERSION = 'HTTP_VERSIONW';
 %end; %else %do;
 %HTTP_VERSION = 'HTTP_VERSIONA';
 %end;

 /* HttpQueryInfo info levels. Generally, there is one info level  */
 /* for each potential RFC822/HTTP/MIME header that an HTTP server */
 /* may send as part of a request response.                        */

 /* The HTTP_QUERY_RAW_HEADERS info level is provided for clients  */
 /* that choose to perform their own header parsing.               */

 dcl HTTP_QUERY_MIME_VERSION              type DWORD value(0),
     HTTP_QUERY_CONTENT_TYPE              type DWORD value(1),
     HTTP_QUERY_CONTENT_TRANSFER_ENCODING type DWORD value(2),
     HTTP_QUERY_CONTENT_ID                type DWORD value(3),
     HTTP_QUERY_CONTENT_DESCRIPTION       type DWORD value(4),
     HTTP_QUERY_CONTENT_LENGTH            type DWORD value(5),
     HTTP_QUERY_CONTENT_LANGUAGE          type DWORD value(6),
     HTTP_QUERY_ALLOW                     type DWORD value(7),
     HTTP_QUERY_PUBLIC                    type DWORD value(8),
     HTTP_QUERY_DATE                      type DWORD value(9),
     HTTP_QUERY_EXPIRES                   type DWORD value(10),
     HTTP_QUERY_LAST_MODIFIED             type DWORD value(11),
     HTTP_QUERY_MESSAGE_ID                type DWORD value(12),
     HTTP_QUERY_URI                       type DWORD value(13),
     HTTP_QUERY_DERIVED_FROM              type DWORD value(14),
     HTTP_QUERY_COST                      type DWORD value(15),
     HTTP_QUERY_LINK                      type DWORD value(16),
     HTTP_QUERY_PRAGMA                    type DWORD value(17),
     HTTP_QUERY_VERSION                   type DWORD value(18),
                              /* special: part of status line */
     HTTP_QUERY_STATUS_CODE               type DWORD value(19),
                              /* special: part of status line */
     HTTP_QUERY_STATUS_TEXT               type DWORD value(20),
                              /* special: part of status line */
     HTTP_QUERY_RAW_HEADERS               type DWORD value(21),
                              /* special: all headers as ASCIIZ */
     HTTP_QUERY_RAW_HEADERS_CRLF          type DWORD value(22),
                              /* special: all headers */
     HTTP_QUERY_CONNECTION                type DWORD value(23),
     HTTP_QUERY_ACCEPT                    type DWORD value(24),
     HTTP_QUERY_ACCEPT_CHARSET            type DWORD value(25),
     HTTP_QUERY_ACCEPT_ENCODING           type DWORD value(26),
     HTTP_QUERY_ACCEPT_LANGUAGE           type DWORD value(27),
     HTTP_QUERY_AUTHORIZATION             type DWORD value(28),
     HTTP_QUERY_CONTENT_ENCODING          type DWORD value(29),
     HTTP_QUERY_FORWARDED                 type DWORD value(30),
     HTTP_QUERY_FROM                      type DWORD value(31),
     HTTP_QUERY_IF_MODIFIED_SINCE         type DWORD value(32),
     HTTP_QUERY_LOCATION                  type DWORD value(33),
     HTTP_QUERY_ORIG_URI                  type DWORD value(34),
     HTTP_QUERY_REFERER                   type DWORD value(35),
     HTTP_QUERY_RETRY_AFTER               type DWORD value(36),
     HTTP_QUERY_SERVER                    type DWORD value(37),
     HTTP_QUERY_TITLE                     type DWORD value(38),
     HTTP_QUERY_USER_AGENT                type DWORD value(39),
     HTTP_QUERY_WWW_AUTHENTICATE          type DWORD value(40),
     HTTP_QUERY_PROXY_AUTHENTICATE        type DWORD value(41),
     HTTP_QUERY_ACCEPT_RANGES             type DWORD value(42),
     HTTP_QUERY_SET_COOKIE                type DWORD value(43),
     HTTP_QUERY_COOKIE                    type DWORD value(44),
     HTTP_QUERY_REQUEST_METHOD            type DWORD value(45),
                              /* special: GET/POST etc. */
     HTTP_QUERY_REFRESH                   type DWORD value(46),
     HTTP_QUERY_CONTENT_DISPOSITION       type DWORD value(47),

 /* HTTP 1.1 defined headers */
     HTTP_QUERY_AGE                       type DWORD value(48),
     HTTP_QUERY_CACHE_CONTROL             type DWORD value(49),
     HTTP_QUERY_CONTENT_BASE              type DWORD value(50),
     HTTP_QUERY_CONTENT_LOCATION          type DWORD value(51),
     HTTP_QUERY_CONTENT_MD5               type DWORD value(52),
     HTTP_QUERY_CONTENT_RANGE             type DWORD value(53),
     HTTP_QUERY_ETAG                      type DWORD value(54),
     HTTP_QUERY_HOST                      type DWORD value(55),
     HTTP_QUERY_IF_MATCH                  type DWORD value(56),
     HTTP_QUERY_IF_NONE_MATCH             type DWORD value(57),
     HTTP_QUERY_IF_RANGE                  type DWORD value(58),
     HTTP_QUERY_IF_UNMODIFIED_SINCE       type DWORD value(59),
     HTTP_QUERY_MAX_FORWARDS              type DWORD value(60),
     HTTP_QUERY_PROXY_AUTHORIZATION       type DWORD value(61),
     HTTP_QUERY_RANGE                     type DWORD value(62),
     HTTP_QUERY_TRANSFER_ENCODING         type DWORD value(63),
     HTTP_QUERY_UPGRADE                   type DWORD value(64),
     HTTP_QUERY_VARY                      type DWORD value(65),
     HTTP_QUERY_VIA                       type DWORD value(66),
     HTTP_QUERY_WARNING                   type DWORD value(67),
     HTTP_QUERY_EXPECT                    type DWORD value(68),
     HTTP_QUERY_PROXY_CONNECTION          type DWORD value(69),
     HTTP_QUERY_UNLESS_MODIFIED_SINCE     type DWORD value(70),
     HTTP_QUERY_ECHO_REQUEST              type DWORD value(71),
     HTTP_QUERY_ECHO_REPLY                type DWORD value(72),

 /* These are the set of headers that should be added back to a  */
 /* request when re-doing a request after a RETRY_WITH response. */
     HTTP_QUERY_ECHO_HEADERS              type DWORD value(73),
     HTTP_QUERY_ECHO_HEADERS_CRLF         type DWORD value(74),

     HTTP_QUERY_MAX                       type DWORD value(74);

 /* HTTP_QUERY_CUSTOM - if this special value is supplied as the   */
 /* dwInfoLevel parameter of HttpQueryInfo() then the lpBuffer     */
 /* parameter contains the name of the header we are to query      */
 dcl HTTP_QUERY_CUSTOM                    type DWORD value(65535);

 /* HTTP_QUERY_FLAG_REQUEST_HEADERS - if this bit is set in the    */
 /* dwInfoLevel parameter of HttpQueryInfo() then the request      */
 /* headers will be queried for the request information            */
 dcl HTTP_QUERY_FLAG_REQUEST_HEADERS type DWORD value('80000000'xu);

 /* HTTP_QUERY_FLAG_SYSTEMTIME - if this bit is set in the         */
 /* dwInfoLevel parameter of HttpQueryInfo() AND the header being  */
 /* queried contains date information, e.g. the "Expires:" header  */
 /* then lpBuffer will contain a SYSTEMTIME structure containing   */
 /* the date and time information converted from the header string */
 dcl HTTP_QUERY_FLAG_SYSTEMTIME      type DWORD value('40000000'xu);

 /* HTTP_QUERY_FLAG_NUMBER - if this bit is set in the dwInfoLevel */
 /* parameter of HttpQueryInfo(), then the value of the header     */
 /* will be converted to a number before being returned to the     */
 /* caller, if applicable                                          */
 dcl HTTP_QUERY_FLAG_NUMBER          type DWORD value('20000000'xu);

 /* HTTP_QUERY_FLAG_COALESCE - combine the values from several     */
 /* headers of the same name into the output buffer                */
 dcl HTTP_QUERY_FLAG_COALESCE        type DWORD value('10000000'xu);

 dcl HTTP_QUERY_MODIFIER_FLAGS_MASK  type DWORD
     value (ior (HTTP_QUERY_FLAG_REQUEST_HEADERS,
                 HTTP_QUERY_FLAG_SYSTEMTIME,
                 HTTP_QUERY_FLAG_NUMBER,
                 HTTP_QUERY_FLAG_COALESCE));

 dcl HTTP_QUERY_HEADER_MASK          type DWORD
     value (inot (HTTP_QUERY_MODIFIER_FLAGS_MASK));

 /* HTTP Response Status Codes: */

 dcl HTTP_STATUS_CONTINUE           type DWORD value(100),
          /* OK to continue with request */
     HTTP_STATUS_SWITCH_PROTOCOLS   type DWORD value(101),
          /* server has switched protocols in upgrade header */

     HTTP_STATUS_OK                 type DWORD value(200),
          /* request completed */
     HTTP_STATUS_CREATED            type DWORD value(201),
          /* object created, reason = new URI */
     HTTP_STATUS_ACCEPTED           type DWORD value(202),
          /* async completion (TBS) */
     HTTP_STATUS_PARTIAL            type DWORD value(203),
          /* partial completion */
     HTTP_STATUS_NO_CONTENT         type DWORD value(204),
          /* no info to return */
     HTTP_STATUS_RESET_CONTENT      type DWORD value(205),
          /* request completed, but clear form */
     HTTP_STATUS_PARTIAL_CONTENT    type DWORD value(206),
          /* partial GET furfilled */

     HTTP_STATUS_AMBIGUOUS          type DWORD value(300),
          /* server couldn't decide what to return */
     HTTP_STATUS_MOVED              type DWORD value(301),
          /* object permanently moved */
     HTTP_STATUS_REDIRECT           type DWORD value(302),
          /* object temporarily moved */
     HTTP_STATUS_REDIRECT_METHOD    type DWORD value(303),
          /* redirection w/ new access method */
     HTTP_STATUS_NOT_MODIFIED       type DWORD value(304),
          /* if-modified-since was not modified */
     HTTP_STATUS_USE_PROXY          type DWORD value(305),
          /* redirection to proxy, locn header specifies proxy to use */
     HTTP_STATUS_REDIRECT_KEEP_VERB type DWORD value(307),
          /* HTTP/1.1: keep same verb */

     HTTP_STATUS_BAD_REQUEST        type DWORD value(400),
          /* invalid syntax */
     HTTP_STATUS_DENIED             type DWORD value(401),
          /* access denied */
     HTTP_STATUS_PAYMENT_REQ        type DWORD value(402),
          /* payment required */
     HTTP_STATUS_FORBIDDEN          type DWORD value(403),
          /* request forbidden */
     HTTP_STATUS_NOT_FOUND          type DWORD value(404),
          /* object not found */
     HTTP_STATUS_BAD_METHOD         type DWORD value(405),
          /* method is not allowed */
     HTTP_STATUS_NONE_ACCEPTABLE    type DWORD value(406),
          /* no response acceptable to client found */
     HTTP_STATUS_PROXY_AUTH_REQ     type DWORD value(407),
          /* proxy authentication required */
     HTTP_STATUS_REQUEST_TIMEOUT    type DWORD value(408),
          /* server timed out waiting for request */
     HTTP_STATUS_CONFLICT           type DWORD value(409),
          /* user should resubmit with more info */
     HTTP_STATUS_GONE               type DWORD value(410),
          /* the resource is no longer available */
     HTTP_STATUS_LENGTH_REQUIRED    type DWORD value(411),
          /* the server refused to accept request w/o a length */
     HTTP_STATUS_PRECOND_FAILED     type DWORD value(412),
          /* precondition given in request failed */
     HTTP_STATUS_REQUEST_TOO_LARGE  type DWORD value(413),
          /* request entity was too large */
     HTTP_STATUS_URI_TOO_LONG       type DWORD value(414),
          /* request URI too long */
     HTTP_STATUS_UNSUPPORTED_MEDIA  type DWORD value(415),
          /* unsupported media type */
     HTTP_STATUS_RETRY_WITH         type DWORD value(449),
          /* retry after doing the appropriate action. */


     HTTP_STATUS_SERVER_ERROR       type DWORD value(500),
          /* internal server error */
     HTTP_STATUS_NOT_SUPPORTED      type DWORD value(501),
          /* required not supported */
     HTTP_STATUS_BAD_GATEWAY        type DWORD value(502),
          /* error response received from gateway */
     HTTP_STATUS_SERVICE_UNAVAIL    type DWORD value(503),
          /* temporarily overloaded */
     HTTP_STATUS_GATEWAY_TIMEOUT    type DWORD value(504),
          /* timed out waiting for gateway */
     HTTP_STATUS_VERSION_NOT_SUP    type DWORD value(505),
          /* HTTP version not supported */

     HTTP_STATUS_FIRST type DWORD value(HTTP_STATUS_CONTINUE),
     HTTP_STATUS_LAST  type DWORD value(HTTP_STATUS_VERSION_NOT_SUP);

 /* prototypes */

 dcl HttpOpenRequestA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr optional,
            (*) type LPCSTR byaddr optional,     /* to char (*) varz */
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('HttpOpenRequestA');
 dcl HttpOpenRequestW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn optional,
            (*) type LPCWSTR byaddr optional,    /* to WCHAR (*) varz */
            type DWORD,
            type DWORD)
     returns (byvalue optional type HINTERNET)
     INTERNETAPI WINAPI external ('HttpOpenRequestW');
 %dcl HttpOpenRequest char ext;
 %if UNICODE ^= '' %then %do;
 %HttpOpenRequest = 'HttpOpenRequestW';
 %end; %else %do;
 %HttpOpenRequest = 'HttpOpenRequestA';
 %end; /* !UNICODE */

 dcl HttpAddRequestHeadersA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD)
     BOOLAPI external ('HttpAddRequestHeadersA');
 dcl HttpAddRequestHeadersW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn,
            type DWORD,
            type DWORD)
     BOOLAPI external ('HttpAddRequestHeadersW');
 %dcl HttpAddRequestHeaders char ext;
 %if UNICODE ^= '' %then %do;
 %HttpAddRequestHeaders = 'HttpAddRequestHeadersW';
 %end; %else %do;
 %HttpAddRequestHeaders = 'HttpAddRequestHeadersA';
 %end;
 /* !UNICODE */

 /* values for dwModifiers parameter of HttpAddRequestHeaders() */

 dcl HTTP_ADDREQ_INDEX_MASK          type DWORD value('0000FFFF'xu),
     HTTP_ADDREQ_FLAGS_MASK          type DWORD value('FFFF0000'xu);

 /* HTTP_ADDREQ_FLAG_ADD_IF_NEW - the header will only be added if it */
 /* doesn't already exist                                             */
 dcl HTTP_ADDREQ_FLAG_ADD_IF_NEW     type DWORD value('10000000'xu);

 /* HTTP_ADDREQ_FLAG_ADD - if HTTP_ADDREQ_FLAG_REPLACE is set but the */
 /* header is not found then if this flag is set, the header is added */
 /* anyway, so long as there is a valid header-value                  */
 dcl HTTP_ADDREQ_FLAG_ADD            type DWORD value('20000000'xu);

 /* HTTP_ADDREQ_FLAG_COALESCE - coalesce headers with same name. e.g. */
 /* "Accept: text/*" and "Accept: audio/*" with this flag results in  */
 /* a single header: "Accept: text/..., audio/..."                    */
 /* PL/I: HTTP_ADDREQ_FLAG_COALESCE_WITH_* shortened to */
 /*       HTTP_ADDREQ_FLAG_COAL_*                       */
 dcl HTTP_ADDREQ_FLAG_COAL_COMMA     type DWORD value('40000000'xu),
     HTTP_ADDREQ_FLAG_COAL_SEMICOLON type DWORD value('01000000'xu),
     HTTP_ADDREQ_FLAG_COALESCE       type DWORD
          value(HTTP_ADDREQ_FLAG_COAL_COMMA);

 /* HTTP_ADDREQ_FLAG_REPLACE - replaces the specified header.         */
 /* Only one header can be supplied in the buffer.                    */
 /* If the header to be replaced is not the first in a list of        */
 /* headers with the same name, then the relative index should be     */
 /* supplied in the low 8 bits of the dwModifiers parameter.          */
 /* If the header-value part is missing, then the header is removed   */
 dcl HTTP_ADDREQ_FLAG_REPLACE        type DWORD value('80000000'xu);


 dcl HttpSendRequestA
     entry (type HINTERNET,
            char (*) varz nonasgn byaddr optional,
            type DWORD,
            type LPVOID,
            type DWORD)
     BOOLAPI external ('HttpSendRequestA');
 dcl HttpSendRequestW
     entry (type HINTERNET,
            WCHAR(*) varz byaddr nonasgn optional,
            type DWORD,
            type LPVOID,
            type DWORD)
     BOOLAPI external ('HttpSendRequestW');
 %dcl HttpSendRequest char ext;
 %if UNICODE ^= '' %then %do;
 %HttpSendRequest = 'HttpSendRequestW';
 %end; %else %do;
 %HttpSendRequest = 'HttpSendRequestA';
 %end; /* !UNICODE */

 dcl HttpSendRequestExA
     entry (type HINTERNET,
            type INTERNET_BUFFERSA nonasgn byaddr optional,
            type INTERNET_BUFFERSA byaddr optional,
            type DWORD,
            type DWORD)
     BOOLAPI external ('HttpSendRequestExA');

 dcl HttpSendRequestExW
     entry (type HINTERNET,
            type INTERNET_BUFFERSW nonasgn byaddr optional,
            type INTERNET_BUFFERSW byaddr optional,
            type DWORD,
            type DWORD)
     BOOLAPI external ('HttpSendRequestExW');
 %dcl HttpSendRequestEx char ext;
 %if UNICODE ^= '' %then %do;
 %HttpSendRequestEx = 'HttpSendRequestExW';
 %end; %else %do;
 %HttpSendRequestEx = 'HttpSendRequestExA';
 %end; /* !UNICODE */

 /* flags for HttpSendRequestEx(), HttpEndRequest() */

 dcl ( HSR_ASYNC       value(WININET_API_FLAG_ASYNC) /* force async */
     , HSR_SYNC        value(WININET_API_FLAG_SYNC)  /* force sync  */
     , HSR_USE_CONTEXT value(WININET_API_FLAG_USE_CONTEXT)
          /* use dwContext value */
     , HSR_INITIATE    value('00000008'xu)
          /* iterative operation (completed by HttpEndRequest) */
     , HSR_DOWNLOAD    value('00000010'xu)   /* download to file */
     , HSR_CHUNKED     value('00000020'xu)
          /* operation is send of chunked data */
     ) type DWORD;

 dcl HttpEndRequestA
     entry (type HINTERNET,
            type INTERNET_BUFFERSA byaddr optional,
            type DWORD,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('HttpEndRequestA');
 dcl HttpEndRequestW
     entry (type HINTERNET,
            type INTERNET_BUFFERSW byaddr optional,
            type DWORD,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('HttpEndRequestW');
 %dcl HttpEndRequest char ext;
 %if UNICODE ^= '' %then %do;
 %HttpEndRequest = 'HttpEndRequestW';
 %end; %else %do;
 %HttpEndRequest = 'HttpEndRequestA';
 %end; /* !UNICODE */


 dcl HttpQueryInfoA
     entry (type HINTERNET,
            type DWORD,
            type LPVOID,
            type DWORD byaddr,
            type DWORD byaddr optional)
     BOOLAPI external ('HttpQueryInfoA');
 dcl HttpQueryInfoW
     entry (type HINTERNET,
            type DWORD,
            type LPVOID,
            type DWORD byaddr,
            type DWORD byaddr optional)
     BOOLAPI external ('HttpQueryInfoW');
 %dcl HttpQueryInfo char ext;
 %if UNICODE ^= '' %then %do;
 %HttpQueryInfo = 'HttpQueryInfoW';
 %end; %else %do;
 %HttpQueryInfo = 'HttpQueryInfoA';
 %end; /* !UNICODE */


 /* Cookie APIs */

 dcl InternetSetCookieA
     entry (char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr)
     BOOLAPI external ('InternetSetCookieA');
 dcl InternetSetCookieW
     entry (WCHAR (*) varz nonasgn byaddr,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn)
     BOOLAPI external ('InternetSetCookieW');
 %dcl InternetSetCookie char ext;
 %if UNICODE ^= '' %then %do;
 %InternetSetCookie = 'InternetSetCookieW';
 %end; %else %do;
 %InternetSetCookie = 'InternetSetCookieA';
 %end; /* !UNICODE */

 dcl InternetGetCookieA
     entry (char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr,
            char (*) varz byaddr,
            type DWORD byaddr)
     BOOLAPI external ('InternetGetCookieA');
 dcl InternetGetCookieW
     entry (WCHAR (*) varz nonasgn byaddr,
            WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr,
            type DWORD byaddr)
     BOOLAPI external ('InternetGetCookieW');
 %dcl InternetGetCookie char ext;
 %if UNICODE ^= '' %then %do;
 %InternetGetCookie = 'InternetGetCookieW';
 %end; %else %do;
 %InternetGetCookie = 'InternetGetCookieA';
 %end; /* !UNICODE */

 /* offline browsing */

 dcl InternetAttemptConnect
     entry (type DWORD)
     returns (byvalue optional type DWORD)
     INTERNETAPI WINAPI external ('InternetAttemptConnect');

 dcl InternetCheckConnectionA
     entry (char (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD)
     BOOLAPI external ('InternetCheckConnectionA');
 dcl InternetCheckConnectionW
     entry (WCHAR(*) varz byaddr nonasgn,
            type DWORD,
            type DWORD)
     BOOLAPI external ('InternetCheckConnectionW');
 %dcl InternetCheckConnection char ext;
 %if UNICODE ^= '' %then %do;
 %InternetCheckConnection = 'InternetCheckConnectionW';
 %end; %else %do;
 %InternetCheckConnection = 'InternetCheckConnectionA';
 %end; /* !UNICODE */

 dcl FLAG_ICC_FORCE_CONNECTION fixed bin (31) value ('00000001'xn);

 /* Internet UI */

 /* InternetErrorDlg - Provides UI for certain Errors. */
 dcl FLAGS_ERROR_UI_FILTER_FOR_ERRORS    type DWORD value('01'xn),
     FLAGS_ERROR_UI_FLAGS_CHANGE_OPTIONS type DWORD value('02'xn),
     FLAGS_ERROR_UI_FLAGS_GENERATE_DATA  type DWORD value('04'xn),
     FLAGS_ERROR_UI_FLAGS_NO_UI          type DWORD value('08'xn),
     FLAGS_ERROR_UI_SERIALIZE_DIALOGS    type DWORD value('10'xn);

 /* If SERIALIZE_DIALOGS flag set, client should implement */
 /* thread-safe non-blocking callback...                   */

 /* InternetAuthNotifyCallback is comment for user, hence omitted */

 define alias PFN_AUTH_NOTIFY
        entry (type DWORD_PTR,/* as passed to InternetErrorDlg */
               type DWORD,  /* error code: success, resend, or cancel */
               type LPVOID) /* reserved: will be set to null */
        returns (byvalue optional type DWORD)
        limited CALLBACK;

 /* ... and last parameter of InternetErrorDlg should point to... */

 define structure
   1 INTERNET_AUTH_NOTIFY_DATA,
     3 cbStruct  type DWORD, /* size of this structure */
     3 dwOptions type DWORD, /* reserved: must set to 0 */
     3 pfnNotify type PFN_AUTH_NOTIFY,
          /* notification callback to retry InternetErrorDlg */
     3 dwContext type DWORD;
          /* context to pass to notification function */

 dcl ResumeSuspendedDownload
     entry (type HINTERNET,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('ResumeSuspendedDownload');

 dcl InternetErrorDlg
     entry (type HWND,
            type HINTERNET,
            type DWORD,
            type DWORD,
            type LPVOID byaddr)
     returns (byvalue optional type DWORD)
     INTERNETAPI WINAPI external ('InternetErrorDlg');

 dcl InternetConfirmZoneCrossingA
     entry (type HWND,
            char (*) varz byaddr,
            char (*) varz byaddr,
            type BOOL)
     returns (byvalue optional type DWORD)
     INTERNETAPI WINAPI external ('InternetConfirmZoneCrossingA');
 dcl InternetConfirmZoneCrossingW
     entry (type HWND,
            WCHAR (*) varz byaddr,
            WCHAR (*) varz byaddr,
            type BOOL)
     returns (byvalue optional type DWORD)
     INTERNETAPI WINAPI external ('InternetConfirmZoneCrossingW');
 %dcl InternetConfirmZoneCrossing char ext;
 %if UNICODE ^= '' %then %do;
   %InternetConfirmZoneCrossing = 'InternetConfirmZoneCrossingW';
 %end; %else %if #WINX32_ ^= '' %then %do;
   %InternetConfirmZoneCrossing = 'InternetConfirmZoneCrossingA';
 %end; %else %do;
 %deact InternetConfirmZoneCrossing;
 dcl InternetConfirmZoneCrossing
     entry (type HWND,
            char (*) varz byaddr,
            char (*) varz byaddr,
            type BOOL)
     returns (byvalue optional type DWORD)
     INTERNETAPI WINAPI external ('InternetConfirmZoneCrossing');
 %end;


 /* Internet API error returns */

 dcl
 ( INTERNET_ERROR_BASE                     value(12000)
 , ERROR_INTERNET_OUT_OF_HANDLES           value(INTERNET_ERROR_BASE+ 1)
 , ERROR_INTERNET_TIMEOUT                  value(INTERNET_ERROR_BASE+ 2)
 , ERROR_INTERNET_EXTENDED_ERROR           value(INTERNET_ERROR_BASE+ 3)
 , ERROR_INTERNET_INTERNAL_ERROR           value(INTERNET_ERROR_BASE+ 4)
 , ERROR_INTERNET_INVALID_URL              value(INTERNET_ERROR_BASE+ 5)
 , ERROR_INTERNET_UNRECOGNIZED_SCHEME      value(INTERNET_ERROR_BASE+ 6)
 , ERROR_INTERNET_NAME_NOT_RESOLVED        value(INTERNET_ERROR_BASE+ 7)
 , ERROR_INTERNET_PROTOCOL_NOT_FOUND       value(INTERNET_ERROR_BASE+ 8)
 , ERROR_INTERNET_INVALID_OPTION           value(INTERNET_ERROR_BASE+ 9)
 , ERROR_INTERNET_BAD_OPTION_LENGTH        value(INTERNET_ERROR_BASE+10)
 , ERROR_INTERNET_OPTION_NOT_SETTABLE      value(INTERNET_ERROR_BASE+11)
 , ERROR_INTERNET_SHUTDOWN                 value(INTERNET_ERROR_BASE+12)
 , ERROR_INTERNET_INCORRECT_USER_NAME      value(INTERNET_ERROR_BASE+13)
 , ERROR_INTERNET_INCORRECT_PASSWORD       value(INTERNET_ERROR_BASE+14)
 , ERROR_INTERNET_LOGIN_FAILURE            value(INTERNET_ERROR_BASE+15)
 , ERROR_INTERNET_INVALID_OPERATION        value(INTERNET_ERROR_BASE+16)
 , ERROR_INTERNET_OPERATION_CANCELLED      value(INTERNET_ERROR_BASE+17)
 , ERROR_INTERNET_INCORRECT_HANDLE_TYPE    value(INTERNET_ERROR_BASE+18)
 , ERROR_INTERNET_INCORRECT_HANDLE_STATE   value(INTERNET_ERROR_BASE+19)
 , ERROR_INTERNET_NOT_PROXY_REQUEST        value(INTERNET_ERROR_BASE+20)
 , ERROR_INTERNET_REGISTRY_VALUE_NOT_FOUND value(INTERNET_ERROR_BASE+21)
 , ERROR_INTERNET_BAD_REGISTRY_PARAMETER   value(INTERNET_ERROR_BASE+22)
 , ERROR_INTERNET_NO_DIRECT_ACCESS         value(INTERNET_ERROR_BASE+23)
 , ERROR_INTERNET_NO_CONTEXT               value(INTERNET_ERROR_BASE+24)
 , ERROR_INTERNET_NO_CALLBACK              value(INTERNET_ERROR_BASE+25)
 , ERROR_INTERNET_REQUEST_PENDING          value(INTERNET_ERROR_BASE+26)
 , ERROR_INTERNET_INCORRECT_FORMAT         value(INTERNET_ERROR_BASE+27)
 , ERROR_INTERNET_ITEM_NOT_FOUND           value(INTERNET_ERROR_BASE+28)
 , ERROR_INTERNET_CANNOT_CONNECT           value(INTERNET_ERROR_BASE+29)
 , ERROR_INTERNET_CONNECTION_ABORTED       value(INTERNET_ERROR_BASE+30)
 , ERROR_INTERNET_CONNECTION_RESET         value(INTERNET_ERROR_BASE+31)
 , ERROR_INTERNET_FORCE_RETRY              value(INTERNET_ERROR_BASE+32)
 , ERROR_INTERNET_INVALID_PROXY_REQUEST    value(INTERNET_ERROR_BASE+33)
 , ERROR_INTERNET_NEED_UI                  value(INTERNET_ERROR_BASE+34)

 , ERROR_INTERNET_HANDLE_EXISTS            value(INTERNET_ERROR_BASE+36)
 , ERROR_INTERNET_SEC_CERT_DATE_INVALID    value(INTERNET_ERROR_BASE+37)
 , ERROR_INTERNET_SEC_CERT_CN_INVALID      value(INTERNET_ERROR_BASE+38)
 , ERROR_INTERNET_HTTP_TO_HTTPS_ON_REDIR   value(INTERNET_ERROR_BASE+39)
 , ERROR_INTERNET_HTTPS_TO_HTTP_ON_REDIR   value(INTERNET_ERROR_BASE+40)
 , ERROR_INTERNET_MIXED_SECURITY           value(INTERNET_ERROR_BASE+41)
 , ERROR_INTERNET_CHG_POST_IS_NON_SECURE   value(INTERNET_ERROR_BASE+42)
 , ERROR_INTERNET_POST_IS_NON_SECURE       value(INTERNET_ERROR_BASE+43)
 , ERROR_INTERNET_CLIENT_AUTH_CERT_NEEDED  value(INTERNET_ERROR_BASE+44)
 , ERROR_INTERNET_INVALID_CA               value(INTERNET_ERROR_BASE+45)
 , ERROR_INTERNET_CLIENT_AUTH_NOT_SETUP    value(INTERNET_ERROR_BASE+46)
 , ERROR_INTERNET_ASYNC_THREAD_FAILED      value(INTERNET_ERROR_BASE+47)
 , ERROR_INTERNET_REDIRECT_SCHEME_CHANGE   value(INTERNET_ERROR_BASE+48)
 , ERROR_INTERNET_DIALOG_PENDING           value(INTERNET_ERROR_BASE+49)
 , ERROR_INTERNET_RETRY_DIALOG             value(INTERNET_ERROR_BASE+50)
 , ERROR_INTERNET_HTTPS_HTTP_SUBMIT_REDIR  value(INTERNET_ERROR_BASE+52)
 , ERROR_INTERNET_INSERT_CDROM             value(INTERNET_ERROR_BASE+53)
 , ERROR_INTERNET_FORTEZZA_LOGIN_NEEDED    value(INTERNET_ERROR_BASE+54)
 , ERROR_INTERNET_SEC_CERT_ERRORS          value(INTERNET_ERROR_BASE+55)
 , ERROR_INTERNET_SEC_CERT_NO_REV          value(INTERNET_ERROR_BASE+56)
 , ERROR_INTERNET_SEC_CERT_REV_FAILED      value(INTERNET_ERROR_BASE+57)

 /* FTP API errors */
 , ERROR_FTP_TRANSFER_IN_PROGRESS         value(INTERNET_ERROR_BASE+110)
 , ERROR_FTP_DROPPED                      value(INTERNET_ERROR_BASE+111)
 , ERROR_FTP_NO_PASSIVE_MODE              value(INTERNET_ERROR_BASE+112)

 /* gopher API errors */
 , ERROR_GOPHER_PROTOCOL_ERROR            value(INTERNET_ERROR_BASE+130)
 , ERROR_GOPHER_NOT_FILE                  value(INTERNET_ERROR_BASE+131)
 , ERROR_GOPHER_DATA_ERROR                value(INTERNET_ERROR_BASE+132)
 , ERROR_GOPHER_END_OF_DATA               value(INTERNET_ERROR_BASE+133)
 , ERROR_GOPHER_INVALID_LOCATOR           value(INTERNET_ERROR_BASE+134)
 , ERROR_GOPHER_INCORRECT_LOCATOR_TYPE    value(INTERNET_ERROR_BASE+135)
 , ERROR_GOPHER_NOT_GOPHER_PLUS           value(INTERNET_ERROR_BASE+136)
 , ERROR_GOPHER_ATTRIBUTE_NOT_FOUND       value(INTERNET_ERROR_BASE+137)
 , ERROR_GOPHER_UNKNOWN_LOCATOR           value(INTERNET_ERROR_BASE+138)

 /* HTTP API errors */
 , ERROR_HTTP_HEADER_NOT_FOUND            value(INTERNET_ERROR_BASE+150)
 , ERROR_HTTP_DOWNLEVEL_SERVER            value(INTERNET_ERROR_BASE+151)
 , ERROR_HTTP_INVALID_SERVER_RESPONSE     value(INTERNET_ERROR_BASE+152)
 , ERROR_HTTP_INVALID_HEADER              value(INTERNET_ERROR_BASE+153)
 , ERROR_HTTP_INVALID_QUERY_REQUEST       value(INTERNET_ERROR_BASE+154)
 , ERROR_HTTP_HEADER_ALREADY_EXISTS       value(INTERNET_ERROR_BASE+155)
 , ERROR_HTTP_REDIRECT_FAILED             value(INTERNET_ERROR_BASE+156)
 , ERROR_HTTP_NOT_REDIRECTED              value(INTERNET_ERROR_BASE+160)
 , ERROR_HTTP_COOKIE_NEEDS_CONFIRMATION   value(INTERNET_ERROR_BASE+161)
 , ERROR_HTTP_COOKIE_DECLINED             value(INTERNET_ERROR_BASE+162)
 , ERROR_HTTP_REDIRECT_NEEDS_CONFIRMATION value(INTERNET_ERROR_BASE+168)

 /* additional Internet API error codes */
 , ERROR_INTERNET_SECURITY_CHANNEL_ERROR  value(INTERNET_ERROR_BASE+157)
 , ERROR_INTERNET_UNABLE_TO_CACHE_FILE    value(INTERNET_ERROR_BASE+158)
 , ERROR_INTERNET_TCPIP_NOT_INSTALLED     value(INTERNET_ERROR_BASE+159)
 , ERROR_INTERNET_DISCONNECTED            value(INTERNET_ERROR_BASE+163)
 , ERROR_INTERNET_SERVER_UNREACHABLE      value(INTERNET_ERROR_BASE+164)
 , ERROR_INTERNET_PROXY_SERVER_UNREACHABLE
                                          value(INTERNET_ERROR_BASE+165)

 , ERROR_INTERNET_BAD_AUTO_PROXY_SCRIPT   value(INTERNET_ERROR_BASE+166)
 , ERROR_INTERNET_UNABLE_TO_DOWNLOAD_SCRIPT
                                          value(INTERNET_ERROR_BASE+167)
 , ERROR_INTERNET_SEC_INVALID_CERT        value(INTERNET_ERROR_BASE+169)
 , ERROR_INTERNET_SEC_CERT_REVOKED        value(INTERNET_ERROR_BASE+170)

 /* InternetAutodial specific errors */

 , ERROR_INTERNET_FAILED_DUETOSECURITYCHECK
                                          value(INTERNET_ERROR_BASE+171)
 , ERROR_INTERNET_NOT_INITIALIZED         value(INTERNET_ERROR_BASE+172)
 , ERROR_INTERNET_NEED_MSN_SSPI_PKG       value(INTERNET_ERROR_BASE+173)
 , ERROR_INTERNET_LOGIN_FAILURE_DISPLAY_ENTITY_BODY
                                          value(INTERNET_ERROR_BASE+174)
 , INTERNET_ERROR_LAST
     value (ERROR_INTERNET_LOGIN_FAILURE_DISPLAY_ENTITY_BODY)
 ) fixed bin (31);


 /* URLCACHE APIs */
 /* datatype definitions. */

 /* cache entry type flags. */

 dcl NORMAL_CACHE_ENTRY        fixed bin(31) value('00000001'xn),
     STICKY_CACHE_ENTRY        fixed bin(31) value('00000004'xn),
     EDITED_CACHE_ENTRY        fixed bin(31) value('00000008'xn),
     TRACK_OFFLINE_CACHE_ENTRY fixed bin(31) value('00000010'xn),
     TRACK_ONLINE_CACHE_ENTRY  fixed bin(31) value('00000020'xn),
     SPARSE_CACHE_ENTRY        fixed bin(31) value('00010000'xn),
     COOKIE_CACHE_ENTRY        fixed bin(31) value('00100000'xn),
     URLHISTORY_CACHE_ENTRY    fixed bin(31) value('00200000'xn),

     URLCACHE_FIND_DEFAULT_FILTER fixed bin (31)
        value (ior (NORMAL_CACHE_ENTRY,
                    COOKIE_CACHE_ENTRY,
                    URLHISTORY_CACHE_ENTRY,
                    TRACK_OFFLINE_CACHE_ENTRY,
                    TRACK_ONLINE_CACHE_ENTRY,
                    STICKY_CACHE_ENTRY));

 /* INTERNET_CACHE_ENTRY_INFO - */

 define structure
   1 INTERNET_CACHE_ENTRY_INFOA,
     2 dwStructSize      type DWORD,  /* version of cache system. */
     2 lpszSourceUrlName type LPSTR,
         /* embedded pointer to the URL name string. */
     2 lpszLocalFileName type LPSTR,
         /* embedded pointer to the local file name. */
     2 CacheEntryType    type DWORD, /* cache type bit mask. */
     2 dwUseCount        type DWORD,
         /* current users count of the cache entry. */
     2 dwHitRate         type DWORD,
         /* num of times the cache entry was retrieved. */
     2 dwSizeLow         type DWORD, /* low  DWORD of the file size. */
     2 dwSizeHigh        type DWORD, /* high DWORD of the file size. */
     2 LastModifiedTime  type FILETIME,
         /* last modified time of the file in GMT format. */
     2 ExpireTime type FILETIME,
         /* expire time of the file in GMT format */
     2 LastAccessTime type FILETIME,
         /* last accessed time in GMT format */
     2 LastSyncTime type FILETIME,
         /* last time the URL was synchronized with the source */
     2 lpHeaderInfo      type LPBYTE,
         /* embedded pointer to the header info. */
     2 dwHeaderInfoSize  type DWORD, /* size of the above header. */
     2 lpszFileExtension type LPSTR,
         /* File extension used to retrieve the urldata as a file. */
     2 * union,
       3 dwReserved      type DWORD,
       3 dwExemptDelta   type DWORD; /* Exemption delta from last acc */


 define alias LPINTERNET_CACHE_ENTRY_INFOA
        handle INTERNET_CACHE_ENTRY_INFOA;

 define structure
   1 INTERNET_CACHE_ENTRY_INFOW,
     2 dwStructSize      type DWORD, /* version of cache system. */
     2 lpszSourceUrlName type LPSTR,
         /* embedded pointer to the URL name string. */
     2 lpszLocalFileName type LPWSTR,
         /* embedded pointer to the local file name. */
     2 CacheEntryType    type DWORD, /* cache type bit mask. */
     2 dwUseCount        type DWORD,
         /* current users count of the cache entry. */
     2 dwHitRate         type DWORD,
         /* num of times the cache entry was retrieved. */
     2 dwSizeLow         type DWORD, /* low  DWORD of the file size. */
     2 dwSizeHigh        type DWORD, /* high DWORD of the file size. */
     2 LastModifiedTime  type FILETIME,
         /* last modified time of the file in GMT format. */
     2 ExpireTime        type FILETIME,
         /* expire time of the file in GMT format */
     2 LastAccessTime    type FILETIME,
         /* last accessed time in GMT format */
     2 LastSyncTime      type FILETIME,
         /* last time the URL was synchronized with the source */
     2 lpHeaderInfo      type LPBYTE,
         /* embedded pointer to the header info. */
     2 dwHeaderInfoSize  type DWORD, /* size of the above header. */
     2 lpszFileExtension type LPWSTR,
         /* File extension used to retrieve the urldata as a file. */
     2 * union,
       3 dwReserved      type DWORD,
       3 dwExemptDelta   type DWORD; /* Exemption delta from last acc */

 define alias LPINTERNET_CACHE_ENTRY_INFOW
        handle INTERNET_CACHE_ENTRY_INFOW;

 %if UNICODE ^= '' %then %do;
 define alias INTERNET_CACHE_ENTRY_INFO type INTERNET_CACHE_ENTRY_INFOW;
 define alias LPINTERNET_CACHE_ENTRY_INFO
        type LPINTERNET_CACHE_ENTRY_INFOW;
 %end; %else %do;
 define alias INTERNET_CACHE_ENTRY_INFO type INTERNET_CACHE_ENTRY_INFOA;
 define alias LPINTERNET_CACHE_ENTRY_INFO
        type LPINTERNET_CACHE_ENTRY_INFOA;
 %end; /* UNICODE */

 define structure
   1 INTERNET_CACHE_TIMESTAMPS,
     2 ftExpires      type FILETIME,
     2 ftLastModified type FILETIME;
 define alias LPINTERNET_CACHE_TIMESTAMPS
        handle INTERNET_CACHE_TIMESTAMPS;

 /* Cache Group */
 define alias GROUPID type LONGLONG;

 /* Cache Group Flags */

 dcl CACHEGROUP_ATTRIBUTE_GET_ALL      type DWORD value('ffffffff'xu)
   , CACHEGROUP_ATTRIBUTE_BASIC        type DWORD value('00000001'xu)
   , CACHEGROUP_ATTRIBUTE_FLAG         type DWORD value('00000002'xu)
   , CACHEGROUP_ATTRIBUTE_TYPE         type DWORD value('00000004'xu)
   , CACHEGROUP_ATTRIBUTE_QUOTA        type DWORD value('00000008'xu)
   , CACHEGROUP_ATTRIBUTE_GROUPNAME    type DWORD value('00000010'xu)
   , CACHEGROUP_ATTRIBUTE_STORAGE      type DWORD value('00000020'xu)

   , CACHEGROUP_FLAG_NONPURGEABLE      type DWORD value('00000001'xu)
   , CACHEGROUP_FLAG_GIDONLY           type DWORD value('00000004'xu)

   , CACHEGROUP_FLAG_FLUSHURL_ONDELETE type DWORD value('00000002'xu)

   , CACHEGROUP_SEARCH_ALL             type DWORD value('00000000'xu)
   , CACHEGROUP_SEARCH_BYURL           type DWORD value('00000001'xu)

   , CACHEGROUP_TYPE_INVALID           type DWORD value('00000001'xu)
   ;

 /* updatable cache group fields */
 dcl CACHEGROUP_READWRITE_MASK         type DWORD
     value (ior(CACHEGROUP_ATTRIBUTE_TYPE,
                CACHEGROUP_ATTRIBUTE_QUOTA,
                CACHEGROUP_ATTRIBUTE_GROUPNAME,
                CACHEGROUP_ATTRIBUTE_STORAGE));

 /* INTERNET_CACHE_GROUP_INFO */

 dcl  GROUPNAME_MAX_LENGTH       fixed bin(31) value(120);
 dcl  GROUP_OWNER_STORAGE_SIZE   fixed bin(31) value(4);

 define structure
   1 INTERNET_CACHE_GROUP_INFOA,
     2 dwGroupSize  type DWORD,
     2 dwGroupFlags type DWORD,
     2 dwGroupType  type DWORD,
     2 dwDiskUsage  type DWORD,    /* in KB */
     2 dwDiskQuota  type DWORD,    /* in KB */
     2 dwOwnerStorage(GROUP_OWNER_STORAGE_SIZE) type DWORD,
     2 szGroupName char (GROUPNAME_MAX_LENGTH-1) varz;
 define alias LPINTERNET_CACHE_GROUP_INFOA
        handle INTERNET_CACHE_GROUP_INFOA;

 define structure
   1 INTERNET_CACHE_GROUP_INFOW,
     2 dwGroupSize  type DWORD,
     2 dwGroupFlags type DWORD,
     2 dwGroupType  type DWORD,
     2 dwDiskUsage  type DWORD,    /* in KB */
     2 dwDiskQuota  type DWORD,    /* in KB */
     2 dwOwnerStorage(GROUP_OWNER_STORAGE_SIZE) type DWORD,
     2 szGroupName WCHAR (GROUPNAME_MAX_LENGTH-1) varz;
 define alias LPINTERNET_CACHE_GROUP_INFOW
        handle INTERNET_CACHE_GROUP_INFOW;

 %if UNICODE ^= '' %then %do;
 define alias   INTERNET_CACHE_GROUP_INFO
         type   INTERNET_CACHE_GROUP_INFOW;
 define alias LPINTERNET_CACHE_GROUP_INFO
         type LPINTERNET_CACHE_GROUP_INFOW;
 %end; %else %do;
 define alias   INTERNET_CACHE_GROUP_INFO
         type   INTERNET_CACHE_GROUP_INFOA;
 define alias LPINTERNET_CACHE_GROUP_INFO
         type LPINTERNET_CACHE_GROUP_INFOA;
 %end; /* UNICODE */

 /* Cache APIs */

 dcl CreateUrlCacheEntryA
     entry (char (*) varz nonasgn byaddr,
            type DWORD,
            char (*) varz nonasgn byaddr,
            char (*) varz byaddr,
            type DWORD)
     BOOLAPI external ('CreateUrlCacheEntryA');
 dcl CreateUrlCacheEntryW
     entry (WCHAR (*) varz nonasgn byaddr,
            type DWORD,
            WCHAR (*) varz nonasgn byaddr,
            WCHAR(*) varz byaddr,
            type DWORD)
     BOOLAPI external ('CreateUrlCacheEntryW');
 %dcl CreateUrlCacheEntry char ext;
 %if UNICODE ^= '' %then %do;
 %CreateUrlCacheEntry = 'CreateUrlCacheEntryW';
 %end; %else %do;
 %CreateUrlCacheEntry = 'CreateUrlCacheEntryA';
 %end; /* !UNICODE */

 dcl CommitUrlCacheEntryA
     entry (char (*) varz nonasgn byaddr,
            char (*) varz nonasgn byaddr,
            type FILETIME,
            type FILETIME,
            type DWORD,
            char (*) varz nonasgn byaddr, /* may be LPBYTE, pun if so */
            type DWORD,
            char (*) varz nonasgn byaddr,
            type DWORD)
     BOOLAPI external ('CommitUrlCacheEntryA');
 dcl CommitUrlCacheEntryW
     entry (WCHAR (*) varz nonasgn byaddr,
            WCHAR (*) varz nonasgn byaddr,
            type FILETIME,
            type FILETIME,
            type DWORD,
            WCHAR (*) varz nonasgn byaddr,
            type DWORD,
            WCHAR(*) varz byaddr nonasgn,
            type DWORD)
     BOOLAPI external ('CommitUrlCacheEntryW');
 %dcl CommitUrlCacheEntry char ext;
 %if UNICODE ^= '' %then %do;
 %CommitUrlCacheEntry = 'CommitUrlCacheEntryW';
 %end; %else %do;
 %CommitUrlCacheEntry = 'CommitUrlCacheEntryA';
 %end; /* !UNICODE */

 dcl RetrieveUrlCacheEntryFileA
     entry (char (*) varz nonasgn byaddr,
            type INTERNET_CACHE_ENTRY_INFOA byaddr,
            type DWORD byaddr,
            type DWORD)
     BOOLAPI external ('RetrieveUrlCacheEntryFileA');
 dcl RetrieveUrlCacheEntryFileW
     entry (WCHAR (*) varz nonasgn byaddr,
            type INTERNET_CACHE_ENTRY_INFOW byaddr,
            type DWORD byaddr,
            type DWORD)
     BOOLAPI external ('RetrieveUrlCacheEntryFileW');
 %dcl RetrieveUrlCacheEntryFile char ext;
 %if UNICODE ^= '' %then %do;
 %RetrieveUrlCacheEntryFile = 'RetrieveUrlCacheEntryFileW';
 %end; %else %do;
 %RetrieveUrlCacheEntryFile = 'RetrieveUrlCacheEntryFileA';
 %end; /* !UNICODE */

 dcl UnlockUrlCacheEntryFileA
     entry (char (*) varz nonasgn byaddr,
            type DWORD)
     BOOLAPI external ('UnlockUrlCacheEntryFileA');
 dcl UnlockUrlCacheEntryFileW
     entry (WCHAR (*) varz nonasgn byaddr,
            type DWORD)
     BOOLAPI external ('UnlockUrlCacheEntryFileW');
 %dcl UnlockUrlCacheEntryFile char ext;
 %if UNICODE ^= '' %then %do;
   %UnlockUrlCacheEntryFile = 'UnlockUrlCacheEntryFileW';
 %end; %else %if #WINX32_ ^= '' %then %do;
   %UnlockUrlCacheEntryFile = 'UnlockUrlCacheEntryFileA';
 %end; %else %do;
 %deact UnlockUrlCacheEntryFile;
 dcl UnlockUrlCacheEntryFile
     entry (char (*) varz nonasgn byaddr,
            type DWORD)
     BOOLAPI external ('UnlockUrlCacheEntryFile');
 %end;

 dcl RetrieveUrlCacheEntryStreamA
     entry (char (*) varz nonasgn byaddr,
            type INTERNET_CACHE_ENTRY_INFOA byaddr,
            type DWORD byaddr,
            type BOOL,
            type DWORD)
     returns (byvalue optional type HANDLE)
     INTERNETAPI WINAPI external ('RetrieveUrlCacheEntryStreamA');
 dcl RetrieveUrlCacheEntryStreamW
     entry (WCHAR (*) varz nonasgn byaddr,
            type INTERNET_CACHE_ENTRY_INFOW byaddr,
            type DWORD byaddr,
            type BOOL,
            type DWORD)
     returns (byvalue optional type HANDLE)
     INTERNETAPI WINAPI external ('RetrieveUrlCacheEntryStreamW');
 %dcl RetrieveUrlCacheEntryStream char ext;
 %if UNICODE ^= '' %then %do;
 %RetrieveUrlCacheEntryStream = 'RetrieveUrlCacheEntryStreamW';
 %end; %else %do;
 %RetrieveUrlCacheEntryStream = 'RetrieveUrlCacheEntryStreamA';
 %end; /* !UNICODE */

 dcl ReadUrlCacheEntryStream
     entry (type HANDLE,
            type DWORD,
            type LPVOID,
            type DWORD byaddr,
            type DWORD)
     BOOLAPI external ('ReadUrlCacheEntryStream');

 dcl UnlockUrlCacheEntryStream
     entry (type HANDLE,
            type DWORD)
     BOOLAPI external ('UnlockUrlCacheEntryStream');

 dcl GetUrlCacheEntryInfoA
     entry (char (*) varz nonasgn byaddr,
            type INTERNET_CACHE_ENTRY_INFOA byaddr,
            type DWORD byaddr)
     BOOLAPI external ('GetUrlCacheEntryInfoA');
 dcl GetUrlCacheEntryInfoW
     entry (WCHAR (*) varz nonasgn byaddr,
            type INTERNET_CACHE_ENTRY_INFOW byaddr,
            type DWORD byaddr)
     BOOLAPI external ('GetUrlCacheEntryInfoW');
 %dcl GetUrlCacheEntryInfo char ext;
 %if UNICODE ^= '' %then %do;
 %GetUrlCacheEntryInfo = 'GetUrlCacheEntryInfoW';
 %end; %else %do;
 %GetUrlCacheEntryInfo = 'GetUrlCacheEntryInfoA';
 %end; /* !UNICODE */

 dcl FindFirstUrlCacheGroup
     entry (type DWORD,
            type DWORD,
            type LPVOID,
            type DWORD,
            type GROUPID byaddr,
            type LPVOID)
     returns (byvalue optional type HANDLE)
     URLCACHEAPI WINAPI external ('FindFirstUrlCacheGroup');

 dcl FindNextUrlCacheGroup
     entry (type HANDLE,
            type GROUPID byaddr,
            type LPVOID)
     returns (byvalue optional type BOOL)
     URLCACHEAPI WINAPI external ('FindNextUrlCacheGroup');

 dcl GetUrlCacheGroupAttributeA
     entry (type GROUPID,
            type DWORD,
            type DWORD,
            type INTERNET_CACHE_GROUP_INFOA byaddr,
            type DWORD byaddr,
            type LPVOID)
     returns (byvalue optional type BOOL)
     URLCACHEAPI WINAPI external ('GetUrlCacheGroupAttributeA');
 dcl GetUrlCacheGroupAttributeW
     entry (type GROUPID,
            type DWORD,
            type DWORD,
            type INTERNET_CACHE_GROUP_INFOW byaddr,
            type DWORD byaddr,
            type LPVOID)
     returns (byvalue optional type BOOL)
     URLCACHEAPI WINAPI external ('GetUrlCacheGroupAttributeW');
 %dcl GetUrlCacheGroupAttribute char ext;
 %if UNICODE  ^= '' %then %do;
   %GetUrlCacheGroupAttribute = 'GetUrlCacheGroupAttributeW';
 %end; %else %do;
   %GetUrlCacheGroupAttribute = 'GetUrlCacheGroupAttributeA';
 %end; /* !UNICODE */

 dcl SetUrlCacheGroupAttributeA
     entry (type GROUPID,
            type DWORD,
            type DWORD,
            type INTERNET_CACHE_GROUP_INFOA byaddr,
            type LPVOID)
     returns (byvalue optional type BOOL)
     URLCACHEAPI WINAPI external ('SetUrlCacheGroupAttributeA');
 dcl SetUrlCacheGroupAttributeW
     entry (type GROUPID,
            type DWORD,
            type DWORD,
            type INTERNET_CACHE_GROUP_INFOW byaddr,
            type LPVOID)
     returns (byvalue optional type BOOL)
     URLCACHEAPI WINAPI external ('SetUrlCacheGroupAttributeW');
 %dcl SetUrlCacheGroupAttribute char ext;
 %if UNICODE  ^= '' %then %do;
   %SetUrlCacheGroupAttribute = 'SetUrlCacheGroupAttributeW';
 %end; %else %do;
   %SetUrlCacheGroupAttribute = 'SetUrlCacheGroupAttributeA';
 %end; /* !UNICODE */

 dcl GetUrlCacheEntryInfoExA
     entry (char (*) varz nonasgn byaddr,
            type INTERNET_CACHE_ENTRY_INFOA byaddr,
            type DWORD byaddr,
            char(*) varz byaddr optional,  /* must omit */
            type DWORD byaddr optional,    /* must omit */
            type LPVOID,                   /* must pass null */
            type DWORD) /* reserved */
     BOOLAPI external ('GetUrlCacheEntryInfoExA');
 dcl GetUrlCacheEntryInfoExW
     entry (WCHAR (*) varz nonasgn byaddr,
            type INTERNET_CACHE_ENTRY_INFOW byaddr,
            type DWORD byaddr,
            WCHAR(*) varz byaddr optional, /* must omit */
            type DWORD byaddr optional,    /* must omit */
            type LPVOID,                   /* must pass null */
            type DWORD) /* reserved */
     BOOLAPI external ('GetUrlCacheEntryInfoExW');
 %dcl GetUrlCacheEntryInfoEx char ext;
 %if UNICODE ^= '' %then %do;
 %GetUrlCacheEntryInfoEx = 'GetUrlCacheEntryInfoExW';
 %end; %else %do;
 %GetUrlCacheEntryInfoEx = 'GetUrlCacheEntryInfoExA';
 %end; /* !UNICODE */

 dcl CACHE_ENTRY_ATTRIBUTE_FC    type DWORD value('00000004'xu),
     CACHE_ENTRY_HITRATE_FC      type DWORD value('00000010'xu),
     CACHE_ENTRY_MODTIME_FC      type DWORD value('00000040'xu),
     CACHE_ENTRY_EXPTIME_FC      type DWORD value('00000080'xu),
     CACHE_ENTRY_ACCTIME_FC      type DWORD value('00000100'xu),
     CACHE_ENTRY_SYNCTIME_FC     type DWORD value('00000200'xu),
     CACHE_ENTRY_HEADERINFO_FC   type DWORD value('00000400'xu),
     CACHE_ENTRY_EXEMPT_DELTA_FC type DWORD value('00000800'xu);

 dcl SetUrlCacheEntryInfoA
     entry (char (*) varz nonasgn byaddr,
            type INTERNET_CACHE_ENTRY_INFOA byaddr nonasgn,
            type DWORD)
     BOOLAPI external ('SetUrlCacheEntryInfoA');
 dcl SetUrlCacheEntryInfoW
     entry (char (*) varz nonasgn byaddr,
            type INTERNET_CACHE_ENTRY_INFOW byaddr nonasgn,
            type DWORD)
     BOOLAPI external ('SetUrlCacheEntryInfoW');
 %dcl SetUrlCacheEntryInfo char ext;
 %if UNICODE ^= '' %then %do;
 %SetUrlCacheEntryInfo = 'SetUrlCacheEntryInfoW';
 %end; %else %do;
 %SetUrlCacheEntryInfo = 'SetUrlCacheEntryInfoA';
 %end; /* !UNICODE */


 /* Cache Group Functions */

 dcl CreateUrlCacheGroup
     entry (type DWORD,
            type LPVOID)  /* must pass NULL */
     returns (byvalue optional type GROUPID)
     INTERNETAPI WINAPI external ('CreateUrlCacheGroup');

 dcl DeleteUrlCacheGroup
     entry (type GROUPID,
            type DWORD,   /* must pass 0 */
            type LPVOID)  /* must pass NULL */
     BOOLAPI external ('DeleteUrlCacheGroup');

 /* Flags for SetUrlCacheEntryGroup */
 dcl INTERNET_CACHE_GROUP_ADD     type DWORD value (0),
     INTERNET_CACHE_GROUP_REMOVE  type DWORD value (1);

 dcl SetUrlCacheEntryGroupA
     entry (char (*) varz nonasgn byaddr,
            type DWORD,
            type GROUPID,
            type LPBYTE,  /* must pass NULL */
            type DWORD,   /* must pass 0    */
            type LPVOID)  /* must pass NULL */
     BOOLAPI external ('SetUrlCacheEntryGroupA');
 dcl SetUrlCacheEntryGroupW
     entry (WCHAR (*) varz nonasgn byaddr,
            type DWORD,
            type GROUPID,
            type LPBYTE,  /* must pass NULL */
            type DWORD,   /* must pass 0    */
            type LPVOID)  /* must pass NULL */
     BOOLAPI external ('SetUrlCacheEntryGroupW');
 %dcl SetUrlCacheEntryGroup char ext;
 %if UNICODE ^= '' %then %do;
   %SetUrlCacheEntryGroup = 'SetUrlCacheEntryGroupW';
 %end; %else %if #WINX32_ ^= '' %then %do;
   %SetUrlCacheEntryGroup = 'SetUrlCacheEntryGroupA';
 %end; %else %do;
 %deact SetUrlCacheEntryGroup;
 dcl SetUrlCacheEntryGroup
     entry (char (*) varz nonasgn byaddr,
            type DWORD,
            type GROUPID,
            type LPBYTE,  /* must pass NULL */
            type DWORD,   /* must pass 0    */
            type LPVOID)  /* must pass NULL */
     BOOLAPI external ('SetUrlCacheEntryGroup');
 %end;

 dcl FindFirstUrlCacheEntryExA
     entry (char (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD,
            type GROUPID,
            type INTERNET_CACHE_ENTRY_INFOA byaddr,
            type DWORD byaddr,
            type LPVOID,                      /* must pass NULL */
            type DWORD byaddr optional,       /* must omit */
            type LPVOID)                      /* must pass NULL */
     returns (byvalue optional type HANDLE)
     INTERNETAPI WINAPI external ('FindFirstUrlCacheEntryExA');

 dcl FindFirstUrlCacheEntryExW
     entry (char (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD,
            type GROUPID,
            type INTERNET_CACHE_ENTRY_INFOW byaddr,
            type DWORD byaddr,
            type LPVOID,                      /* must pass NULL */
            type DWORD byaddr optional,       /* must omit */
            type LPVOID)                      /* must pass NULL */
     returns (byvalue optional type HANDLE)
     INTERNETAPI WINAPI external ('FindFirstUrlCacheEntryExW');

 %dcl FindFirstUrlCacheEntryEx char ext;
 %if UNICODE ^= '' %then %do;
 %FindFirstUrlCacheEntryEx = 'FindFirstUrlCacheEntryExW';
 %end; %else %do;
 %FindFirstUrlCacheEntryEx = 'FindFirstUrlCacheEntryExA';
 %end; /* !UNICODE */

 dcl FindNextUrlCacheEntryExA
     entry (type HANDLE,
            type INTERNET_CACHE_ENTRY_INFOA byaddr,
            type DWORD byaddr,
            type LPVOID,                      /* must pass NULL */
            type DWORD byaddr optional,       /* must omit */
            type LPVOID)                       /* must pass NULL */
     BOOLAPI external ('FindNextUrlCacheEntryExA');

 dcl FindNextUrlCacheEntryExW
     entry (type HANDLE,
            type INTERNET_CACHE_ENTRY_INFOW byaddr,
            type DWORD byaddr,
            type LPVOID,                       /* must pass NULL */
            type DWORD byaddr optional,       /* must omit */
            type LPVOID)                       /* must pass NULL */
     BOOLAPI external ('FindNextUrlCacheEntryExW');

 %dcl FindNextUrlCacheEntryEx char ext;
 %if UNICODE ^= '' %then %do;
 %FindNextUrlCacheEntryEx = 'FindNextUrlCacheEntryExW';
 %end; %else %do;
 %FindNextUrlCacheEntryEx = 'FindNextUrlCacheEntryExA';
 %end; /* !UNICODE */

 dcl FindFirstUrlCacheEntryA
     entry (char (*) varz nonasgn byaddr,
            type INTERNET_CACHE_ENTRY_INFOA byaddr,
            type DWORD byaddr)
     returns (byvalue optional type HANDLE)
     INTERNETAPI WINAPI external ('FindFirstUrlCacheEntryA');
 dcl FindFirstUrlCacheEntryW
     entry (char (*) varz nonasgn byaddr,
            type INTERNET_CACHE_ENTRY_INFOW byaddr,
            type DWORD byaddr)
     returns (byvalue optional type HANDLE)
     INTERNETAPI WINAPI external ('FindFirstUrlCacheEntryW');
 %dcl FindFirstUrlCacheEntry char ext;
 %if UNICODE ^= '' %then %do;
 %FindFirstUrlCacheEntry = 'FindFirstUrlCacheEntryW';
 %end; %else %do;
 %FindFirstUrlCacheEntry = 'FindFirstUrlCacheEntryA';
 %end; /* !UNICODE */

 dcl FindNextUrlCacheEntryA
     entry (type HANDLE,
            type INTERNET_CACHE_ENTRY_INFOA byaddr,
            type DWORD byaddr)
     BOOLAPI external ('FindNextUrlCacheEntryA');
 dcl FindNextUrlCacheEntryW
     entry (type HANDLE,
            type INTERNET_CACHE_ENTRY_INFOW byaddr,
            type DWORD byaddr)
     BOOLAPI external ('FindNextUrlCacheEntryW');
 %dcl FindNextUrlCacheEntry char ext;
 %if UNICODE ^= '' %then %do;
 %FindNextUrlCacheEntry = 'FindNextUrlCacheEntryW';
 %end; %else %do;
 %FindNextUrlCacheEntry = 'FindNextUrlCacheEntryA';
 %end; /* !UNICODE */

 dcl FindCloseUrlCache
     entry (type HANDLE)
     BOOLAPI external ('FindCloseUrlCache');

 dcl DeleteUrlCacheEntryA
     entry (char (*) varz nonasgn byaddr)
     BOOLAPI external ('DeleteUrlCacheEntryA');
 dcl DeleteUrlCacheEntryW
     entry (WCHAR (*) varz nonasgn byaddr)
     BOOLAPI external ('DeleteUrlCacheEntryW');
 %dcl DeleteUrlCacheEntry char ext;
 %if UNICODE ^= '' %then %do;
   %DeleteUrlCacheEntry = 'DeleteUrlCacheEntryW';
 %end; %else %if #WINX32_ ^= '' %then %do;
   %DeleteUrlCacheEntry = 'DeleteUrlCacheEntryA';
 %end; %else %do;
 %deact DeleteUrlCacheEntry;
 dcl DeleteUrlCacheEntry
     entry (char (*) varz nonasgn byaddr)
     BOOLAPI external ('DeleteUrlCacheEntry');
 %end;

 /* Autodial APIs */

 dcl InternetDialA
     entry (type HWND,
            char (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD byaddr,
            type DWORD)
     returns (byvalue optional type DWORD)
     INTERNETAPI WINAPI external ('InternetDialA');
 dcl InternetDialW
     entry (type HWND,
            WCHAR (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD byaddr,
            type DWORD)
     returns (byvalue optional type DWORD)
     INTERNETAPI WINAPI external ('InternetDialW');
 %dcl InternetDial char ext;
 %if UNICODE ^= '' %then %do;
   %InternetDial = 'InternetDialW';
 %end; %else %if #WINX32_ ^= '' %then %do;
   %InternetDial = 'InternetDialA';
 %end; %else %do;
 %deact InternetDial;
 dcl InternetDial
     entry (type HWND,
            char (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD byaddr,
            type DWORD)
     returns (byvalue optional type DWORD)
     INTERNETAPI WINAPI external ('InternetDial');
 %end;

 /* Flags for InternetDial - must not conflict with InternetAutodial  */
 /*                          flags as they are valid here also.       */
 dcl INTERNET_DIAL_FORCE_PROMPT type DWORD value('2000'xu),
     INTERNET_DIAL_SHOW_OFFLINE type DWORD value('4000'xu),
     INTERNET_DIAL_UNATTENDED   type DWORD value('8000'xu);

 dcl InternetHangUp
     entry (type DWORD,
            type DWORD)
     returns (byvalue optional type DWORD)
     INTERNETAPI WINAPI external ('InternetHangUp');

 dcl INTERENT_GOONLINE_REFRESH type DWORD value ('00000001'xu),
     INTERENT_GOONLINE_MASK    type DWORD value ('00000001'xu);

 dcl InternetGoOnlineA
     entry (char (*) varz nonasgn byaddr,
            type HWND,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('InternetGoOnlineA');
 dcl InternetGoOnlineW
     entry (WCHAR (*) varz nonasgn byaddr,
            type HWND,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('InternetGoOnlineW');
 %dcl InternetGoOnline char ext;
 %if UNICODE ^= '' %then %do;
   %InternetGoOnline = 'InternetGoOnlineW';
 %end; %else %if #WINX32_ ^= '' %then %do;
   %InternetGoOnline = 'InternetGoOnlineA';
 %end; %else %do;
 %deact InternetGoOnline;
 dcl InternetGoOnline
     entry (char (*) varz nonasgn byaddr,
            type HWND,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('InternetGoOnline');
 %end;

 dcl InternetAutodial
     entry (type DWORD,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('InternetAutodial');

 /* Flags for InternetAutodial */
 dcl INTERNET_AUTODIAL_FORCE_ONLINE        type DWORD value (1),
     INTERNET_AUTODIAL_FORCE_UNATTENDED    type DWORD value (2),
     INTERNET_AUTODIAL_FAILIFSECURITYCHECK type DWORD value (4),
     INTERNET_AUTODIAL_FLAGS_MASK          type DWORD
          value (ior (INTERNET_AUTODIAL_FORCE_ONLINE,
                      INTERNET_AUTODIAL_FORCE_UNATTENDED,
                      INTERNET_AUTODIAL_FAILIFSECURITYCHECK));

 dcl InternetAutodialHangup
     entry (type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('InternetAutodialHangup');

 dcl InternetGetConnectedState
     entry (type DWORD byaddr,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('InternetGetConnectedState');

 dcl InternetGetConnectedStateExA
     entry (type DWORD byaddr,
            char(*) varz byaddr,
            type DWORD,
            type DWORD)
     returns (byvalue optional type BOOL)
    INTERNETAPI WINAPI external ('InternetGetConnectedStateExA');
 dcl InternetGetConnectedStateExW
     entry (type DWORD byaddr,
            WCHAR(*) varz byaddr,
            type DWORD,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('InternetGetConnectedStateExW');
 %dcl InternetGetConnectedStateEx char ext;
 %if UNICODE ^= '' %then %do;
   %InternetGetConnectedStateEx = 'InternetGetConnectedStateExW';
 %end; %else %if #WINX32_ ^= '' %then %do;
   %InternetGetConnectedStateEx = 'InternetGetConnectedStateExA';
 %end; %else %do;
 %deact InternetGetConnectedStateEx;
 dcl InternetGetConnectedStateEx
     entry (type DWORD byaddr,
            char(*) varz byaddr,
            type DWORD,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('InternetGetConnectedStateEx');
 %end;

 /* Flags for InternetGetConnectedState and Ex */
 dcl INTERNET_CONNECTION_MODEM      type DWORD value('01'xu),
     INTERNET_CONNECTION_LAN        type DWORD value('02'xu),
     INTERNET_CONNECTION_PROXY      type DWORD value('04'xu),
     INTERNET_CONNECTION_MODEM_BUSY type DWORD value('08'xu),
                                    /* no longer used */
     INTERNET_RAS_INSTALLED         type DWORD value('10'xu),
     INTERNET_CONNECTION_OFFLINE    type DWORD value('20'xu),
     INTERNET_CONNECTION_CONFIGURED type DWORD value('40'xu);

 /* Custom dial handler functions */

 /* Custom dial handler prototype */

 define alias PFN_DIAL_HANDLER
     entry (type HWND,
            pointer,    /* to char (*) varz nonasgn byaddr */
            type DWORD,
            type DWORD byaddr)
     returns (byvalue optional type DWORD)
     limited CALLBACK;

 /* Flags for custom dial handler */
 dcl INTERNET_CUSTOMDIAL_CONNECT             type DWORD value (0),
     INTERNET_CUSTOMDIAL_UNATTENDED          type DWORD value (1),
     INTERNET_CUSTOMDIAL_DISCONNECT          type DWORD value (2),
     INTERNET_CUSTOMDIAL_SHOWOFFLINE         type DWORD value (4);

 /* Custom dial handler supported functionality flags */
 dcl INTERNET_CUSTOMDIAL_SAFE_FOR_UNATTENDED type DWORD value (1),
     INTERNET_CUSTOMDIAL_WILL_SUPPLY_STATE   type DWORD value (2),
     INTERNET_CUSTOMDIAL_CAN_HANGUP          type DWORD value (4);

 dcl InternetSetDialStateA
     entry (char (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('InternetSetDialStateA');
 dcl InternetSetDialStateW
     entry (WCHAR (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('InternetSetDialStateW');
 %dcl InternetSetDialState char ext;
 %if UNICODE ^= '' %then %do;
   %InternetSetDialState = 'InternetSetDialStateW';
 %end; %else %if #WINX32_ ^= '' %then %do;
   %InternetSetDialState = 'InternetSetDialStateA';
 %end; %else %do;
 %deact InternetSetDialState;
 dcl InternetSetDialState
     entry (char (*) varz nonasgn byaddr,
            type DWORD,
            type DWORD)
     returns (byvalue optional type BOOL)
     INTERNETAPI WINAPI external ('InternetSetDialState');
 %end;

 /* States for InternetSetDialState */
 dcl INTERNET_DIALSTATE_DISCONNECTED type DWORD value (1);

 %end; /* !defined(_WININET_) */
